/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite;

import java.util.ArrayList;
import net.geoadel.bibcite.GUI.BibCiteGUI;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import net.geoadel.bibcite.DataFields.SessionData;
//import net.geoadel.bibcite.Database.DefaultDataset;
import net.geoadel.bibcite.Database.DefaultDataset;
import net.geoadel.bibcite.Database.db4o;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.DatabaseObjects.User;
import net.geoadel.bibcite.DatabaseObjects.dbObject;

/**
 * bibcite is a program to store and organize quotations and bibliography entries together with related files.
 * It features user specific privacy settings together with an encryption scheme for private bibliographies and or quotations.
 * It exports into bibTex compatible formats that seem to work well with JabRef
 * and import the same format
 *
 * This particular class starts the GUI thread and calls the login / register user interface
 * 
 * @DocumentationStatus finished
 * @GPLStatus clear
 * @author Georg Adelmann
 * @url http://www.geoadel.net
 */
public class BibCite {

    public static void main(final String[] args) {
        try {
            // Set System L&F
            UIManager.setLookAndFeel(
                    UIManager.getSystemLookAndFeelClassName());
        } catch (UnsupportedLookAndFeelException e) {
            // handle exception
        } catch (ClassNotFoundException e) {
            // handle exception
        } catch (InstantiationException e) {
            // handle exception
        } catch (IllegalAccessException e) {
            // handle exception
        }


        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI, this only starts the login view
        // (if login credentials are correct, the login view then spawns the main window
        // giving the passed username)
        javax.swing.SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                BibCiteGUI gc = new BibCiteGUI();

                boolean debug = false;

                if (debug) {
                    // initialize database access
                    dbModel.createDBAccess();

                    // set testuser
                    User user = new User("testUser", "abc123!", "English");
                    SessionData.setUser(user);

                    dbModel.db_reset(SessionData.getPathDatabase());

                    dbModel.storeUser(user);
                    ArrayList<dbObject> objects = DefaultDataset.getDefaultDataset();
                    for (dbObject object : objects) {
                        dbModel.storeObject(object);
                    }

                    gc.createGUI(gc);
                    System.out.println("Creating gui for user " + user.getName());
                } else {
                    // If custom database path is given as argument, call createDBAccess() after
                    // having set SessionData.setPathDatabase() to ensure, that db_access()
                    // in db4o.java uses the custom path - otherwise the standard path in SessionData
                    // is used.
                    if (args != null) {
                        if (args.length == 0) {
                            // initialize database access
                            dbModel.createDBAccess();
                            gc.createLoginGUI(gc);
                        }
                        if (args.length == 1) {
                            SessionData.setPathBase(args[0]);
                            // initialize database access
                            dbModel.createDBAccess();
                            gc.createGUI(gc);
                        }
                        if (args.length == 2) {
                            // initialize database access
                            dbModel.createDBAccess();
                            User user = dbModel.loginUser(args[0], args[1]);
                            if (user != null) {
                                SessionData.setUser(user);
                                gc.createGUI(gc);
                            } else {
                                System.out.println("Error: Username or password were wrong...");
                            }
                        }
                        if (args.length == 3) {
                            SessionData.setPathBase(args[0]);
                            // initialize database access
                            dbModel.createDBAccess();
                            
                            User user = dbModel.loginUser(args[1], args[2]);
                            if (user != null) {
                                SessionData.setUser(user);
                                gc.createGUI(gc);
                            } else {
                                System.out.println("Error: Username or password were wrong...");
                            }
                        }
                    } else {
                        // open login gui
                        gc.createLoginGUI(gc);
                    }
                }
            }
        });
    }
}
