/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package net.geoadel.bibcite.DataFields;

import java.io.File;
import java.security.PrivateKey;
import java.util.HashMap;
import java.util.UUID;
import javax.crypto.SecretKey;
import net.geoadel.bibcite.DatabaseObjects.User;

/**
 *  This class provides the current users login password, which cannot be kept in the User-class, because then it would be stored in the database unencrypted, which it never should be
 * @author Georg Adelmann
 */
public class SessionData {
    static private String passwordLoginUser;
    static private SecretKey secretKeyUser;

    static private PrivateKey privateDecryptionKeyUser;

    static private HashMap<UUID, String> hmDecryptedPrivateObjectPasswords = new HashMap<UUID, String>();

    static private User user;

    public static String pathBase = ".bibciteTest123"+File.separatorChar+"data"+File.separatorChar;
    
     /**
     * The password used for encrypting database objects which are not set private
     * This password is hardcoded and needs to be the same for all bibcite programs that
     * want to exchange encrypted databases, since otherwise the informations cannot be
     * decrypted
     */
     static private String passwordProgramPermanent = "Fd6U6T#xd5oQh5R,9CWE7AY9P+L29x9xB0eI$-";

     /**
      * Sets the user login password - should be done at the start of each program session
      * to make sure, this password in here is the correct one for the user (since it is used
      * to encrypt / decrypt the user permanent password)
      * @param password     unencrypted user login password
      */
    public static void setLoginPassword(String password){
        SessionData.passwordLoginUser = password;
    }

    /**
     * Get the user login password (unencrypted) - used only for encrypting / decrypting the user permanent password
     * @return  unencrypted user login password
     */
    public static String getPasswordUserLogin(){
        // System.out.println("Returning login password "+loginPassword);
        return SessionData.passwordLoginUser;
    }

    /**
     *  Returns the permanent program password (used for encrypting / decrypting all non-private
     * objects)
     * @return  permanent program password
     */
    public static String getPasswordProgramPermanent(){
        return passwordProgramPermanent;
    }

    /**
     * Set the current user
     * @param user
     */
    public static void setUser(User user){
        SessionData.user = user;
        SessionData.user.removeKeysUnused();
    }

    /**
     *
     * @return  the current user
     */
    public static User getUser() {
        return SessionData.user;
    }
    
    public static void setPathBase(String pathBase){
        SessionData.pathBase = pathBase;
    }
    
    public static String getPathDatabase(){
        return pathBase+"database"+File.separatorChar;
    }
    
    public static String getPathFiles(){
        return pathBase+"files"+File.separatorChar;
    }

    /**
     *
     * @param uuid
     * @return
     */
    public static String getObjectFieldsPassword(UUID uuid){
        // System.out.println("User "+SessionData.user.getName()+" has passwords for: "+uuid+" -> "+hmDecryptedPrivateObjectPasswords.containsKey(uuid));

        /*
         * If hmDecryptedPrivateObjectPasswords does not
         * contain the decrypted password for the object with UUID uuid
         * decrypt the password from the user and put the decrypted String
         * in hmDecryptedPrivateObjectPasswords, this is much faster than decrypting the
         * password anew all the time.
         */
        if(!hmDecryptedPrivateObjectPasswords.containsKey(uuid)){
            // try to load password from user
            String password = SessionData.user.getAccessPrivatePassword(uuid);
            // System.out.println("Using password for object "+uuid+": "+password);
            // put decrypted password in the HashMap, so it only is decrypted once
            hmDecryptedPrivateObjectPasswords.put(uuid, password);
            return password;
        } else {
            // System.out.println("returning the password for object "+uuid+": "+hmDecryptedPrivateObjectPasswords.get(uuid));
            return hmDecryptedPrivateObjectPasswords.get(uuid);
        }
    }
}
