/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.Database;

import java.util.ArrayList;
import java.util.UUID;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.DatabaseObjects.Note;
import net.geoadel.bibcite.DatabaseObjects.Person;
import net.geoadel.bibcite.DatabaseObjects.Quotation;
import net.geoadel.bibcite.DatabaseObjects.User;
import net.geoadel.bibcite.DatabaseObjects.dbFile;
import net.geoadel.bibcite.DatabaseObjects.dbObject;
import net.geoadel.bibcite.GUI.BibCiteGUI;
import net.geoadel.bibcite.GUI.ObjectEditors.BibliographyEditor;
import net.geoadel.bibcite.GUI.ObjectEditors.NoteEditor;
import net.geoadel.bibcite.GUI.ObjectEditors.PersonEditor;
import net.geoadel.bibcite.GUI.ObjectListPanels.FileEditorPanel;
import net.geoadel.bibcite.GUI.ObjectListPanels.QuotationEditorPanel;
import net.geoadel.bibcite.GUI.TreeObjectSelectorGUI;
import net.geoadel.bibcite.Tools.FileOperations;
import net.geoadel.bibcite.Tools.ObjectTools;

/**
 * This class delegates informations from one class to another using their event driven interfaces
 * The events are listened to in the respective classes but then the respective calls are made through this class
 * to other classes
 *
 * @author Georg Adelmann
 */
public class dbModel {

    public static ArrayList<dbObject> getRevisionsOf(UUID uuid) {
        return new db4o().getRevisionsByUUID(SessionData.getUser(), uuid);
    }

    public static ArrayList<User> getUsers() {
        return new db4o().getUsers();
    }

    public static void createDBAccess() {
        new db4o().db_access(SessionData.getPathDatabase());
    }

    public static ArrayList<Bibliography> getBibliographiesNewestVersions() {
        return new db4o().getBibliographiesNewestVersions(SessionData.getUser());
    }

    public static ArrayList<Quotation> getQuotationsNewestVersions() {
        return new db4o().getQuotationsNewestVersions(SessionData.getUser());
    }

    public static ArrayList<Note> getNotesNewestVersions() {
        return new db4o().getNotesNewestVersions(SessionData.getUser());
    }

    public static ArrayList<Person> getPersonsNewestVersions() {
        return new db4o().getPersonsNewestVersions(SessionData.getUser());
    }

    public static ArrayList<dbObject> getObjectsAll() {
        return new db4o().getObjects(SessionData.getUser());
    }

    public static ArrayList<String> getObjectKeys() {
        return new db4o().getKeys(SessionData.getUser());
    }

    public static ArrayList<Quotation> getBibliographyQuotes(Bibliography bib) {
        return new db4o().getQuotationsByBibUUID(bib.getUUID(), SessionData.getUser());
    }

    public static ArrayList<dbFile> getFilesNewestVersions() {
        return new db4o().getFilesNewestVersions(SessionData.getUser());
    }

    public static Object getFileByUUID(UUID uuid) {
        return new db4o().getObjectsByUUID(uuid, SessionData.getUser());
    }

    public static ArrayList<dbFile> getFilesRelatedToUUID(UUID sourceUUID) {
        return new db4o().getFilesBySourceUUID(sourceUUID, SessionData.getUser());
    }

    public static ArrayList<Note> getNotesRelatedToUUID(UUID sourceUUID) {
        return new db4o().getNotesRelatedToUUID(sourceUUID, SessionData.getUser());
    }

    public static User getUserByUUID(UUID userUUID) {
        return new db4o().getUserByUUID(userUUID);
    }

    public static ArrayList<String> getUsernames() {
        return new db4o().getUserNameList();
    }

    public static ArrayList<String> getAuthorKeyList() {
        return new db4o().getAuthorKeyList(SessionData.getUser());
    }

    public static User loginUser(String name, String password) {
        return new db4o().userLogin(name, password);
    }

    public static User registerUser(String name, String password, String languagePreferred) {
        return new db4o().userRegister(name, password, languagePreferred);
    }

    public static void storeUser(User user) {
        new db4o().storeUser(user);
    }

    public static Object[] returnSearchResults(ArrayList<String> alSearch) {
        return new db4o().returnSearchResults(alSearch, SessionData.getUser());
    }

    public static void db_reset(String pathDatabase) {
        new db4o().db_reset(pathDatabase);
    }

    public void EditorEvents() {
    }

    static public void createBibEditor() {
        TreeObjectSelectorGUI.showButtonsBib();
        BibCiteGUI.itemNewBib.doClick();
    }

    static public void createPersonEditor() {
        TreeObjectSelectorGUI.showButtonsPerson();
        BibCiteGUI.setEditorArea(new PersonEditor(new Person()));
    }

    static public void createNoteEditor() {
        TreeObjectSelectorGUI.showButtonsNote();
        BibCiteGUI.setEditorArea(new NoteEditor(new Note()));
    }

    static public void createQuoteEditor(QuotationEditorPanel quotePanel) {
        quotePanel.addQuotation();
    }

    static public void loadBibRevision(UUID uuid, Long revision) {
        ArrayList<Bibliography> bib = ObjectTools.dbObjectToBib(new db4o().getObjectByDateUUID(uuid, revision, SessionData.getUser()));
        if (!bib.isEmpty() && bib.get(0) != null) {
            BibCiteGUI.setEditorArea(new BibliographyEditor(bib.get(0)));
        }
    }

    /**
     * Creates file related to database object with UUID sourceUUID and then returns the new dbFile, usually used by FileEditorPanel to create new file
     * @param sourceUUID    UUID of related object
     * @return  newly created dbFile
     */
    public static dbFile createFile(UUID sourceUUID) {
        dbFile file = FileOperations.createFileFromTemplate();
        /**
         * Add relation to sourceUUID to relation ArrayList of file
         */
        file.addRelatedObject(sourceUUID);
        return file;
    }

    /**
     * import file and return it; useful for adding new files to database objects, as used by FileEditorPanel.bCreateNewFileActionPerformed()
     * @param sourceUUID
     * @return
     */
    public static dbFile importFile(UUID sourceUUID) {
        dbFile file = FileOperations.importFiles(BibCiteGUI.frame, sourceUUID);
        return file;
    }

    /**
     * import file and display it as new single file editor. Useful for importing files not directly related to other
     * database elements
     * @return
     */
    public static dbFile importFile() {
        dbFile dbfile = FileOperations.importFiles(BibCiteGUI.frame, null);
        if (dbfile != null) {
            FileEditorPanel panel = new FileEditorPanel(dbfile);
            BibCiteGUI.setEditorArea(panel);
        }
        return dbfile;
    }

    public static void updateTree() {
        db4o db = new db4o();
        new BibCiteGUI().updateTree(db.getBibliographiesNewestVersions(SessionData.getUser()), db.getQuotationsNewestVersions(SessionData.getUser()), db.getNotesByUserNewest(SessionData.getUser()), db.getFilesByUserNewest(SessionData.getUser()), db.getPersonsNewest(SessionData.getUser()));
        // BibCiteGUI.clearEditorArea();
    }

    /**
     * Creates new file and then sets editor area to single file editor for new file
     */
    public static void createFile() {
        dbFile file = FileOperations.createFileFromTemplate();
        if (file != null) {
            BibCiteGUI.setEditorArea(new FileEditorPanel(file));
        }
    }

    public static void deleteObject(UUID uuid) {
        db4o db = new db4o();
        db.eraseObject(uuid, SessionData.getUser());
        new BibCiteGUI().updateTree(db.getBibliographiesNewestVersions(SessionData.getUser()), db.getQuotationsNewestVersions(SessionData.getUser()), db.getNotesByUserNewest(SessionData.getUser()), db.getFilesByUserNewest(SessionData.getUser()), db.getPersonsNewest(SessionData.getUser()));
    }

    public static void deleteObject(dbObject object) {
        deleteObject(object.getUUID());
        if (object instanceof dbFile) {
            FileOperations.eraseFile(object.getUUID(), (dbFile) object);
        }
    }

    public static void storeObject(dbObject object) {
        db4o db = new db4o();
        db.storeObject(object, SessionData.getUser());
    }
}
