/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.DatabaseObjects;

import java.util.ArrayList;
import java.util.UUID;
import net.geoadel.bibcite.DataFields.SessionData;

/**
 * Bibliography is used as an object to store all bibliography informations, according to and extending the bibtex standard
 *
 * @GPLStatus   clear
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class Bibliography extends dbObject {

    private String icon = "Book16.png";
    // the datafields are provided
    private String address, bibAbstract, booktitle, crossref, ePrint, howpublished;
    private String identifier, institution, journal, language, license, organization, pages, publisher, school, series;
    
    public String getIconFileName() {
        return icon;
    }
    
    public String getIdentifier() {
        return decrypt(identifier);
    }

    public void setIdentifier(String identifier) {
        this.identifier = encrypt(identifier);
    }

    private String subtitle, title, type, typeTechReport, url;
    private String urlDate = Long.toString(0L);
    private String chapter, edition, month, number, volume, year;
    
    // authorName(Lastname, Firstname);
    private ArrayList<String> authors = new ArrayList<String>();
    private ArrayList<String> editors = new ArrayList<String>();

    /**
     * returns the language set for this bibliography
     * 
     * @return  language of the bibliography (or more specific of the language set by the user for the bibliography)
     */
    public String getLanguage() {
        return decrypt(language);
    }

    public void setLanguage(String language) {
        this.language = encrypt(language);
    }

    public String getSubtitle() {
        return decrypt(subtitle);
    }

    public void setSubtitle(String subtitle) {
        this.subtitle = encrypt(subtitle);
    }

    public long getUrlDate() {
        return Long.valueOf(decrypt(urlDate));
    }

    public void setUrlDate(Long urlDate) {
        this.urlDate = encrypt(String.valueOf(urlDate));
    }

    public String getEPrint() {
        return decrypt(ePrint);
    }

    public void setEPrint(String ePrint) {
        this.ePrint = encrypt(ePrint);
    }

    public String getTypeTechReport() {
        return decrypt(typeTechReport);
    }

    public void setTypeTechReport(String typeTechReport) {
        this.typeTechReport = encrypt(typeTechReport);
    }

    public String getUrl() {
        return decrypt(url);
    }

    public void setUrl(String url) {
        this.url = encrypt(url);
    }

    public void setAddress(String address) {
        this.address = encrypt(address);
    }

    public void setBibAbstract(String bibAbstract) {
        this.bibAbstract = encrypt(bibAbstract);
    }

    public void setBooktitle(String booktitle) {
        this.booktitle = encrypt(booktitle);
    }

    public void setChapter(String chapter) {
        this.chapter = encrypt(chapter);
    }

    public void setCrossref(String crossref) {
        this.crossref = encrypt(crossref);
    }

    public void setEdition(String edition) {
        this.edition = encrypt(edition);
    }

    public void setHowpublished(String howpublished) {
        this.howpublished = encrypt(howpublished);
    }

    public void setInstitution(String institution) {
        this.institution = encrypt(institution);
    }

    public void setJournal(String journal) {
        this.journal = encrypt(journal);
    }

    public void setMonth(String month) {
        this.month = encrypt(month);
    }

    public void setNumber(String number) {
        this.number = encrypt(number);
    }

    public void setOrganization(String organization) {
        this.organization = encrypt(organization);
    }

    public void setPages(String pages) {
        this.pages = encrypt(pages);
    }

    public void setPublisher(String publisher) {
        this.publisher = encrypt(publisher);
    }

    public void setSchool(String school) {
        this.school = encrypt(school);
    }

    public void setSeries(String series) {
        this.series = encrypt(series);
    }

    public void setTitle(String title) {
        this.title = encrypt(title);
    }

    public void setType(String type) {
        this.type = encrypt(type);
    }

    public void setVolume(String volume) {
        this.volume = encrypt(volume);
    }

    public void setYear(String year) {
        this.year = encrypt(year);
    }

    public String getAddress() {
        return decrypt(address);
    }

    public String getBibAbstract() {
        return decrypt(bibAbstract);
    }

    public String getBooktitle() {
        return decrypt(booktitle);
    }

    public String getChapter() {
        return decrypt(chapter);
    }

    public String getCrossref() {
        return decrypt(crossref);
    }

    public String getEdition() {
        return decrypt(edition);
    }

    public String getHowpublished() {
        return decrypt(howpublished);
    }

    public String getInstitution() {
        return decrypt(institution);
    }

    public String getJournal() {
        return decrypt(journal);
    }

    public String getMonth() {
        return decrypt(month);
    }

    public String getNumber() {
        return decrypt(number);
    }

    public String getOrganization() {
        return decrypt(organization);
    }

    public String getPages() {
        return decrypt(pages);
    }

    public String getPublisher() {
        return decrypt(publisher);
    }

    public String getSchool() {
        return decrypt(school);
    }

    public String getSeries() {
        return decrypt(series);
    }

    public String getTitle() {
        return decrypt(title);
    }

    public String getType() {
        return decrypt(type);
    }

    public String getVolume() {
        return decrypt(volume);
    }

    public String getYear() {
        return decrypt(year);
    }

    /**
     *
     * @return  returns the author names prepared for displaying them in the JTextPane, since they expect to be
     *          lastname, firstname +NEWLINE
     *          nextAuthor lastname, firstname ...
     */
    public String getAuthorNameAsText() {
        String authorText = "";
        String NEWLINE = System.getProperty("line.separator");
        int i = 0;
        for(String author: getAuthors()){
            if(i==0){
                authorText = author;
            } else {
                authorText = authorText + NEWLINE + author;
            }
            i++;
        }
        // System.out.println("authors: "+authorText);
        return authorText;
    }

    public void setAuthors(ArrayList<String> authors) {
        this.authors.clear();
        for(String author : authors){
            this.authors.add(encrypt(author));
        }
    }

    public ArrayList<String> getAuthors() {
        ArrayList<String> decryptedAuthors = new ArrayList<String>();
        for(String author : this.authors){
            decryptedAuthors.add(decrypt(author));
        }
        return decryptedAuthors;
    }

    public void setEditors(ArrayList<String> editors) {
        this.editors.clear();
        for(String editor : editors){
            this.editors.add(encrypt(editor));
        }
    }

    public ArrayList<String> getEditors() {
        ArrayList<String> decryptedEditors = new ArrayList<String>();
        for(String editor : this.editors){
            decryptedEditors.add(decrypt(editor));
        }
        return decryptedEditors;
    }

    /**
     *
     * @return      returns all editors as one string divided by the newline-character
     * used for the editorName textarea in the BibliographyEditor
     */
    public String getEditorNameAsText() {
        String editorText = "";
        String NEWLINE = System.getProperty("line.separator");
        int i = 0;
        for(String editor: getEditors()){
            if(i==0){
                editorText = editor;
            } else {
                editorText = editorText + NEWLINE + editor;
            }
            i++;
        }
        return editorText;
    }
    
    public Bibliography(){
        isPrivate = false;
        setUUID();
        setDateEdited();
        setUserUUID(SessionData.getUser().getUUID());
        setLanguage(SessionData.getUser().getPreferredLanguage());
        setTitle("");
        setName("");
        setYear("2000");
    }

    /**
     * Instantiate a bibliography object by setting all datafields
     * @param uuid
     * @param bibAbstract
     * @param address
     * @param booktitle
     * @param crossref
     * @param howpublished
     * @param institution
     * @param journal
     * @param key
     * @param note
     * @param organization
     * @param publisher
     * @param school
     * @param series
     * @param title
     * @param type
     * @param chapter
     * @param edition
     * @param month
     * @param year
     * @param number
     * @param pages
     * @param volume
     * @param authorName
     * @param editorName
     * @param license
     * @param url
     * @param typeTechReport
     * @param ePrint
     * @param urlDate
     * @param user
     * @param subtitle
     * @param dateEdited
     * @param isPrivate
     * @param alAllowedAccessGroups
     * @param language
     * @param identifier
     */
    public Bibliography(UUID uuid, String bibAbstract, String address, String booktitle, String crossref, String howpublished, String institution, String journal, String key, String organization, String publisher, String school, String series, String title, String type, String chapter, String edition, String month, String year, String number, String pages, String volume, ArrayList<String> authors, ArrayList<String> editors, String license, String url, String typeTechReport, String ePrint, long urlDate, User user, String subtitle, long dateEdited, boolean isPrivate, String language, String identifier, ArrayList<String> alTags) {
        // System.out.println("Bibliography was instantiated by user:" + user);

        this.isPrivate = isPrivate; // needs to be set first, to make suNotere, it is set correctly, so it can correctly be decided, if encryption is taking place

        setUUID(uuid);

        setBibAbstract(bibAbstract);
        setAddress(address);
        setBooktitle(booktitle);
        setCrossref(crossref);
        setHowpublished(howpublished);
        setInstitution(institution);
        setJournal(journal);
        setKey(key);
        setOrganization(organization);
        setPages(pages);
        setPublisher(publisher);
        setSchool(school);
        setSeries(series);
        setTitle(title);
        setType(type);

        setChapter(chapter);
        setEdition(edition);
        setMonth(month);
        setNumber(number);
        setVolume(volume);
        setYear(year);
        setIdentifier(identifier);

        setAuthors(authors);
        setEditors(editors);

        setLicense(license);

        setUrl(url);
        setEPrint(ePrint);
        setTypeTechReport(typeTechReport);
        setUrlDate(urlDate);

        setUserUUID(user.getUUID());

        setSubtitle(subtitle);

        setDateEdited(dateEdited);

        setLanguage(language);

        addTags(alTags);

        setName(title); // set the dbObject field name to value title - this is used in the TreeView GUI to retrieve and display the String value for one of the JTrees
    }

    public String getLicense() {
        if (license == null) {
            return "";
        }
        return decrypt(license);
    }

    private void setLicense(String license) {
        this.license = encrypt(license);
    }


    @Override
    public String toString() {
        return getTitle();
    }

    /**
     * String representation of most fields - used by db4o.java for searching for String parameters
     * @return  String containing fields relevant for search algorithm
     */
    public String getFieldSummary() {
        StringBuilder result = new StringBuilder();
        String NL = System.getProperty("line.separator");
        if(!this.getKey().isEmpty()){
            result.append(this.getKey()).append(NL);
        }
        result.append(this.getUUID()).append(NL);
        if(!this.getIdentifier().isEmpty()){
            result.append(this.getIdentifier()).append(NL);
        }
        if(!this.getTitle().isEmpty()){
            result.append(this.getTitle()).append(NL);
        }
        if(!this.getYear().isEmpty()){
            result.append(this.getYear()).append(NL);
        }
        if(!this.getAuthorNameAsText().isEmpty()){
            result.append(getAuthorNameAsText()).append(NL);
        }
        if(!this.getEditorNameAsText().isEmpty()){
            result.append(getEditorNameAsText()).append(NL);
        }
        if(!this.getSubtitle().isEmpty()){
            result.append(this.getSubtitle()).append(NL);
        }
        if(!this.getType().isEmpty()){
            result.append(this.getType()).append(NL);
        }
        if(!this.getBibAbstract().isEmpty()){
            result.append(this.getBibAbstract()).append(NL);
        }
        if(!this.getLicense().isEmpty()){
            result.append(this.getLicense()).append(NL);
        }
        if(!this.getOrganization().isEmpty()){
            result.append(this.getOrganization()).append(NL);
        }
        if(!this.getInstitution().isEmpty()){
            result.append(this.getInstitution()).append(NL);
        }
        if(!this.getAddress().isEmpty()){
            result.append(this.getAddress()).append(NL);
        }
        if(!this.getJournal().isEmpty()){
            result.append(this.getJournal()).append(NL);
        }
        if(!this.getPages().isEmpty()){
            result.append(this.getPages()).append(NL);
        }
        return result.toString();
    }

    public int compareTo(Bibliography b) {
        int titleDifference = this.getTitle().replaceAll("\\W", "").compareToIgnoreCase(b.getTitle().replaceAll("\\W", ""));
        return titleDifference == 0 ? titleDifference : Integer.valueOf(this.getYear()) - Integer.valueOf(b.getYear()); 
    }
}
