/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package net.geoadel.bibcite.DatabaseObjects;

import java.util.ArrayList;
import java.util.UUID;
import net.geoadel.bibcite.DataFields.SessionData;

/**
 * database object for storing notes
 * 
 * @GPLStatus clean
 * @DocumentationStatus NOT finished
 * @author Georg Adelmann
 */
public class Note extends dbObject {
    private String icon = "Note16.png";
    String note;

    public Note(UUID noteUUID, ArrayList<UUID> alRelatedObjects, String name, String key, String text, boolean isPrivate, ArrayList<String> alTags, Long dateEdited) {
        this.isPrivate = isPrivate;

        setNoteText(text);

        setRelatedObjects(alRelatedObjects);

        this.setName(name);
        this.setKey(key);
        setUUID(noteUUID);
        this.isPrivate = isPrivate;
        setUserUUID( SessionData.getUser().getUUID());
        this.addTags(alTags);
        setDateEdited(dateEdited);
    }

    public Note(){
        this.isPrivate = false;
        this.setName("");
        this.setNoteText("");
        setUserUUID( SessionData.getUser().getUUID());
        setDateEdited();
        this.addRelatedObject(SessionData.getUser().getUUID());
        setUUID();
    }
        
    public String getIconFileName() {
        return icon;
    }
    

    public void setNoteText(String note){
        this.note = encrypt(note);
    }

    public String getNoteText(){
        return decrypt(note);
    }

    @Override
    public String toString(){
        if(getNoteText().length()<=20){
            return getNoteText();
        } else {
            return getNoteText().substring(0,19);
        }
    }

     /**
     * String representation of most fields - used by db4o.java for searching for String parameters
     * @return  String containing fields relevant for search algorithm
     */
    public String getFieldSummary(){
        StringBuilder result = new StringBuilder();
        String NL = System.getProperty("line.separator");
        result.append(NL);
        result.append("UUID: "+getUUID()).append(NL);
        result.append("Name: "+getName()).append(NL);
        result.append("Note: "+getNoteText()).append(NL);
        return result.toString();
    }

    public int compareTo(Note n) {
        int retValue = getNoteText().replaceAll("\\W", "").compareToIgnoreCase(n.getNoteText().replaceAll("\\W", ""));
        return retValue;
    }
}
