/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */


package net.geoadel.bibcite.DatabaseObjects;

import java.util.ArrayList;
import java.util.UUID;
import net.geoadel.bibcite.DataFields.SessionData;

/**
 * Person is used to store person data in the database
 * 
 * @GPLStatus   clear
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class Person extends dbObject {
    private String icon = "Person16.png";
    // data fields of objects
    private String name;
    private boolean isAlive;
    private String dateBirth;
    private String dateDeath;
    private String addressPostal;
    private String addressMail;
    private ArrayList<String> languageSkills;
    private String placeBirth;
    private String placeDeath;
    private String placeCurrent;
    private String description;
    private String mainJob;
    private ArrayList<UUID> family = new ArrayList<UUID>();
    private ArrayList<UUID> friends = new ArrayList<UUID>();

    /*
     * Only to be used by classes extending person (e.g. User) that take care of setting User
     */
    public Person(){
        setUUID();
        setName("");
    }

    /**
     * instantiate Person to be filled with data later
     */
    public Person(User user){
        setUUID();
        setName("");
        setUserUUID(user.getUUID());
        setDateEdited();
    }

    public Person(UUID uuid, String name, boolean isAlive, Long dateBirth, Long dateDeath, String addressPostal, String addressMail, String description, Boolean isPrivate, User user, String key){
        this.isPrivate = isPrivate;
        setUUID(uuid);
        setName(name);
        this.isAlive = isAlive;
        setDateBirth(dateBirth.toString());
        setDateDeath(dateDeath.toString());
        setAddressMail(addressMail);
        setAddressPostal(addressPostal);
        setDescription(description);
        setUserUUID(user.getUUID());
        setKey(key);
        setDateEdited();
    }

    /**
     * Instantiate person with name, uuid is set to random (used, if only the name is known, usually not stored)
     * @param name  last name of person
     */
    public Person(String name){
        setUUID();
        this.isPrivate = false;
        this.setName(name);
        setDateEdited();
        setKey("");
    }
    
    public String getIconFileName() {
        return icon;
    }

    /**
     * @return the birthDate
     */
    public Long getDateBirth() {
        if(dateBirth != null){
            return Long.parseLong(decrypt(dateBirth.toString()));
        }
        return 0L;
    }

    /**
     * @param birthDate the birthDate to set
     */
    public void setBirthDate(Long birthDate) {
        this.dateBirth = birthDate.toString();
    }

    /**
     * @return the deathDate
     */
    public Long getDateDeath() {
        if(dateDeath != null){
            return Long.parseLong(decrypt(dateDeath.toString()));
        }
        return 0L;
    }

    /**
     * @param deathDate the deathDate to set
     */
    public void setDeathDate(Long deathDate) {
        this.dateDeath = deathDate.toString();
    }

    /**
     * @return the addressPostal
     */
    public String getAddressPostal() {
        return decrypt(addressPostal);
    }

    /**
     * @param addressPostal the addressPostal to set
     */
    public void setAddressPostal(String addressPostal) {
        this.addressPostal = encrypt(addressPostal);
    }

    /**
     * @return the addressMail
     */
    public String getAddressMail() {
        return decrypt(addressMail);
    }

    /**
     * @param addressMail the addressMail to set
     */
    public void setAddressMail(String addressMail) {
        this.addressMail = encrypt(addressMail);
    }

    /**
     * @return the languageSkills
     */
    public ArrayList<String> getLanguageSkills() {
        return languageSkills;
    }

    /**
     * @param languageSkills the languageSkills to set
     */
    public void setLanguageSkills(ArrayList<String> languageSkills) {
        this.languageSkills = languageSkills;
    }

    /**
     * @return the birthPlace
     */
    public String getPlaceBirth() {
        return decrypt(placeBirth);
    }

    /**
     * @param birthPlace the birthPlace to set
     */
    public void setPlaceBirth(String birthPlace) {
        this.placeBirth = encrypt(birthPlace);
    }

        /**
     * @return the birthPlace
     */
    public String getPlaceDeath() {
        return decrypt(placeDeath);
    }

    /**
     * @param birthPlace the birthPlace to set
     */
    public void setPlaceDeath(String placeDeath) {
        this.placeDeath = encrypt(placeDeath);
    }

    /**
     * @return the currentPlace
     */
    public String getPlaceCurrent() {
        return decrypt(placeCurrent);
    }

    /**
     * @param currentPlace the currentPlace to set
     */
    public void setCurrentPlace(String currentPlace) {
        this.placeCurrent = encrypt(currentPlace);
    }

    /**
     * @return the description
     */
    public String getDescription() {
        return decrypt(description);
    }

    /**
     * @param description the description to set
     */
    public void setDescription(String description) {
        this.description = encrypt(description);
    }

    /**
     * @return the mainJob
     */
    public String getMainJob() {
        return decrypt(mainJob);
    }

    /**
     * @param mainJob the mainJob to set
     */
    public void setMainJob(String mainJob) {
        this.mainJob = encrypt(mainJob);
    }

    /**
     * @return the family
     */
    public ArrayList<UUID> getFamily() {
        return family;
    }

     /**
     * @return the isAlive status
     */
    public boolean isAlive() {
        return isAlive;
    }

     /**
     * @return the isAlive status
     */
    public void setIsAlive(boolean isAlive) {
        this.isAlive = isAlive;
    }

    /**
     * @param family the family to set
     */
    public void setFamily(ArrayList<UUID> family) {
        this.family = family;
    }

    /**
     * @return the friends
     */
    public ArrayList<UUID> getFriends() {
        return friends;
    }

    /**
     * @param friends the friends to set
     */
    public void setFriends(ArrayList<UUID> friends) {
        this.friends = friends;
    }

    public boolean isFamily(UUID uuid){
        return family.contains(uuid);
    }

    public boolean isFriend(UUID uuid){
        return friends.contains(uuid);
    }

    public int compareTo(Person p) {
        int retValue = getName().replaceAll("\\W", "").compareToIgnoreCase(p.getName().replaceAll("\\W", ""));
        return retValue;
    }

    private void setDateBirth(String dateBirth) {
        this.dateBirth = encrypt(dateBirth);
    }

    private void setDateDeath(String dateDeath) {
        this.dateDeath = encrypt(dateDeath);
    }

    @Override
    public String toString(){
        return getKey();
    }

     /**
     * String representation of most fields - used by db4o.java for searching for String parameters
     * @return  String containing fields relevant for search algorithm
     */
    public String getFieldSummary(){
        StringBuilder result = new StringBuilder();
        String NL = System.getProperty("line.separator");
        result.append(NL);
        result.append(getName()).append(NL);
        result.append(getDateBirth()).append(NL);
        result.append(getDateDeath()).append(NL);
        result.append(getPlaceCurrent()).append(NL);
        result.append(getPlaceBirth()).append(NL);
        result.append(getPlaceDeath()).append(NL);
        result.append(getDescription()).append(NL);
        result.append(getAddressMail()).append(NL);
        result.append(getAddressPostal()).append(NL);
        result.append(getLanguageSkills()).append(NL);
        result.append(getMainJob()).append(NL);
        return result.toString();
    }
}