/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.DatabaseObjects;

import java.util.ArrayList;
import java.util.UUID;
import net.geoadel.bibcite.DataFields.SessionData;

/**
 * Quotation object to handle data and store them
 * @GPLStatus   clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class Quotation extends dbObject {
    // quotation specific data fields
    private String icon = "Quote16.png";
    
    String quote;
    String gist;
    String pageStart;
    String pageEnd;
    UUID sourceUUID; // the uuid of the source of the quote (journal, book, etc. bibliography entry)

    // empty quotation to be filled with data later
    public Quotation() {
        setUUID();
        setDateEdited();
        setName("");
    }

    /**
     *
     * @param uuid
     * @param sourceUUID
     * @param alTags
     * @param quote
     * @param gist
     * @param pageStart
     * @param pageEnd
     * @param dateEdited
     * @param user
     * @param isPrivate
     */
    public Quotation(UUID uuid, UUID sourceUUID, ArrayList<String> alTags, String key, String quote, String gist, String pageStart, String pageEnd, Long dateEdited, boolean isPrivate) {
        this.isPrivate = isPrivate;

        setUUID(uuid);
        if(sourceUUID != null){
            addRelatedObject(sourceUUID);
        }

        this.addTags(alTags);

        this.setQuote(quote);
        this.gist = encrypt(gist);

        this.setPageEnd(pageEnd);
        this.setPageStart(pageStart);

        this.setKey(key);

        setDateEdited(dateEdited);

        setUserUUID(SessionData.getUser().getUUID());

        this.setName(pageStart + "-" + pageEnd + ": " + gist);
    }
//

    public Quotation(UUID uuid, boolean isPrivate, ArrayList<UUID> alRelatedObjects, ArrayList<String> alTags, String key, String quote, String gist, String pageStart, String pageEnd, Long dateEdited) {
        this.isPrivate = isPrivate;
        setUUID(uuid);
        setRelatedObjects(alRelatedObjects);

        this.addTags(alTags);

        this.setQuote(quote);
        this.setGist(gist);

        this.setPageEnd(pageEnd);
        this.setPageStart(pageStart);

        this.setKey(key);

        setDateEdited(dateEdited);

        setUserUUID(SessionData.getUser().getUUID());

        this.setName(pageStart + "-" + pageEnd + ": " + gist);
    }
    
    public String getIconFileName() {
        return icon;
    }
    
    public String getPageEnd() {
        return decrypt(pageEnd);
    }

    public void setPageEnd(String pageEnd) {
        this.pageEnd = encrypt(pageEnd);
    }

    public String getPageStart() {
        return decrypt(pageStart);
    }

    public void setPageStart(String pageStart) {
        this.pageStart = encrypt(pageStart);
    }

    public String getGist() {
        return decrypt(gist);
    }

    public void setQuoteGist(String quoteCore) {
        this.gist = encrypt(quoteCore);
    }

    /**
     * String representation of most fields - used by db4o.java for searching for String parameters
     * @return  String containing fields relevant for search algorithm
     */
    public String getFieldSummary() {
        StringBuilder result = new StringBuilder();
        String NL = System.getProperty("line.separator");
        result.append(NL);
        result.append(getKey()).append(NL);
        result.append(getGist()).append(NL);
        result.append(getQuote()).append(NL);
        result.append(getPageStart()).append(NL);
        result.append(getPageEnd());
        return result.toString();
    }

    public int compareTo(Quotation q) {
        // System.out.println("q1-pagestart-length " + q.getPageStart().length() + " q2-pagestart-length " + this.getPageStart().length());

        // if both quotes have page start entries
        if ((q.getPageStart().length() >= 1) && this.getPageStart().length() >= 1) {
            int diffPageStart = Integer.valueOf(this.getPageStart()) - Integer.valueOf(q.getPageStart());
            return diffPageStart == 0 ? Integer.valueOf(this.getPageEnd()) - Integer.valueOf(q.getPageEnd()) : diffPageStart;
        }
        if (q.getPageStart().length() == 0 && this.getPageStart().length() > 0) {
            return 1;
        }
        if (this.getPageStart().length() == 0 && q.getPageStart().length() > 0) {
            return -1;
        }
        return 0;
    }

    public void setQuote(String quote) {
        this.quote = encrypt(quote);
    }

    public String getQuote() {
        return decrypt(quote);
    }

    public void setGist(String gist) {
        this.gist = encrypt(gist);
    }

    @Override
    public String toString() {
        if (getGist().length() > 3) {
            return getGist();
        } else if (getQuote().length() > 20) {
            return getQuote().substring(0, 19);
        } else {
            return getQuote();
        }
    }
}
