/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package net.geoadel.bibcite.DocType;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * DocumentType for eight numbers of digits (more could create problems with integer max value)
 * 
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
    public class DocumentTypeEightDigit extends PlainDocument {

     /**
     * Provides for any JTextPane, JTextArea, JTextField etc. that implements this DocumentType
     * a restriction to only accept digits, eight maximum (higher may risk to go beyond integer max value)
     * @param offset    offset within the document, after which the change occurred
     * @param text  text to insert into the containing UI element
     * @param attributes    the attributes to be associated with the inserted text
     * @throws BadLocationException
     */
        @Override
        public void insertString(int offset, String text, AttributeSet attributes)
                throws BadLocationException {
            String newText = "";
            for (int i = 0; i < text.length(); i++) {
                if (Character.isDigit(text.charAt(i)) & (getLength() + text.length()) <= 8) {
                    newText = newText.concat(Character.toString(text.charAt(i)));
                }
            }
            super.insertString(offset, newText, attributes);
        }
    }