/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package net.geoadel.bibcite.DocType;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * DocumentType for four digits (e.g. year number)
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class DocumentTypeFourDigit extends PlainDocument {

    /**
     * Provides for any JTextPane, JTextArea, JTextField etc. that implements this DocumentType
     * a restriction to only accept digits, maximum four by checking the String characters to be inserted
     * and deciding which to take and which to ignore based on comparison with allowed, disallowed characters or
     * other rules (like total string length)
     * @param offset    offset within the document, after which the change occurred
     * @param text  text to insert into the containing UI element
     * @param attributes    the attributes to be associated with the inserted text
     * @throws BadLocationException
     */
    @Override
    public void insertString(int offset, String text, AttributeSet attributes)
            throws BadLocationException {
        String newText = ""; // the new text to replace the text that was to be inserted before checking if it adhered to the document restrictions
        for (int i = 0; i < text.length(); i++) {
            // if character to be inserted is digit and length is smalle ror equal four, then insert it into the string newText
            if (Character.isDigit(text.charAt(i)) & (getLength() + text.length()) <= 4) {
                newText = newText.concat(Character.toString(text.charAt(i)));
            }
        }
        // insert newText string at offset within txt UI element
        super.insertString(offset, newText, attributes);
    }
}
