/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * AccessManagerGUI.java
 *
 * Created on 23-Nov-2010, 15:44:45
 */
package net.geoadel.bibcite.GUI.AttributeEditors;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.io.File;
import java.util.ArrayList;
import java.util.UUID;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.Database.db4o;
import net.geoadel.bibcite.DatabaseObjects.User;
import net.geoadel.bibcite.DatabaseObjects.dbObject;
import net.geoadel.bibcite.Tools.FileOperations;

/**
 *
 * @author Georg Adelmann
 */
public class AccessManagerGUI extends javax.swing.JPanel {

    private ArrayList<User> alUsers;
    private UUID objectUUID;

    /** Creates new form AccessManagerGUI */
    public AccessManagerGUI(dbObject object) {
        // load the data into the user table
        this.alUsers = dbModel.getUsers();

        objectUUID = object.getUUID();
        // System.out.println("!!! AccessManagerGUI - object uuid "+objectUUID);

        initComponents();

        DefaultListModel modelUntrusted = new DefaultListModel();
        DefaultListModel modelTrusted = new DefaultListModel();
        jList1.setModel(modelUntrusted);
        jList2.setModel(modelTrusted);

        // System.out.println("User list contains " + alUsers.size() + " entries");
        for (User user : alUsers) {
            /**
             * The current user should not be contained in the list
             */
            if (user.getUUID() != SessionData.getUser().getUUID()) {
                // System.out.println("Checking user " + user.getName());
                if (user.hasAccessToPrivateObject(objectUUID)) {
                    modelTrusted.addElement(user);
                } else {
                    modelUntrusted.addElement(user);
                }
            }
        }

        CustomCellRenderer renderer = new CustomCellRenderer();
        jList1.setCellRenderer(renderer);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList();
        jScrollPane2 = new javax.swing.JScrollPane();
        jList2 = new javax.swing.JList();
        jLabel3 = new javax.swing.JLabel();

        jLabel1.setText("jLabel1");

        jPanel1.setPreferredSize(new java.awt.Dimension(462, 300));

        jLabel2.setText("Users without access");

        jList1.setToolTipText("Move the user names with a double click from side to the other");
        jList1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jList1MouseClicked(evt);
            }
        });
        jScrollPane1.setViewportView(jList1);

        jList2.setModel(new javax.swing.DefaultListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        jList2.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jList2MouseClicked(evt);
            }
        });
        jScrollPane2.setViewportView(jList2);

        jLabel3.setText("Allowed private access");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 207, Short.MAX_VALUE))
                .addGap(18, 18, 18)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel3)
                        .addGap(69, 69, 69))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 215, Short.MAX_VALUE)
                        .addContainerGap())))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel2)
                    .addComponent(jLabel3))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 313, Short.MAX_VALUE)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 313, Short.MAX_VALUE))
                .addContainerGap())
        );

        jScrollPane3.setViewportView(jPanel1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 466, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 363, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    private void jList1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList1MouseClicked
        if (evt.getClickCount() > 1) {
            if (jList1.getSelectedIndex() != -1) {
                // System.out.println("Selected index " + ((User) jList1.getSelectedValue()).getName());
                Object selectedObject = jList1.getSelectedValue();
                ((DefaultListModel)jList2.getModel()).addElement(jList1.getSelectedValue());
                ((DefaultListModel)jList1.getModel()).removeElement(jList1.getSelectedValue());
                jList1.updateUI();
                jList2.updateUI();
                // System.out.println("Granted private access to user "+((User)selectedObject).getName() + " to object "+objectUUID);
                /**
                 * Give password to user
                 */
                ((User)selectedObject).grantAccessToPrivateObject(objectUUID, SessionData.getObjectFieldsPassword(objectUUID));
            }
        }
    }//GEN-LAST:event_jList1MouseClicked

    private void jList2MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList2MouseClicked
        if (evt.getClickCount() > 1) {
            if (jList2.getSelectedIndex() != -1) {
                // System.out.println("Selected index " + ((User) jList2.getSelectedValue()).getName());
                ((DefaultListModel)jList1.getModel()).addElement(jList2.getSelectedValue());
                ((DefaultListModel)jList2.getModel()).removeElement(jList2.getSelectedValue());
                jList1.updateUI();
                jList2.updateUI();
            }
        }
    }//GEN-LAST:event_jList2MouseClicked

    class CustomCellRenderer extends JLabel implements ListCellRenderer {

        private ImageIcon image;
        private User user;

        public CustomCellRenderer() {
            setOpaque(true);

        }

        public Component getListCellRendererComponent(
                JList list, Object user, int index,
                boolean isSelected, boolean cellHasFocus) {
            // Display the text for this item
            setText(((User) user).getName());
            File icon = FileOperations.getImagePreviewMedium(((User) user).getUUID());
            // Set the correct image
            if (icon.exists()) {
                image = new ImageIcon(icon.getPath());
            } else {
                image = new ImageIcon("images/person.png");
            }


            // Draw the correct colors and font
            if (isSelected) {
                // Set the color and font for a selected item
                setBackground(Color.red);
                setForeground(Color.white);
                setFont(new Font("Roman", Font.BOLD, 24));
            } else {
                // Set the color and font for an unselected item
                setBackground(Color.white);
                setForeground(Color.black);
                setFont(new Font("Roman", Font.PLAIN, 12));
            }

            return this;
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JList jList1;
    private javax.swing.JList jList2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    // End of variables declaration//GEN-END:variables
}
