
/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * RelationManager.java
 *
 * Created on 25.12.2010, 22:46:05
 */

package net.geoadel.bibcite.GUI.AttributeEditors;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.DatabaseObjects.Note;
import net.geoadel.bibcite.DatabaseObjects.Person;
import net.geoadel.bibcite.DatabaseObjects.Quotation;
import net.geoadel.bibcite.DatabaseObjects.dbFile;
import net.geoadel.bibcite.DatabaseObjects.dbObject;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.Tools.FileOperations;

/**
 *
 * @author Georg Adelmann
 */
public class RelationManager extends javax.swing.JPanel {

    private ArrayList<dbObject> alObjects;
    private dbObject objectCurrent;

    /** Creates new form RelationManager */
    public RelationManager(dbObject objectCurrent) {
        this.objectCurrent = objectCurrent;
        
        initComponents();

        alObjects = dbModel.getObjectsAll();

        DefaultListModel modelRelated = new DefaultListModel();
        DefaultListModel modelUnrelated = new DefaultListModel();
        lNotRelated.setModel(modelUnrelated);
        lRelated.setModel(modelRelated);

        // initiate the custom nodePanel (which enables preview icons next to dbObject entries, if they
        // have a file image (only png for now) with their uuid (can be set via GUI by calling 'setIcon')
        ListCellRenderer renderer = new CustomCellRenderer();

        // tell the JTrees to use the custom renderer
        lNotRelated.setCellRenderer(renderer);
        lRelated.setCellRenderer(renderer);

        for(dbObject object : alObjects){
            // System.out.println(objectCurrent);
            if(objectCurrent.getRelatedObjects().contains(object.getUUID())){
                modelRelated.addElement(object);
            } else {
                modelUnrelated.addElement(object);
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        lNotRelated = new javax.swing.JList();
        jScrollPane2 = new javax.swing.JScrollPane();
        lRelated = new javax.swing.JList();
        jLabel3 = new javax.swing.JLabel();

        jPanel1.setPreferredSize(new java.awt.Dimension(462, 300));

        jLabel2.setText("Elements not related");

        lNotRelated.setToolTipText("Move the user names with a double click from side to the other");
        lNotRelated.setLayoutOrientation(javax.swing.JList.VERTICAL_WRAP);
        lNotRelated.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                lNotRelatedMouseClicked(evt);
            }
        });
        jScrollPane1.setViewportView(lNotRelated);

        lRelated.setModel(new javax.swing.DefaultListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        lRelated.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                lRelatedMouseClicked(evt);
            }
        });
        jScrollPane2.setViewportView(lRelated);

        jLabel3.setText("Related elements");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(12, 12, 12)
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 225, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jLabel2)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel3)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 234, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(jLabel2))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE))
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 495, Short.MAX_VALUE)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 495, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 324, Short.MAX_VALUE)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 324, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void lNotRelatedMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_lNotRelatedMouseClicked
        if ((evt.getClickCount() > 1) && (lNotRelated.getSelectedIndex() != -1)) {
            // System.out.println("Selected index " + ((User) jList1.getSelectedValue()).getName());
            Object selectedObject = lNotRelated.getSelectedValue();
            ((DefaultListModel)lRelated.getModel()).addElement(lNotRelated.getSelectedValue());
            ((DefaultListModel)lNotRelated.getModel()).removeElement(lNotRelated.getSelectedValue());
            lNotRelated.updateUI();
            lRelated.updateUI();
            System.out.println("This object "+ objectCurrent.getName() +" has added " + ((dbObject)selectedObject).getName() + " to its list of related objects.");
            /**
             * Add selected object to relatedObject-arraylist of current object
             * and store the current object
             */
            objectCurrent.addRelatedObject(((dbObject)selectedObject).getUUID());
            dbModel.storeObject((dbObject)selectedObject);
        }
}//GEN-LAST:event_lNotRelatedMouseClicked

    private void lRelatedMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_lRelatedMouseClicked
        if (evt.getClickCount() > 1 && lRelated.getSelectedIndex() != -1) {

            Object selectedObject = lRelated.getSelectedValue();

            // System.out.println("Selected index " + ((User) jList2.getSelectedValue()).getName());
            ((DefaultListModel)lNotRelated.getModel()).addElement(lRelated.getSelectedValue());
            ((DefaultListModel)lRelated.getModel()).removeElement(lRelated.getSelectedValue());
            lNotRelated.updateUI();
            lRelated.updateUI();

            System.out.println("This object "+ objectCurrent.getName() +" has removed " + ((dbObject)selectedObject).getName() + " from its list of related objects.");
            /**
             * Add selected object to relatedObject-arraylist of current object
             * and store the current object
             */
            objectCurrent.removeRelatedObject(((dbObject)selectedObject).getUUID());
            dbModel.storeObject((dbObject)selectedObject);
        }
}//GEN-LAST:event_lRelatedMouseClicked


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JList lNotRelated;
    private javax.swing.JList lRelated;
    // End of variables declaration//GEN-END:variables


      /**
     * This inner class provides a custom cell nodePanel, that has support for
     * background colors based on selection state and icons
     */
    class CustomCellRenderer implements ListCellRenderer {

        JLabel name = new JLabel("");
        JPanel nodePanel = new JPanel();

      
        DefaultListCellRenderer defaultRenderer = new DefaultListCellRenderer();
        Color backgroundSelectionColor;
        Color backgroundNonSelectionColor;

        public CustomCellRenderer(){
            name.setForeground(Color.BLACK);
            // Align all entries to the left.
            nodePanel.setLayout(new BorderLayout());
            nodePanel.add(name, BorderLayout.WEST);
        }

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            Component returnValue = null;
            name.setText("<html><font size=3>" + value.toString());
             // System.out.println("^^^  Path of standard image: "+f.getAbsolutePath());
            try {
                name.setIcon(((dbObject)value).getIcon());
                // if object has uuid, try to get its specific preview icon for displaying (if one exists)
                if (((dbObject)value).getUUID() != null 
                      && FileOperations.hasPreviewSmall(((dbObject)value).getUUID())) {
                         name.setIcon(new ImageIcon(FileOperations.loadPreviewImageSmall(((dbObject)value).getUUID())));
                }
            nodePanel.setEnabled(list.isEnabled());
            return nodePanel;
            } catch(ClassCastException e) { 
                //No dbObject object has been passed
                return defaultRenderer.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            }
        }
    }
}
