/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.GUI;

import net.geoadel.bibcite.GUI.ObjectListPanels.FileEditorPanel;
import net.geoadel.bibcite.GUI.ObjectEditors.QuotationEditor;
import net.geoadel.bibcite.GUI.ObjectEditors.BibliographyEditor;
import net.geoadel.bibcite.GUI.ObjectEditors.PersonEditor;
import java.awt.BorderLayout;
import net.geoadel.bibcite.Database.db4o;
import net.geoadel.bibcite.Events.TreeViewSelectionEvent;
import net.geoadel.bibcite.Events.TreeViewSelectionListener;
import net.geoadel.bibcite.ImportExport.*;
import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.DatabaseObjects.Quotation;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.SwingWorker;
import net.geoadel.bibcite.Cryptography.EncryptDecrypt;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.DatabaseObjects.Note;
import net.geoadel.bibcite.DatabaseObjects.Person;
import net.geoadel.bibcite.DatabaseObjects.User;
import net.geoadel.bibcite.DatabaseObjects.dbFile;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.GUI.ObjectEditors.FileEditor;
import net.geoadel.bibcite.GUI.ObjectEditors.NoteEditor;
import net.geoadel.bibcite.GUI.Preferences.PreferencesEditor;
import net.geoadel.bibcite.Tools.FileOperations;

/**
 * Provides basic GUI connecting and combining the whole GUI interface for bibcite (including BibliographyEditor, TreeObjectSelectorGUI and LoginRegisterGUI
 * 
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class BibCiteGUI {
    static ProgressNotificator progress = new ProgressNotificator();
    static ArrayList<Bibliography> alBibsNewest = new ArrayList<Bibliography>();
    static ArrayList<Quotation> alQuotesNewest = new ArrayList<Quotation>();
    static ArrayList<Note> alNotesNewest = new ArrayList<Note>();
    static ArrayList<dbFile> alFilesNewest = new ArrayList<dbFile>();
    static ArrayList<Person> alPersonsNewest = new ArrayList<Person>();
    static String username;
    public static User user = SessionData.getUser();// make the user accessible in case it cannot be given to the method in need (e.g. the 'toString()' override method in dbFile where otherwise the encrypted filename would be visible in the list, would have to create a custom list model to adapt)
    static public JFrame frame = new JFrame("bibcite");
    static JSplitPane splitPane;
    static public BibCiteGUI gc;
    static TreeObjectSelectorGUI tree;
    static public JMenuItem itemNewBib;
    static public JMenuItem itemNewPerson;

    public static void clearEditorArea() {
        splitPane.setRightComponent(new JPanel());
    }
    EncryptDecrypt crypt = new EncryptDecrypt();

    /**
     * create the Login and Register GUI and set its frame accordingly
     * @param gc    the bibciteGUI class that instantiated this class, necessary for some other classes to keep in touch and update other GUI elements when changed
     */
    public void createLoginGUI(BibCiteGUI gc) {
        BibCiteGUI.gc = gc;
        dbModel.createDBAccess();

        

        JDialog fLogin = new JDialog(frame, "Login or Register...", true);
        fLogin.setSize(480, 260);
        LoginRegisterGUI lrg = new LoginRegisterGUI(fLogin);
        fLogin.add(lrg);
        fLogin.setVisible(true);
       // Switch to LoginRegisterGUI-Eventhandling.
        fLogin.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE); //might be superfluous
        // Login-Dialog disposed:
        if (lrg.getUsername() != null) {
            // Login has been successfull.
            // if a user was found with same name and same passwordHash
            // System.out.println("Logged in as: "+lrg.getUsername()+" pw: "+lrg.getUserPassword());
            username = lrg.getUsername();
            user = lrg.getUser();

            SessionData.setLoginPassword(lrg.getUserPassword());
            SessionData.setUser(user);

            SessionData.getUser().removeKeysUnused();

            fLogin.setVisible(false);
            fLogin.dispose();
            gc.createGUI(gc);
        } else {
            // No Login has taken place. Close entire application.
            // if no user with the same name and same passwordHash was found
            // System.out.println("No valid credentials were given...");
            fLogin.setVisible(false);
            fLogin.dispose();
            frame.dispose();
        }
    }

    /**
     * create basic GUI structure, frame and menu bar
     */
    public void createGUI(BibCiteGUI gc) {
        BibCiteGUI.gc = gc;
        // System.out.println("bibcite.createSplitPane() opened database");
        
        JMenuBar menuBar = new JMenuBar();
        JMenu menuFile = new JMenu("File");
        JMenu menuImport = new JMenu("Import");

        JMenuItem itemImport = new JMenuItem("Import...");
        JMenuItem itemImportPDF = new JMenuItem("Import pdf (individual bib-entries, e.g. articles)...");
        JMenuItem itemImportPDFSingle = new JMenuItem("Import pdf (single bib-entry, e.g. book chapters)...");

        menuImport.add(itemImport);
        menuImport.add(itemImportPDF);
        menuImport.add(itemImportPDFSingle);

        JMenuItem itemExport = new JMenuItem("Export...");
        JMenuItem itemExportFiles = new JMenuItem("Export files...");
        JMenuItem itemExit = new JMenuItem("Exit");

        menuFile.add(menuImport);

        menuFile.add(itemExport);
        menuFile.add(itemExportFiles);
        menuFile.add(itemExit);

        JMenu menuSettings = new JMenu("Settings");
        JMenuItem itemUserSettings = new JMenuItem("User settings...");

        menuSettings.add(itemUserSettings);

        JMenu menuData = new JMenu("Data");
        itemNewBib = new JMenuItem("Create Bibliography...");
        // JMenuItem itemNewQuote = new JMenuItem("Create new Quote...");
        itemNewBib.addActionListener(new MenuActionListener());

        itemNewPerson = new JMenuItem("Create Person...");
        itemNewBib.addActionListener(new MenuActionListener());
        itemNewPerson.addActionListener(new MenuActionListener());

        itemExport.addActionListener(new MenuActionListener());
        itemExportFiles.addActionListener(new MenuActionListener());
        itemImport.addActionListener(new MenuActionListener());
        itemImportPDF.addActionListener(new MenuActionListener());
        itemImportPDFSingle.addActionListener(new MenuActionListener());
        itemExit.addActionListener(new MenuActionListener());
        itemUserSettings.addActionListener(new MenuActionListener());

        menuData.add(itemNewBib);
        menuData.add(itemNewPerson);

        // menuData.add(itemNewQuote);

        menuBar.add(menuFile);
        // menuBar.add(menuData);
        menuBar.add(menuSettings);

        frame.setLayout(new BorderLayout());
        frame.add(createSplitPane(), BorderLayout.CENTER);

        frame.add(progress, BorderLayout.SOUTH);

        frame.setJMenuBar(menuBar);

        frame.setVisible(true);
        frame.setSize(new Dimension(800, 600));
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
    }

    /**
     * create the split pane with JTreeView on the left and the content on the right
     * @return  JSplitPane filled with components
     */
    private JSplitPane createSplitPane() {
        // System.out.println("BibCiteGUI createSplitPane() User:"+user);
        splitPane = new JSplitPane();
        splitPane.setContinuousLayout(true);
        splitPane.setOneTouchExpandable(true);
        // System.out.println("Adding Treeview");

        alBibsNewest = dbModel.getBibliographiesNewestVersions();
        alQuotesNewest = dbModel.getQuotationsNewestVersions();
        alNotesNewest = dbModel.getNotesNewestVersions();
        alFilesNewest = dbModel.getFilesNewestVersions();
        alPersonsNewest = dbModel.getPersonsNewestVersions();

        tree = new TreeObjectSelectorGUI();
        // System.out.println("bibciteGUI.createSplitPane() alBibsNewest size:"+alBibsNewest.size());
        JPanel panelTree = tree.TreeView(alBibsNewest, alQuotesNewest, alNotesNewest, alFilesNewest, alPersonsNewest, user);
        splitPane.setLeftComponent(panelTree);

        dbModel.createBibEditor();
        tree.addTreeViewListener(new TreeViewListener());
        return splitPane;
    }

    class TreeViewListener implements TreeViewSelectionListener {

        /**
         * Here the editor area is updated with the selected entry from the JTree, it should display
         */
        public void selectionChanged(TreeViewSelectionEvent e) {
            progress.setProgressIndeterminate(true);
            progress.setText("Loading entry");

            // find out, whether the selected object was an instance of Person or Bibliography and display the adjunct editor view
            if (e.getUserObject() instanceof Person & !(e.getUserObject() instanceof User)) {
                splitPane.setRightComponent(new PersonEditor((Person)e.getUserObject()));
            }
            if (e.getUserObject() instanceof Bibliography) {
                splitPane.setRightComponent(new BibliographyEditor((Bibliography)e.getUserObject()));
            }
            if (e.getUserObject() instanceof Note) {
                splitPane.setRightComponent(new NoteEditor((Note)e.getUserObject()));
            }
            if (e.getUserObject() instanceof dbFile) {
                splitPane.setRightComponent(new FileEditor((dbFile)e.getUserObject()));
            }
            if (e.getUserObject() instanceof Quotation) {
                splitPane.setRightComponent(new QuotationEditor((Quotation) e.getUserObject()));
            }
            splitPane.setDividerLocation(200);
            progress.setText("");
            progress.setProgressIndeterminate(false);
        }
    }

    /**
     * The ActionListener for the menu
     */
    class MenuActionListener implements ActionListener {

        public void actionPerformed(ActionEvent ae) {
            // System.out.println(ae.getActionCommand());
            if (ae.getActionCommand().contains("Exit")) {

                System.exit(0);
            }
            else if (ae.getActionCommand().equals("Export...")) {
                // System.out.println("Exporting...");
                JDialog dialogExport = new JDialog(BibCiteGUI.frame, "Login or Register...", true);
                dialogExport.setModal(true);

                ExportGUI export = new ExportGUI(user, dialogExport);
                dialogExport.add(export);
                dialogExport.pack();
                // dialogExport.setSize(new Dimension(400,300));
                dialogExport.setVisible(true);

                if (export.isExport()) {
                    if (export.selectedFormat().equals("bibTex")) {
                        Export_bibTex.export_bibTex(export.selectedBibliographies(), export.isExportPrivate(), export.isExportQuotes(), export.isExportPersons(), export.isExportNotes(), export.isExportSeparate());
                    }
                }
            }
            else if(ae.getActionCommand().equals("Export files...")){
                System.out.println("Showing export files dialog.");
                JDialog dialog = new JDialog();
                ExportFilesGUI efg = new ExportFilesGUI(dialog);
                dialog.add(efg);
                dialog.pack();
                dialog.setVisible(true);
            }
            else if (ae.getActionCommand().contains("Import...")) {
                // System.out.println("Showing filechooser");

                //Create a file chooser
                final JFileChooser fc = new JFileChooser();
                fc.setFileFilter(new net.geoadel.bibcite.FileFilters.FileFilter_bibTex());
                fc.setMultiSelectionEnabled(true);
                //In response to a button click:
                int returnVal = fc.showOpenDialog(null);

                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    SwingWorker<Void, Void> importWorker = 
                        new SwingWorker<Void, Void>() {
                            public Void doInBackground() {
                                new Import_bibTex().importObjectsFromBibTex(fc.getSelectedFiles());
                                user = SessionData.getUser();
                                updateTree(dbModel.getBibliographiesNewestVersions(), dbModel.getQuotationsNewestVersions(), dbModel.getNotesNewestVersions(), dbModel.getFilesNewestVersions(), dbModel.getPersonsNewestVersions());
                                return null;
                            }
                            public void done() {
                                try {
                                   get();
                                   ProgressNotificator.setText("");
                                   ProgressNotificator.setProgressIndeterminate(false);
                                } catch (InterruptedException ex) {
                                   ex.printStackTrace();
                                } catch (Exception ex) {
                                }
                            }
                       };
                    importWorker.execute();
                    ProgressNotificator.setText("Importing files");
                    ProgressNotificator.setProgressIndeterminate(true);
                } else {
                    // log.append("Open command cancelled by user." + newline);
                }
            }
            else if (ae.getActionCommand().contains("Import pdf (single bib-entry, e.g. book chapters)...")) {
                //Create a file chooser
                final JFileChooser fc = new JFileChooser();
                fc.setFileFilter(new net.geoadel.bibcite.FileFilters.FileFilter_pdf());

                fc.setMultiSelectionEnabled(true);
                //In response to a button click:
                int returnVal = fc.showOpenDialog(null);
                ProgressNotificator.setText("Importing files");
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                   SwingWorker<Void, Void> importWorker = 
                        new SwingWorker<Void, Void>() {
                            public Void doInBackground() {
                                FileOperations.importPDFsToBibs(fc.getSelectedFiles(), true);
                                updateTree(dbModel.getBibliographiesNewestVersions(), dbModel.getQuotationsNewestVersions(), dbModel.getNotesNewestVersions(), dbModel.getFilesNewestVersions(), dbModel.getPersonsNewestVersions());
                                return null;
                            }
                            public void done() {
                                try {
                                   get();
                                   ProgressNotificator.setText("");
                                   ProgressNotificator.setProgressIndeterminate(false);
                                } catch (InterruptedException ex) {
                                   ex.printStackTrace();
                                } catch (Exception ex) {
                                }
                            }
                       };
                    
                    // import files each as bib entries with the file attached
                   
                    ProgressNotificator.setText("Importing files");
                    ProgressNotificator.setProgressIndeterminate(true); 
                    importWorker.execute();
                } else {
                    // log.append("Open command cancelled by user." + newline);
                }
            }
            else if (ae.getActionCommand().contains("Import pdf (individual bib-entries, e.g. articles)...")) {
                //Create a file chooser
                final JFileChooser fc = new JFileChooser();
                fc.setFileFilter(new net.geoadel.bibcite.FileFilters.FileFilter_pdf());

                fc.setMultiSelectionEnabled(true);
                //In response to a button click:
                int returnVal = fc.showOpenDialog(null);

                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    // import files each as bib entries with the file attached
                     SwingWorker<Void, Void> importWorker = 
                        new SwingWorker<Void, Void>() {
                            public Void doInBackground() {
                                FileOperations.importPDFsToBibs(fc.getSelectedFiles(), false);
                                user = SessionData.getUser();
                                updateTree(dbModel.getBibliographiesNewestVersions(), dbModel.getQuotationsNewestVersions(), dbModel.getNotesNewestVersions(), dbModel.getFilesNewestVersions(), dbModel.getPersonsNewestVersions());
                                return null;
                            }
                            public void done() {
                                try {
                                   get();
                                   ProgressNotificator.setText("");
                                   ProgressNotificator.setProgressIndeterminate(false);
                                } catch (InterruptedException ex) {
                                   ex.printStackTrace();
                                } catch (Exception ex) {
                                }
                            }
                       };      
                    // import files each as bib entries with the file attached                 
                    ProgressNotificator.setText("Importing files");
                    ProgressNotificator.setProgressIndeterminate(true); 
                    importWorker.execute();
                } else {
                    // log.append("Open command cancelled by user." + newline);
                }
            }
            // System.out.println(System.getProperty("user.home"));
            else if (ae.getActionCommand().contains("Create Bibliography...")) {
                // System.out.println("Mainmenu - New Bibliography");
                splitPane.setRightComponent(new BibliographyEditor(new Bibliography()));
                TreeObjectSelectorGUI.showButtonsBib();
                splitPane.setDividerLocation(200);
            }
            else if (ae.getActionCommand().contains("Create Person...")) {
                // System.out.println("Mainmenu - New Person");
                splitPane.setRightComponent(new PersonEditor(new Person()));
                TreeObjectSelectorGUI.showButtonsPerson();
                splitPane.setDividerLocation(200);
            }
            else if (ae.getActionCommand().contains("Create Note...")) {
                // System.out.println("Mainmenu - New Person");
                splitPane.setRightComponent(new NoteEditor(new Note()));
                TreeObjectSelectorGUI.showButtonsNote();
                splitPane.setDividerLocation(200);
            }
            else if (ae.getActionCommand().contains("User settings...")) {
                PreferencesEditor preferencesGUI = new PreferencesEditor();
                splitPane.setRightComponent(preferencesGUI);
                splitPane.setDividerLocation(200);
            }
        }
    }

    // this updates the tree by setting a new TreeView to the splitPane, replacing the old treeview
    public void updateTree(ArrayList<Bibliography> bibs, ArrayList<Quotation> quotes, ArrayList<Note> notes, ArrayList<dbFile> files, ArrayList<Person> persons) {
        if(splitPane == null){
            createSplitPane();
        }
        tree = new TreeObjectSelectorGUI();
        JPanel panelTree = tree.TreeView(bibs, quotes, notes, files, persons, SessionData.getUser());
        tree.addTreeViewListener(new TreeViewListener());
        splitPane.setLeftComponent(panelTree);
        splitPane.setDividerLocation(200);
    }

    public static void setEditorArea(QuotationEditor editor) {
        splitPane.setRightComponent(editor);
        splitPane.setDividerLocation(200);
    }

    public static void setEditorArea(BibliographyEditor editor) {
        splitPane.setRightComponent(editor);
        splitPane.setDividerLocation(200);
    }

    public static void setEditorArea(FileEditorPanel panel) {
        splitPane.setRightComponent(panel);
        splitPane.setDividerLocation(200);
    }

    public static void setEditorArea(NoteEditor editor) {
        splitPane.setRightComponent(editor);
        splitPane.setDividerLocation(200);
    }

    public static void setEditorArea(PersonEditor editor) {
        splitPane.setRightComponent(editor);
        splitPane.setDividerLocation(200);
    }
}
