/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * ImageViewerPanel.java
 *
 * Created on 15.06.2010, 16:59:36
 */
package net.geoadel.bibcite.GUI;

import net.geoadel.bibcite.GUI.ObjectEditors.FileExternalEditingDialog;
import java.awt.AlphaComposite;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.db4o;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.DatabaseObjects.User;
import net.geoadel.bibcite.DatabaseObjects.dbFile;
import net.geoadel.bibcite.Tools.FileOperations;

/**
 * Provides a rudimentary image viewer panel with zoom in and out options and an edit-button for editing the image externally
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class ImageViewerPanel extends javax.swing.JPanel {

    File file; // the file to be displayed
    BufferedImage originalImage; // the file where the image is derived from, needed for calling external editing application
    BufferedImage resizedImage; // the resized image (should always be recalculated from original image to avoid subsequential quality degradation)
    int width;  // current width of resizedImage
    int height; // current height of resizedImage
    User user;
    dbFile dbfile;
    UUID sourceUUID;

    /**
     * Creates new form ImageViewerPanel
     */
    public ImageViewerPanel(BufferedImage originalImage, User user, dbFile dbfile, File file, UUID sourceUUID) {
        this.user = user;
        this.sourceUUID = sourceUUID;
        this.dbfile = dbfile;
        this.originalImage = originalImage;
        this.file = file; // path is used for calling external edit applications
        this.height = originalImage.getHeight();
        this.width = originalImage.getWidth();
        initComponents();
        lImage.setIcon(new ImageIcon(originalImage));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jToolBar1 = new javax.swing.JToolBar();
        bZoomNo = new javax.swing.JButton();
        bZoomMagnify = new javax.swing.JButton();
        bZoomOut = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JToolBar.Separator();
        bEdit = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        lImage = new javax.swing.JLabel();

        jToolBar1.setRollover(true);

        bZoomNo.setText("=");
        bZoomNo.setFocusable(false);
        bZoomNo.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bZoomNo.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bZoomNo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bZoomNoActionPerformed(evt);
            }
        });
        jToolBar1.add(bZoomNo);

        bZoomMagnify.setText("+");
        bZoomMagnify.setFocusable(false);
        bZoomMagnify.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bZoomMagnify.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bZoomMagnify.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bZoomMagnifyActionPerformed(evt);
            }
        });
        jToolBar1.add(bZoomMagnify);

        bZoomOut.setText("-");
        bZoomOut.setFocusable(false);
        bZoomOut.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bZoomOut.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bZoomOut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bZoomOutActionPerformed(evt);
            }
        });
        jToolBar1.add(bZoomOut);
        jToolBar1.add(jSeparator1);

        bEdit.setText("Edit...");
        bEdit.setFocusable(false);
        bEdit.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bEdit.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bEdit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bEditActionPerformed(evt);
            }
        });
        jToolBar1.add(bEdit);

        jScrollPane1.setViewportView(lImage);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 596, Short.MAX_VALUE)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 596, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 383, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    /**
     * If button bZoomNo was pressed, set resizedImage to originalImage and reset width and height
     * @param evt
     */
    private void bZoomNoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bZoomNoActionPerformed
        lImage.setIcon(new ImageIcon(originalImage));
        lImage.repaint();
        resizedImage = originalImage;
        height = resizedImage.getHeight();
        width = resizedImage.getWidth();
    }//GEN-LAST:event_bZoomNoActionPerformed

    /**
     * If bZoomMagnify was pressed, scale image by 20 percent
     * @param evt
     */
    private void bZoomMagnifyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bZoomMagnifyActionPerformed
        if (width * height < 10000000) {
            int type = originalImage.getType();
            height = Math.round(height * 1.2f);
            width = Math.round(width * 1.2f);
            lImage.setIcon(new ImageIcon(resizeImage(originalImage, type, width, height)));
            lImage.repaint();
        } else {
            // System.out.println("width " + Math.round(width * 1.2f) + " height " + Math.round(height * 1.2f) + " too large");
        }

    }//GEN-LAST:event_bZoomMagnifyActionPerformed

    /**
     * If bZoomOut was pressed, reduce image size by 20 percent
     * @param evt
     */
    private void bZoomOutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bZoomOutActionPerformed
        int type = originalImage.getType();
        height = Math.round(height * 0.8f);
        width = Math.round(width * 0.8f);
        lImage.setIcon(new ImageIcon(resizeImage(originalImage, type, width, height)));
        this.updateUI();
        this.repaint();
    }//GEN-LAST:event_bZoomOutActionPerformed

    /**
     * If bEdit was pressed, open image in default operating system editing application
     * @param evt
     */
    private void bEditActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bEditActionPerformed
        try {
            // create temporary file (in Operating System temporary folder) to open it
            File tempFile = FileOperations.createTemporaryFile(dbfile, user);

            // open
            Desktop.getDesktop().open(tempFile);
            JDialog dialogEdit = new JDialog(BibCiteGUI.gc.frame, "Login or Register...", true);
            dialogEdit.setModal(true);
            FileExternalEditingDialog fExEd = new FileExternalEditingDialog(dialogEdit, false, tempFile.getName());
            dialogEdit.add(fExEd);

            dialogEdit.setSize(new Dimension(400, 300));
            dialogEdit.setVisible(true);

            if (fExEd.getChosenOption().equals("New")) { // if in dialog the option New revision was chosen, get the file from the temp directory and move it as a new revision into the database
                dbFile f = new dbFile(tempFile, tempFile.length(), SessionData.getUser(), dbfile.getUUID(), dbfile.getDateEdited(), dbfile.isPrivate(), dbfile.getRelatedObjects(), dbfile.getTags(), dbfile.getName(), dbfile.getKey(), dbfile.getCustomFields());
                dbModel.storeObject(f);
            }
            if (fExEd.getChosenOption().equals("Overwrite")) {
                dbFile f = new dbFile(tempFile, tempFile.length(), SessionData.getUser(), dbfile.getUUID(), dbfile.getDateEdited(), dbfile.isPrivate(), dbfile.getRelatedObjects(), dbfile.getTags(), dbfile.getName(), dbfile.getKey(), dbfile.getCustomFields());
                dbModel.storeObject(f);
            }
        } catch (IOException ex) {
            Logger.getLogger(ImageViewerPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_bEditActionPerformed

    /**
     * resizes image
     * @param originalImage original image to be resized
     * @param type      band type
     * @param width     new width of image
     * @param height    new height of image
     * @return  resized BufferedImage
     */
    private BufferedImage resizeImage(BufferedImage originalImage, int type, int width, int height) {
        resizedImage = new BufferedImage(width, height, type);
        Graphics2D g = resizedImage.createGraphics();

        // set rendering hints for better quality
        g.setComposite(AlphaComposite.Src);
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        g.drawImage(originalImage, 0, 0, width, height, null);
        g.dispose();
        return resizedImage;
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bEdit;
    private javax.swing.JButton bZoomMagnify;
    private javax.swing.JButton bZoomNo;
    private javax.swing.JButton bZoomOut;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToolBar.Separator jSeparator1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JLabel lImage;
    // End of variables declaration//GEN-END:variables
}
