/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * BibliographyViewer.java
 *
 * Created on Aug 18, 2010, 3:56:02 PM
 */

package net.geoadel.bibcite.GUI.ObjectEditors;

import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;
import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.DatabaseObjects.Quotation;
import net.geoadel.bibcite.ImportExport.Export_html;

/**
 * Provides a basic html viewer for displaying the html file of a bibliography entry together with all of its quotes
 * @author Georg Adelmann
 */
public class BibliographyViewer extends javax.swing.JPanel {
    File f;
    /** Creates new form BibliographyViewer */
    public BibliographyViewer(Bibliography bib, ArrayList<Quotation> alQuotes) {
        initComponents();
        // System.out.println("Title of bib for BibliographyViewer: "+bib.getTitle());
        try {
            f = Export_html.exportTempHtml(bib, alQuotes);
            // System.out.println("File path: "+f.getPath());

            // add a HTMLEditorKit to the editor pane
            HTMLEditorKit kit = new HTMLEditorKit();
            jEditorPane1.setEditorKit(kit);

            // add some styles to the html
            StyleSheet styleSheet = kit.getStyleSheet();
            String[] rules = Export_html.getStyle().split("\n");
            for(String rule : rules){
                styleSheet.addRule(rule);
            }

            Document doc = kit.createDefaultDocument();
            jEditorPane1.setDocument(doc);
            // jEditorPane1.setText(f.toURI());
            jEditorPane1.setPage(f.toURI().toURL());

        } catch (IOException ex) {
            // System.out.println("Did not find temporary html file");
            Logger.getLogger(BibliographyViewer.class.getName()).log(Level.SEVERE, null, ex);
        } catch (NullPointerException npe){
            // System.out.println("NullPointer, probably the bibliography does not exist");
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jEditorPane1 = new javax.swing.JEditorPane();
        jToolBar1 = new javax.swing.JToolBar();
        bOpenBrowser = new javax.swing.JButton();

        jEditorPane1.setPreferredSize(new java.awt.Dimension(80, 80));
        jScrollPane1.setViewportView(jEditorPane1);

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);

        bOpenBrowser.setText("Open in browser...");
        bOpenBrowser.setFocusable(false);
        bOpenBrowser.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bOpenBrowser.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bOpenBrowser.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bOpenBrowserActionPerformed(evt);
            }
        });
        jToolBar1.add(bOpenBrowser);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 442, Short.MAX_VALUE)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 442, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 413, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void bOpenBrowserActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bOpenBrowserActionPerformed
        try {
            Desktop desktop = Desktop.getDesktop();
            desktop.open(f);
        } catch (IOException ex) {
            Logger.getLogger(BibliographyViewer.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_bOpenBrowserActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bOpenBrowser;
    private javax.swing.JEditorPane jEditorPane1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToolBar jToolBar1;
    // End of variables declaration//GEN-END:variables

}
