/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * FileEditor.java
 *
 * Created on Aug 9, 2010, 2:46:17 PM
 */
package net.geoadel.bibcite.GUI.ObjectEditors;

import net.geoadel.bibcite.GUI.ObjectListPanels.FileEditorPanel;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import net.geoadel.bibcite.Cryptography.EncryptDecrypt;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.db4o;
import net.geoadel.bibcite.DatabaseObjects.User;
import net.geoadel.bibcite.DatabaseObjects.dbFile;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.GUI.AttributeEditors.AccessManagerGUI;
import net.geoadel.bibcite.GUI.BibCiteGUI;
import net.geoadel.bibcite.GUI.ImageViewerPanel;
import net.geoadel.bibcite.GUI.AttributeEditors.RelationManager;
import net.geoadel.bibcite.Tools.FileOperations;
import net.geoadel.bibcite.Tools.ImageOperations;

/**
 *  JPanel providing an editor for file-entries in the database
 * @author Georg Adelmann
 */
public class FileEditor extends javax.swing.JPanel {

    private User user;
    private UUID sourceUUID;
    FileEditorPanel parentPanel;
    boolean minimized = false;
    BufferedImage bImage;
    private ImageViewerPanel iv;
    public dbFile dbfile;
    AccessManagerGUI accessManager;

    /**
     * Creates a new FileEditor for displaying newly added files after clicking on 'add new'
     * @param sourceUUID
     * @param user
     * @param parentPanel
     */
    public FileEditor(UUID sourceUUID) {
        this.sourceUUID = sourceUUID;
        this.parentPanel = parentPanel;
        this.user = user;

        initComponents();

        dbfile = loadFileExternal();

        initData(dbfile);

        jTabbedPane1.add(new RelationManager(dbfile), "Relations");

        final ComboBoxRevisionAccess cbRevisions = new ComboBoxRevisionAccess(dbModel.getRevisionsOf(dbfile.getUUID()));
        cbRevisions.getModel().setSelectedItem(dbfile);
        cbRevisions.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dbfile = (dbFile) cbRevisions.getSelectedItem();
                initData((dbFile) cbRevisions.getSelectedItem());
            }
        });
        jToolBar1.add(cbRevisions);
    }

    /**
     * For displaying files that are loaded from the database
     * @param sourceUUID
     * @param dbfile
     * @param alRevisions
     * @param user
     * @param parentPanel
     */
    public FileEditor(dbFile dbfileLoad) {
        this.dbfile = dbfileLoad;
        this.parentPanel = parentPanel;

        initComponents();

        jTabbedPane1.add(new RelationManager(dbfile), "Relations");

        final ComboBoxRevisionAccess cbRevisions = new ComboBoxRevisionAccess(dbModel.getRevisionsOf(dbfile.getUUID()));
        cbRevisions.getModel().setSelectedItem(dbfile);
        cbRevisions.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dbfile = (dbFile) cbRevisions.getSelectedItem();
                initData((dbFile) cbRevisions.getSelectedItem());
            }
        });
        jToolBar1.add(cbRevisions);
    }

    public void initData(dbFile dbfile) {
        ArrayList<String> tags = dbfile.getTags();
        String sTags = "";
        // System.out.println("Sorting through note tags:");
        for (int i = 0; i < tags.size(); i++) {
            // System.out.println(tags.get(i));
            sTags = sTags.concat(tags.get(i));
            if (i < tags.size() - 1) {
                sTags = sTags.concat(", ");
            }
        }
        tfTags.setText(sTags);

        lFileDate.setText(dbfile.getDateEdited().toString());
        lFileName.setText(dbfile.getName());
        tfFileName.setText(dbfile.getName());
        lFileSize.setText(dbfile.getSize().toString());
        lUUIDValue.setText(dbfile.getUUID().toString());

        cbIsPrivate.setSelected(dbfile.isPrivate());

        loadImage();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jToolBar1 = new javax.swing.JToolBar();
        tbMaxMin = new javax.swing.JToggleButton();
        lFileName = new javax.swing.JLabel();
        cbIsPrivate = new javax.swing.JCheckBox();
        bIsPrivate = new javax.swing.JButton();
        bFileSave = new javax.swing.JButton();
        bFileRemove = new javax.swing.JButton();
        bFileOpen = new javax.swing.JButton();
        bSetIcon = new javax.swing.JButton();
        lRevisions = new javax.swing.JLabel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        lFileSize = new javax.swing.JLabel();
        lFileDate = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        tfTags = new javax.swing.JTextField();
        lImage = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        tfFileName = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        lUUID = new javax.swing.JLabel();
        lUUIDValue = new javax.swing.JLabel();

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);

        tbMaxMin.setText("+");
        tbMaxMin.setFocusable(false);
        tbMaxMin.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        tbMaxMin.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        tbMaxMin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tbMaxMinActionPerformed(evt);
            }
        });
        jToolBar1.add(tbMaxMin);

        lFileName.setText("Filename");
        lFileName.setMaximumSize(new java.awt.Dimension(200, 13));
        lFileName.setMinimumSize(new java.awt.Dimension(180, 13));
        lFileName.setPreferredSize(new java.awt.Dimension(180, 13));
        jToolBar1.add(lFileName);

        cbIsPrivate.setToolTipText("set private");
        cbIsPrivate.setFocusable(false);
        cbIsPrivate.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        cbIsPrivate.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        cbIsPrivate.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        cbIsPrivate.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                cbIsPrivateStateChanged(evt);
            }
        });
        cbIsPrivate.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbIsPrivateItemStateChanged(evt);
            }
        });
        jToolBar1.add(cbIsPrivate);

        bIsPrivate.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/privateKey24.png"))); // NOI18N
        bIsPrivate.setToolTipText("set private");
        bIsPrivate.setBorderPainted(false);
        bIsPrivate.setFocusPainted(false);
        bIsPrivate.setFocusable(false);
        bIsPrivate.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bIsPrivate.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bIsPrivate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bIsPrivateActionPerformed(evt);
            }
        });
        jToolBar1.add(bIsPrivate);

        bFileSave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/fileSave.png"))); // NOI18N
        bFileSave.setToolTipText("Save");
        bFileSave.setEnabled(false);
        bFileSave.setPreferredSize(new java.awt.Dimension(30, 20));
        bFileSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bFileSaveActionPerformed(evt);
            }
        });
        jToolBar1.add(bFileSave);

        bFileRemove.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/fileDelete.png"))); // NOI18N
        bFileRemove.setToolTipText("Erase");
        bFileRemove.setMaximumSize(new java.awt.Dimension(30, 20));
        bFileRemove.setMinimumSize(new java.awt.Dimension(30, 20));
        bFileRemove.setPreferredSize(new java.awt.Dimension(30, 20));
        bFileRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bFileRemoveActionPerformed(evt);
            }
        });
        jToolBar1.add(bFileRemove);

        bFileOpen.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/fileImport.png"))); // NOI18N
        bFileOpen.setToolTipText("<html><body>Open file in external application<p>Needs standard application set in User settings</body></html>");
        bFileOpen.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bFileOpenActionPerformed(evt);
            }
        });
        jToolBar1.add(bFileOpen);

        bSetIcon.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/ImagetoIcon.png"))); // NOI18N
        bSetIcon.setToolTipText("Use a small version of this image as preview icon for this object");
        bSetIcon.setEnabled(false);
        bSetIcon.setMaximumSize(new java.awt.Dimension(24, 36));
        bSetIcon.setMinimumSize(new java.awt.Dimension(24, 36));
        bSetIcon.setPreferredSize(new java.awt.Dimension(40, 30));
        bSetIcon.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bSetIconActionPerformed(evt);
            }
        });
        jToolBar1.add(bSetIcon);

        lRevisions.setText("Rev.");
        lRevisions.setPreferredSize(new java.awt.Dimension(30, 13));
        jToolBar1.add(lRevisions);

        lFileSize.setText("Size");

        lFileDate.setText("Date added");

        jLabel3.setText("Tags");

        tfTags.setToolTipText("Tags are divided by comata ','");
        tfTags.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tfTagsActionPerformed(evt);
            }
        });
        tfTags.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
            }
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                tfTagsCaretPositionChanged(evt);
            }
        });
        tfTags.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                tfTagsKeyTyped(evt);
            }
        });

        lImage.setPreferredSize(new java.awt.Dimension(300, 200));
        lImage.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                lImageMouseClicked(evt);
            }
        });

        jLabel1.setText("Name");

        tfFileName.setText("jTextField1");
        tfFileName.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
            }
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                tfFileNameCaretPositionChanged(evt);
            }
        });

        jLabel2.setText("Size (kb):  ");

        jLabel4.setText("Date added:   ");

        lUUID.setText("UUID");

        lUUIDValue.setText("uuid");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel3)
                            .addComponent(jLabel1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(tfFileName)
                            .addComponent(tfTags, javax.swing.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE)))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel4)
                            .addComponent(jLabel2)
                            .addComponent(lUUID))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lUUIDValue, javax.swing.GroupLayout.DEFAULT_SIZE, 218, Short.MAX_VALUE)
                            .addComponent(lFileSize)
                            .addComponent(lFileDate))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(lImage, javax.swing.GroupLayout.DEFAULT_SIZE, 309, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(lImage, javax.swing.GroupLayout.Alignment.LEADING, 0, 200, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel1)
                            .addComponent(tfFileName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(30, 30, 30)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel3)
                            .addComponent(tfTags, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(18, 18, 18)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel2)
                            .addComponent(lFileSize))
                        .addGap(30, 30, 30)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(lFileDate)
                            .addComponent(jLabel4))
                        .addGap(18, 18, 18)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(lUUID)
                            .addComponent(lUUIDValue))))
                .addGap(131, 131, 131))
        );

        jTabbedPane1.addTab("File", jPanel1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 656, Short.MAX_VALUE)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 656, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 288, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void bFileRemoveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bFileRemoveActionPerformed
        dbModel.deleteObject(dbfile.getUUID());
        this.setVisible(false);
        lImage.setIcon(null);
        lImage.updateUI();
}//GEN-LAST:event_bFileRemoveActionPerformed

    private void bFileSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bFileSaveActionPerformed
        dbModel.storeObject(getFileCurrent());
}//GEN-LAST:event_bFileSaveActionPerformed

    public dbFile getFileCurrent() {
        String[] arrayTags = tfTags.getText().split(",");
        ArrayList<String> tags = new ArrayList<String>();
        for (int i = 0; i < arrayTags.length; i++) {
            tags.add(arrayTags[i].trim());
        }

        // System.out.println("selected Name: "+selectedDBFile.getName());
        ArrayList<UUID> alRelations = dbfile.getRelatedObjects();
        if (sourceUUID != null) {
            alRelations.add(sourceUUID);
        }

        dbFile fileCurrent = new dbFile(dbfile.getFile(), SessionData.getUser(), alRelations, dbfile.getUUID(), cbIsPrivate.isSelected(), tfFileName.getText(), dbfile.getKey());
        return fileCurrent;

    }

    private void bSetIconActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bSetIconActionPerformed
        try {
            UUID uuid = parentPanel.getSourceUUID();
            // if sourceUUID != null, then this editor is nested within another database object, in that case, set the preview icons for this object
            if (uuid == null) {
                uuid = dbfile.getUUID();
            }
            FileOperations.transformBufferedImageToPreviewImages(ImageOperations.getIconSmall(bImage), ImageOperations.getIconMedium(bImage), ImageOperations.getIconLarge(bImage), uuid, cbIsPrivate.isSelected(), SessionData.getUser());
        } catch (IOException ex) {
            Logger.getLogger(FileEditor.class.getName()).log(Level.SEVERE, null, ex);
        }
}//GEN-LAST:event_bSetIconActionPerformed

    private void bFileOpenActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bFileOpenActionPerformed
        try {
            File tempFile = FileOperations.createTemporaryFile(dbfile, SessionData.getUser());
            String fileName = tempFile.toString();
            System.out.println(fileName);
            dbFile dbOrigin = dbfile;

            // create the program execute string that has the temporary file as its parameter
            String type = dbOrigin.getName().substring(dbOrigin.getName().lastIndexOf(".") + 1).toUpperCase();

            String programSettings = SessionData.getUser().getOperatingSystemDefaultProgram(SessionData.getUser().getDefaultOperatingSystem(), type);
            // System.out.println("Opening file type " + type+" with "+programSettings);
            Desktop desktop = Desktop.getDesktop();

            if (programSettings != null || desktop.isSupported(Desktop.Action.OPEN)) {
                if (programSettings != null) {
                    ArrayList<String> command = new ArrayList<String>();
                    command.add(programSettings);
                    command.add(tempFile.getPath());

                    // System.out.println("Try to open file " + tempFile.getPath() + " with help of " + execute);
                    new ProcessBuilder(command).start();
                } else {
                    desktop.open(tempFile);
                }

                // start the dialog that lets the user choose to overwrite the existing database file revision or create a new revision
                JDialog dialogEdit = new JDialog(BibCiteGUI.frame, "Edit file in external application...", true);

                dialogEdit.setModal(true);
                System.out.println(fileName);
                FileExternalEditingDialog fExEd = new FileExternalEditingDialog(dialogEdit, false, fileName);
                dialogEdit.add(fExEd);
                // dialogEdit.pack();
                dialogEdit.setSize(new Dimension(400, 300));
                dialogEdit.setVisible(true);

                // if in dialog the option New revision was chosen, get the file from the temp directory and move it as a new revision into the database
                // this works by invoking the dbFile constructor which sets a new date, so that the db4o.java class does not delete the old dbfile entry (as it does, if a file with same uuid and date is written)
                if (fExEd.getChosenOption().equals("New")) {
                    // I use db.retrieveFileByUUID(SessionData.getUser(), dbOrigin.getUUID()).getRelatedObjects() to retrieve the current relations, since relations can be changed by RelationManager, which stores its changes in the database and NOT in the related data field
                    dbFile f = new dbFile(tempFile, SessionData.getUser(), ((dbFile) dbModel.getFileByUUID(dbOrigin.getUUID())).getRelatedObjects(), dbOrigin.getUUID(), dbOrigin.isPrivate(), dbOrigin.getName(), dbOrigin.getKey());
                    dbModel.storeObject(f);
                }
                // if Overwrite was chosen, the old file is overwritten, since db4o.java checks for date and uuid and they are the same as for the old dbfile, which in this case is deleted
                if (fExEd.getChosenOption().equals("Overwrite")) {
                    dbFile f = new dbFile(tempFile, tempFile.length(), SessionData.getUser(), dbOrigin.getUUID(), dbOrigin.getDateEdited(), dbOrigin.isPrivate(), ((dbFile) dbModel.getFileByUUID(dbOrigin.getUUID())).getRelatedObjects(), dbOrigin.getTags(), dbOrigin.getName(), dbOrigin.getKey(), dbOrigin.getCustomFields());
                    dbModel.storeObject(f);
                }
            } else {
                // System.out.println("Operating system does not support opening this file type or correct program not set in User Preferences.");
            }

        } catch (IOException ex) {
            Logger.getLogger(ImageViewerPanel.class.getName()).log(Level.SEVERE, null, ex);
        }

}//GEN-LAST:event_bFileOpenActionPerformed

    private void tfTagsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tfTagsActionPerformed
        bFileSave.setEnabled(true);
}//GEN-LAST:event_tfTagsActionPerformed

    private void tfTagsCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_tfTagsCaretPositionChanged
        bFileSave.setEnabled(true);
}//GEN-LAST:event_tfTagsCaretPositionChanged

    private void tfTagsKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_tfTagsKeyTyped
        bFileSave.setEnabled(true);
}//GEN-LAST:event_tfTagsKeyTyped

    private void lImageMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_lImageMouseClicked
        if (lImage.getIcon() != null) {
            JFrame frame = new JFrame(dbfile.getName());
            iv = new ImageViewerPanel(bImage, SessionData.getUser(), dbfile, dbfile.getFile(), sourceUUID);
            frame.add(iv);
            frame.pack();
            frame.setVisible(true);
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        }
}//GEN-LAST:event_lImageMouseClicked

    private void tbMaxMinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tbMaxMinActionPerformed
        if (minimized) {
            // setSize(new Dimension(getWidth(), 500));
            setPreferredSize(new Dimension(((Component)this.getParent()).getWidth() - 20, 380));
            setMaximumSize(new Dimension(((Component)this.getParent()).getWidth() - 20, 380));
            revalidate();
            // this.setSize(new Dimension(500,300));
            // this.setPreferredSize(new Dimension(this.getWidth(),400));
            minimized = false;
        } else {
            setPreferredSize(new Dimension(((Component)this.getParent()).getWidth() - 20, 30));
            setMaximumSize(new Dimension(((Component)this.getParent()).getWidth() - 20, 30));
            revalidate();
            // this.setSize(new Dimension(500,90));
            // this.setPreferredSize(new Dimension(this.getWidth(),70));
            minimized = true;
        }
        ((Component)this.getParent()).validate();
        // jTabbedPane1.setVisible(!minimized);
    }//GEN-LAST:event_tbMaxMinActionPerformed

    private void cbIsPrivateStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_cbIsPrivateStateChanged
}//GEN-LAST:event_cbIsPrivateStateChanged

    private void cbIsPrivateItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cbIsPrivateItemStateChanged
        if (!cbIsPrivate.isSelected()) {
            jTabbedPane1.remove(accessManager);
        } else {
            if (accessManager == null) {
                accessManager = new AccessManagerGUI(dbfile);
            }
            jTabbedPane1.add(accessManager, "Access");
        }
        bFileSave.setEnabled(true);
}//GEN-LAST:event_cbIsPrivateItemStateChanged

    private void bIsPrivateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bIsPrivateActionPerformed
        cbIsPrivate.setSelected(!cbIsPrivate.isSelected());
}//GEN-LAST:event_bIsPrivateActionPerformed

    private void tfFileNameCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_tfFileNameCaretPositionChanged
        dbfile.setName(tfFileName.getText());
        lFileName.setText(tfFileName.getText());
    }//GEN-LAST:event_tfFileNameCaretPositionChanged

    public dbFile loadFileExternal() {
        return new dbModel().importFile();
    }

    public void loadImage() {
        String fileName = dbfile.getName();
        String fileType = fileName.substring(fileName.lastIndexOf(".") + 1).toLowerCase();
        // System.out.println(fileType + " of file " + dbfile.getFile().getPath());
        if (fileType.equals("png") || fileType.equals("gif") || fileType.equals("jpg") || fileType.equals("jpeg")) {
            bSetIcon.setEnabled(true);
            try {
                if (dbfile.isPrivate()) {
                    // get the byte array of the selected file
                    byte[] b = FileOperations.getFileAsByte(dbfile.getUUID(), dbfile.getRevision());

                    EncryptDecrypt crypt = new EncryptDecrypt();
                    b = crypt.decryptPrivate(b, dbfile.getUUID()); // try encryption

                    InputStream in = new ByteArrayInputStream(b); // use an inputstream to make the byte array availlable to ImageIO.read()
                    bImage = ImageIO.read(in);
                } else {
                    try {
                        // System.out.println("Loading file " + FileOperations.getFile(dbfile.getUUID(), dbfile.getRevision()).getPath());
                        // load image-file into BufferedImage, if not private that can be done directly

                        // get the byte array of the selected file
                        byte[] b = FileOperations.getFileAsByte(dbfile.getUUID(), dbfile.getRevision());

                        EncryptDecrypt crypt = new EncryptDecrypt();
                        b = crypt.decryptPublic(b); // try encryption

                        InputStream in = new ByteArrayInputStream(b); // use an inputstream to make the byte array availlable to ImageIO.read()
                        bImage = ImageIO.read(in);

                    } catch (javax.imageio.IIOException iio) {
                        // System.out.println("Could not load file " + dbfile.getFile().getPath());
                        // the file could not be loaded, quite likely the file is damaged alreay before trying to load it.
                    }
                }
                // scale image for icon
                int iconWidth = 1;
                int iconHeight = 1;
                // System.out.println(bImage.getWidth() + ", " + bImage.getHeight());
                if (bImage.getWidth() > bImage.getHeight()) {
                    iconWidth = 300;
                    float newHeight = ((float) iconWidth / (float) bImage.getWidth()) * (float) bImage.getHeight();
                    iconHeight = Math.round(newHeight);
                } else {
                    iconHeight = 200;
                    float newWidth = ((float) iconHeight / (float) bImage.getHeight()) * (float) bImage.getWidth();
                    iconWidth = Math.round(newWidth);
                }
                // System.out.println("Setting icon image with width/height:" + iconWidth + ", " + iconHeight);

                // set label lImage to scaled icon
                lImage.setIcon(new ImageIcon(bImage.getScaledInstance(iconWidth, iconHeight, BufferedImage.SCALE_SMOOTH)));


            } catch (IOException ex) {
                Logger.getLogger(QuotationEditor.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else { // if the selected file is not an image, set null for Icon
            lImage.setIcon(null);
            bSetIcon.setEnabled(false);
            // System.out.println("File "+FileOperations.getFilePath(fileUUID));
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bFileOpen;
    private javax.swing.JButton bFileRemove;
    private javax.swing.JButton bFileSave;
    private javax.swing.JButton bIsPrivate;
    private javax.swing.JButton bSetIcon;
    private javax.swing.JCheckBox cbIsPrivate;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JLabel lFileDate;
    private javax.swing.JLabel lFileName;
    private javax.swing.JLabel lFileSize;
    private javax.swing.JLabel lImage;
    private javax.swing.JLabel lRevisions;
    private javax.swing.JLabel lUUID;
    private javax.swing.JLabel lUUIDValue;
    private javax.swing.JToggleButton tbMaxMin;
    private javax.swing.JTextField tfFileName;
    private javax.swing.JTextField tfTags;
    // End of variables declaration//GEN-END:variables

    /**
     * Sets the text of tag TextField (used by FileEditorSingle, to keep this and FileEditorSingle synchron
     * @param text
     */
    void setTagsText(String text) {
        tfTags.setText(text);
    }

    public void deleteFile() {
        dbModel.deleteObject(dbfile);
    }

    public void showButtons(boolean b) {
        bFileOpen.setVisible(b);
        bFileRemove.setVisible(b);
        bFileSave.setVisible(b);
        bSetIcon.setVisible(b);

        bFileOpen.setEnabled(b);
        bFileRemove.setEnabled(b);
        bFileSave.setEnabled(b);
        bSetIcon.setEnabled(b);
    }

    public void toggleMinMaxEditor() {
        tbMaxMin.doClick();
    }
}
