/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * NoteEditor.java
 *
 * Created on 26.07.2010, 00:03:14
 */
package net.geoadel.bibcite.GUI.ObjectEditors;

import java.awt.Component;
import net.geoadel.bibcite.GUI.ObjectListPanels.NoteEditorPanel;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.UUID;
import net.geoadel.bibcite.DatabaseObjects.*;
import java.awt.image.BufferedImage;
import java.io.File;
import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.GUI.AttributeEditors.AccessManagerGUI;
import net.geoadel.bibcite.GUI.AttributeEditors.RelationManager;
import net.geoadel.bibcite.GUI.ObjectListPanels.TextAreaPanel;

/**
 * This should be used, if NoteEditor is shown on its own in a NoteEditorPanel, not when it is embedded as view in other Editors (e.g. FileEditor)
 *
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class NoteEditor extends ObjectEditor {

    Note note;
    boolean isPrivate = false;
    BufferedImage bImage;
    File selectedFile;
    String fileSeparator = System.getProperty("file.separator");
    UUID uuid = UUID.randomUUID();
    UUID sourceUUID;

    boolean loading = true;
    DefaultListModel dlm = new DefaultListModel();
    NoteEditorPanel parentPanel;
    AccessManagerGUI accessManager;

    /**
     * Creates new form NoteEditor
     * e.g. if in BibliographyEditor Button AddNote was pressed
     */
    public NoteEditor(Note note) {
        this.note = note;
        // System.out.println("Loading revision of note: "+note.getFieldSummary());
        uuid = note.getUUID();

        initComponents();
        initData(note);

    }

    private void initData(Note note) {
        // load tags
        ArrayList<String> tags = note.getTags();
        String sTags = "";
        // System.out.println("Sorting through note tags:");
        for (int i = 0; i < tags.size(); i++) {
            // System.out.println(tags.get(i));
            sTags = sTags.concat(tags.get(i));
            if (i < tags.size() - 1) {
                sTags = sTags.concat(", ");
            }
        }
        tfTags.setText(sTags);
        tfTags.updateUI();
        tpNotes.setText(note.getNoteText());
        tfKeyValue.setText(note.getName());

        accessManager = new AccessManagerGUI(note);
        jTabbedPane1.add(new RelationManager(note), "Relations");

        final ComboBoxRevisionAccess cbRevisions = new ComboBoxRevisionAccess(dbModel.getRevisionsOf(note.getUUID()));
        cbRevisions.getModel().setSelectedItem(note);
        cbRevisions.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                initData((Note) cbRevisions.getSelectedItem());
            }
        });
        jToolBar1.add(cbRevisions);
        loading = false;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSeparator3 = new javax.swing.JSeparator();
        jToolBar1 = new javax.swing.JToolBar();
        tbMaxMin = new javax.swing.JToggleButton();
        tfKeyValue = new javax.swing.JTextField();
        jSeparator4 = new javax.swing.JToolBar.Separator();
        cbIsPrivate = new javax.swing.JCheckBox();
        jButton2 = new javax.swing.JButton();
        jSeparator2 = new javax.swing.JToolBar.Separator();
        bNoteSave = new javax.swing.JButton();
        bNoteErase = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JToolBar.Separator();
        lRevisions = new javax.swing.JLabel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        spBottom = new javax.swing.JScrollPane();
        panelBottom = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jScrollPane7 = new javax.swing.JScrollPane();
        tpNotes = new javax.swing.JTextPane();
        jLabel5 = new javax.swing.JLabel();
        tfTags = new javax.swing.JTextField();

        setBorder(javax.swing.BorderFactory.createEtchedBorder());
        setMaximumSize(new java.awt.Dimension(32771, 230));
        setMinimumSize(new java.awt.Dimension(575, 230));
        setPreferredSize(new java.awt.Dimension(575, 230));
        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.PAGE_AXIS));

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);
        jToolBar1.setMaximumSize(new java.awt.Dimension(32767, 27));
        jToolBar1.setMinimumSize(new java.awt.Dimension(500, 35));
        jToolBar1.setPreferredSize(new java.awt.Dimension(500, 27));

        tbMaxMin.setText("+");
        tbMaxMin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tbMaxMinActionPerformed(evt);
            }
        });
        jToolBar1.add(tbMaxMin);

        tfKeyValue.setText("");
        tfKeyValue.setPreferredSize(new java.awt.Dimension(150, 28));
        tfKeyValue.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tfKeyValueCaretUpdate(evt);
            }
        });
        jToolBar1.add(tfKeyValue);
        jToolBar1.add(jSeparator4);

        cbIsPrivate.setFocusable(false);
        cbIsPrivate.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        cbIsPrivate.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        cbIsPrivate.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                cbIsPrivateStateChanged(evt);
            }
        });
        jToolBar1.add(cbIsPrivate);

        jButton2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/privateKey24.png"))); // NOI18N
        jButton2.setToolTipText("private");
        jButton2.setBorderPainted(false);
        jButton2.setFocusPainted(false);
        jButton2.setFocusable(false);
        jButton2.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton2.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton2);
        jToolBar1.add(jSeparator2);

        bNoteSave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/noteSave.png"))); // NOI18N
        bNoteSave.setToolTipText("Save");
        bNoteSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bNoteSaveActionPerformed(evt);
            }
        });
        jToolBar1.add(bNoteSave);

        bNoteErase.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/noteDelete.png"))); // NOI18N
        bNoteErase.setToolTipText("Erase");
        bNoteErase.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bNoteEraseActionPerformed(evt);
            }
        });
        jToolBar1.add(bNoteErase);
        jToolBar1.add(jSeparator1);

        lRevisions.setText("Revisions");
        lRevisions.setPreferredSize(new java.awt.Dimension(70, 13));
        jToolBar1.add(lRevisions);

        add(jToolBar1);

        spBottom.setMinimumSize(new java.awt.Dimension(500, 150));
        spBottom.setPreferredSize(new java.awt.Dimension(500, 150));

        panelBottom.setMaximumSize(new java.awt.Dimension(32767, 400));
        panelBottom.setMinimumSize(new java.awt.Dimension(450, 150));
        panelBottom.setPreferredSize(new java.awt.Dimension(450, 150));

        jLabel1.setText("Notes");

        tpNotes.setName("note"); // NOI18N
        tpNotes.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                tpNotesMouseClicked(evt);
            }
        });
        tpNotes.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
            }
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                tpNotesCaretPositionChanged(evt);
            }
        });
        jScrollPane7.setViewportView(tpNotes);

        jLabel5.setText("Tags");

        tfTags.setToolTipText("Separate tags by commata");

        javax.swing.GroupLayout panelBottomLayout = new javax.swing.GroupLayout(panelBottom);
        panelBottom.setLayout(panelBottomLayout);
        panelBottomLayout.setHorizontalGroup(
            panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelBottomLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel1)
                    .addComponent(jLabel5))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane7, javax.swing.GroupLayout.DEFAULT_SIZE, 500, Short.MAX_VALUE)
                    .addComponent(tfTags, javax.swing.GroupLayout.DEFAULT_SIZE, 500, Short.MAX_VALUE))
                .addGap(14, 14, 14))
        );
        panelBottomLayout.setVerticalGroup(
            panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelBottomLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel1)
                    .addComponent(jScrollPane7, javax.swing.GroupLayout.DEFAULT_SIZE, 105, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel5, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(tfTags, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        spBottom.setViewportView(panelBottom);

        jTabbedPane1.addTab("Note", spBottom);

        add(jTabbedPane1);
    }// </editor-fold>//GEN-END:initComponents

    /**
     * if tpNotes was double clicked by mouse, open more comfortable instance of
     * TextAreaPanel for easier editing
     * @param evt
     */
    private void tpNotesMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tpNotesMouseClicked
        if (evt.isControlDown() & evt.getClickCount() > 1) {
            JFrame frame = new JFrame("Texteditor");
            TextAreaPanel tap = new TextAreaPanel();
            String text = tap.TextAreaPanel(tpNotes);
            frame.add(tap);
            frame.pack();
            frame.setVisible(true);
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            tpNotes.setText(text);
        }
    }//GEN-LAST:event_tpNotesMouseClicked

    private void bNoteEraseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bNoteEraseActionPerformed
        dbModel.deleteObject(uuid);
        this.setVisible(false);
}//GEN-LAST:event_bNoteEraseActionPerformed

    private void bNoteSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bNoteSaveActionPerformed
        storeNote();
}//GEN-LAST:event_bNoteSaveActionPerformed

    public void storeNote() {
        String[] tags = tfTags.getText().split(", ");
        ArrayList<String> arrayTags = new ArrayList<String>();
        arrayTags.addAll(Arrays.asList(tags));

        /**
         * get current date as specified SimpleDateFormat
         * denoting year (four digits), month (two digits), day of month (two digits),
         * hour of day (two digits), minutes of hour (two digits) and seconds (two digits)
         */
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        Date dt = new Date();
        Long dateEdited = Long.valueOf(format.format(dt));

        ArrayList<UUID> alRelated = note.getRelatedObjects();
        if (!alRelated.contains(sourceUUID) & sourceUUID != null) {
            alRelated.add(sourceUUID);
        } else {
            for (UUID uuid : alRelated) {
            }
        }

        Note note = new Note(uuid, alRelated, tfKeyValue.getText(), tfKeyValue.getText(), tpNotes.getText(), cbIsPrivate.isSelected(), arrayTags, dateEdited);
        dbModel.storeObject(note);
        dbModel.updateTree();
    }

    private void tbMaxMinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tbMaxMinActionPerformed
        toggleMinMaxEditor();
}//GEN-LAST:event_tbMaxMinActionPerformed

    private void tpNotesCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_tpNotesCaretPositionChanged
        bNoteSave.setEnabled(true);
    }//GEN-LAST:event_tpNotesCaretPositionChanged

    private void cbIsPrivateStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_cbIsPrivateStateChanged
        if (!cbIsPrivate.isSelected()) {
            jTabbedPane1.remove(accessManager);
        } else {
            jTabbedPane1.add(accessManager, "Access");
        }
        bNoteSave.setEnabled(true);
    }//GEN-LAST:event_cbIsPrivateStateChanged

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        cbIsPrivate.setSelected(!cbIsPrivate.isSelected());
    }//GEN-LAST:event_jButton2ActionPerformed

    private void tfKeyValueCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tfKeyValueCaretUpdate
        // System.out.println("tfKeyValue.getText() "+tfKeyValue.getText());
    }//GEN-LAST:event_tfKeyValueCaretUpdate

    public void setTagsText(String text) {
        tfTags.setText(text);
    }

    public void setNoteText(String text) {
        tpNotes.setText(text);
    }

    public void setIsPrivate(boolean isPrivate) {
        cbIsPrivate.setSelected(isPrivate);
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bNoteErase;
    private javax.swing.JButton bNoteSave;
    private javax.swing.JCheckBox cbIsPrivate;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JScrollPane jScrollPane7;
    private javax.swing.JToolBar.Separator jSeparator1;
    private javax.swing.JToolBar.Separator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JToolBar.Separator jSeparator4;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JLabel lRevisions;
    private javax.swing.JPanel panelBottom;
    private javax.swing.JScrollPane spBottom;
    private javax.swing.JToggleButton tbMaxMin;
    private javax.swing.JTextField tfKeyValue;
    private javax.swing.JTextField tfTags;
    private javax.swing.JTextPane tpNotes;
    // End of variables declaration//GEN-END:variables

    public void deleteNote() {
        // System.out.println("Erasing note "+note.getUUID());
        dbModel.deleteObject(uuid);
    }

    public void showButtons(boolean b) {
        bNoteErase.setVisible(b);
        bNoteSave.setVisible(b);
        bNoteErase.setEnabled(b);
        bNoteSave.setEnabled(b);
    }

    public UUID getSourceUUID() {
        return sourceUUID;
    }
}
