/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * PersonEditor.java
 *
 * Created on Jul 1, 2010, 12:22:10 PM
 */
package net.geoadel.bibcite.GUI.ObjectEditors;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import net.geoadel.bibcite.GUI.ObjectListPanels.NoteEditorPanel;
import net.geoadel.bibcite.GUI.ObjectListPanels.FileEditorPanel;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.DatabaseObjects.Person;
import net.geoadel.bibcite.DocType.DocumentTypeDateLong;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.GUI.AttributeEditors.AccessManagerGUI;
import net.geoadel.bibcite.GUI.AttributeEditors.RelationManager;
import net.geoadel.bibcite.Tools.BaseN;
import net.geoadel.bibcite.Tools.FileOperations;

/**
 * Provides a person editor
 * @author Georg Adelmann
 */
public class PersonEditor extends ObjectEditor {

    public Person person;
    Long dateBirth, dateDeath;
    String nameFirst, nameLast;
    String addressPostal, addressMail, placeBirth, placeCurrent, placeDeath, description, note;
    ArrayList<String> personKeyList;
    boolean loading = true;
    AccessManagerGUI accessManager;

    public PersonEditor(Person person) {
        this.person = person;

        initComponents();
        initData(person);
        initDefaultState();
        
        loading = false;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTabbedPane1 = new javax.swing.JTabbedPane();
        jScrollPane4 = new javax.swing.JScrollPane();
        jPanel2 = new javax.swing.JPanel();
        pBasics = new javax.swing.JPanel();
        tfName = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        tpDescription = new javax.swing.JTextPane();
        jLabel3 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        lImage = new javax.swing.JLabel();
        pAddress = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        tfMail = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        tfPlaceCurrent = new javax.swing.JTextField();
        jScrollPane5 = new javax.swing.JScrollPane();
        tpAddressPostal = new javax.swing.JTextPane();
        pBirthDearth = new javax.swing.JPanel();
        cbAlive = new javax.swing.JCheckBox();
        lBirthDate = new javax.swing.JLabel();
        tfBirthDate = new javax.swing.JTextField();
        lBirthPlace = new javax.swing.JLabel();
        tfBirthPlace = new javax.swing.JTextField();
        lDeathDate = new javax.swing.JLabel();
        lDeathPlace = new javax.swing.JLabel();
        tfDeathDate = new javax.swing.JTextField();
        tfDeathPlace = new javax.swing.JTextField();
        jToolBar1 = new javax.swing.JToolBar();
        jLabel9 = new javax.swing.JLabel();
        lKey = new javax.swing.JLabel();
        cbKeyLock = new javax.swing.JCheckBox();
        jButton2 = new javax.swing.JButton();
        jSeparator2 = new javax.swing.JToolBar.Separator();
        cbIsPrivate = new javax.swing.JCheckBox();
        jButton1 = new javax.swing.JButton();
        jSeparator3 = new javax.swing.JToolBar.Separator();
        jButton3 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();
        jSeparator4 = new javax.swing.JToolBar.Separator();
        jLabel10 = new javax.swing.JLabel();

        setPreferredSize(new java.awt.Dimension(667, 500));
        addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                formFocusGained(evt);
            }
        });

        jPanel2.setPreferredSize(new java.awt.Dimension(400, 430));

        tfName.setToolTipText("Name of person (e.g. Georg Adelmann)");
        tfName.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tfNameCaretUpdate(evt);
            }
        });

        tpDescription.setToolTipText("Description of person");
        jScrollPane2.setViewportView(tpDescription);

        jLabel3.setText("Name");

        jLabel7.setText("Description");

        lImage.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Person160.png"))); // NOI18N
        lImage.setToolTipText("Image of person");
        lImage.setMaximumSize(new java.awt.Dimension(130, 160));
        lImage.setMinimumSize(new java.awt.Dimension(130, 160));
        lImage.setPreferredSize(new java.awt.Dimension(130, 160));

        javax.swing.GroupLayout pBasicsLayout = new javax.swing.GroupLayout(pBasics);
        pBasics.setLayout(pBasicsLayout);
        pBasicsLayout.setHorizontalGroup(
            pBasicsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pBasicsLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(pBasicsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel3)
                    .addComponent(jLabel7))
                .addGap(26, 26, 26)
                .addGroup(pBasicsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(tfName)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 346, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(lImage, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        pBasicsLayout.setVerticalGroup(
            pBasicsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pBasicsLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(pBasicsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(lImage, javax.swing.GroupLayout.DEFAULT_SIZE, 169, Short.MAX_VALUE)
                    .addGroup(pBasicsLayout.createSequentialGroup()
                        .addGroup(pBasicsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(tfName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel3))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(pBasicsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(pBasicsLayout.createSequentialGroup()
                                .addComponent(jLabel7)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 127, Short.MAX_VALUE))
                            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 140, Short.MAX_VALUE))))
                .addContainerGap())
        );

        jLabel4.setText("Address");

        jLabel8.setText("Mail");

        tfMail.setToolTipText("electronic mail address");

        jLabel5.setText("Current Location");

        tfPlaceCurrent.setToolTipText("Current place");

        tpAddressPostal.setToolTipText("Postal address");
        jScrollPane5.setViewportView(tpAddressPostal);

        javax.swing.GroupLayout pAddressLayout = new javax.swing.GroupLayout(pAddress);
        pAddress.setLayout(pAddressLayout);
        pAddressLayout.setHorizontalGroup(
            pAddressLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pAddressLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(pAddressLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addComponent(jLabel8))
                .addGap(50, 50, 50)
                .addGroup(pAddressLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(pAddressLayout.createSequentialGroup()
                        .addComponent(tfMail, javax.swing.GroupLayout.PREFERRED_SIZE, 138, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel5)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(tfPlaceCurrent))
                    .addComponent(jScrollPane5, javax.swing.GroupLayout.PREFERRED_SIZE, 486, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pAddressLayout.setVerticalGroup(
            pAddressLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pAddressLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(pAddressLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addComponent(jScrollPane5, javax.swing.GroupLayout.PREFERRED_SIZE, 57, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(pAddressLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel8)
                    .addComponent(tfMail, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel5)
                    .addComponent(tfPlaceCurrent, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        cbAlive.setSelected(true);
        cbAlive.setText("is alive");
        cbAlive.setToolTipText("If unchecked, two additional fields come up, date & place of death");
        cbAlive.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                cbAliveStateChanged(evt);
            }
        });
        cbAlive.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbAliveItemStateChanged(evt);
            }
        });

        lBirthDate.setText("Date of birth");

        tfBirthDate.setToolTipText("<html><body>Date, at least indicating year of birth - but can be as finetuned as <p>YearMonthDayHourMinute, e.g. 199504301247</body></html>");
        tfBirthDate.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tfBirthDateCaretUpdate(evt);
            }
        });

        lBirthPlace.setText("Place of birth");

        tfBirthPlace.setToolTipText("Enter country or city, country or city, state");

        lDeathDate.setText("Date of death");

        lDeathPlace.setText("Place of death");

        tfDeathDate.setToolTipText("YearMonthDayHourMinute, e.g. 199504301247");

        tfDeathPlace.setToolTipText("enter country or city, country or city, state, country");

        javax.swing.GroupLayout pBirthDearthLayout = new javax.swing.GroupLayout(pBirthDearth);
        pBirthDearth.setLayout(pBirthDearthLayout);
        pBirthDearthLayout.setHorizontalGroup(
            pBirthDearthLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pBirthDearthLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(pBirthDearthLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(cbAlive)
                    .addGroup(pBirthDearthLayout.createSequentialGroup()
                        .addGroup(pBirthDearthLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lBirthDate)
                            .addComponent(lBirthPlace))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(pBirthDearthLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(tfBirthPlace)
                            .addComponent(tfBirthDate, javax.swing.GroupLayout.DEFAULT_SIZE, 188, Short.MAX_VALUE))
                        .addGap(18, 18, 18)
                        .addGroup(pBirthDearthLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lDeathPlace)
                            .addComponent(lDeathDate))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(pBirthDearthLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(tfDeathPlace, javax.swing.GroupLayout.DEFAULT_SIZE, 197, Short.MAX_VALUE)
                            .addComponent(tfDeathDate, javax.swing.GroupLayout.DEFAULT_SIZE, 197, Short.MAX_VALUE))))
                .addContainerGap())
        );
        pBirthDearthLayout.setVerticalGroup(
            pBirthDearthLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pBirthDearthLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(cbAlive)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(pBirthDearthLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lBirthDate)
                    .addComponent(tfBirthDate, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(tfDeathDate, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(lDeathDate))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(pBirthDearthLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lBirthPlace)
                    .addComponent(tfBirthPlace, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(lDeathPlace)
                    .addComponent(tfDeathPlace, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(17, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(pBasics, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(pAddress, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(pBirthDearth, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap(29, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addComponent(pBasics, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pAddress, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pBirthDearth, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jScrollPane4.setViewportView(jPanel2);

        jTabbedPane1.addTab("Person", jScrollPane4);

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);
        jToolBar1.add(jLabel9);

        lKey.setText("Key");
        lKey.setToolTipText("Person key, useful for referencing persons in external documents, e.g. LaTex or LibreOffice");
        lKey.setMinimumSize(new java.awt.Dimension(200, 13));
        lKey.setPreferredSize(new java.awt.Dimension(200, 13));
        jToolBar1.add(lKey);

        cbKeyLock.setToolTipText("<html>If selected, the person key is locked and will not be changed if you change relevant keys. <br> Important, if you want keys to be recognized over longer periods of time.</html>");
        cbKeyLock.setFocusable(false);
        cbKeyLock.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        cbKeyLock.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        cbKeyLock.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbKeyLockItemStateChanged(evt);
            }
        });
        jToolBar1.add(cbKeyLock);

        jButton2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/lock.png"))); // NOI18N
        jButton2.setBorderPainted(false);
        jButton2.setFocusPainted(false);
        jButton2.setFocusable(false);
        jButton2.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton2.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton2);

        jSeparator2.setSeparatorSize(new java.awt.Dimension(16, 12));
        jToolBar1.add(jSeparator2);

        cbIsPrivate.setToolTipText("set Person to Private (only you and people added on the Access-tab can then see and access the private revisions of this object)");
        cbIsPrivate.setFocusable(false);
        cbIsPrivate.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        cbIsPrivate.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        cbIsPrivate.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                cbIsPrivateStateChanged(evt);
            }
        });
        cbIsPrivate.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbIsPrivateItemStateChanged(evt);
            }
        });
        jToolBar1.add(cbIsPrivate);

        jButton1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/privateKey24.png"))); // NOI18N
        jButton1.setToolTipText("set private");
        jButton1.setBorderPainted(false);
        jButton1.setFocusPainted(false);
        jButton1.setFocusable(false);
        jButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton1);

        jSeparator3.setSeparatorSize(new java.awt.Dimension(15, 12));
        jToolBar1.add(jSeparator3);

        jButton3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/personSave.png"))); // NOI18N
        jButton3.setToolTipText("Save person");
        jButton3.setFocusable(false);
        jButton3.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton3.setMaximumSize(new java.awt.Dimension(30, 36));
        jButton3.setMinimumSize(new java.awt.Dimension(30, 36));
        jButton3.setPreferredSize(new java.awt.Dimension(30, 36));
        jButton3.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton3);

        jButton4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/personDelete.png"))); // NOI18N
        jButton4.setToolTipText("Delete person entry");
        jButton4.setFocusable(false);
        jButton4.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton4.setPreferredSize(new java.awt.Dimension(30, 36));
        jButton4.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jButton4);

        jSeparator4.setSeparatorSize(new java.awt.Dimension(15, 12));
        jToolBar1.add(jSeparator4);

        jLabel10.setText("Revision");
        jToolBar1.add(jLabel10);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 667, Short.MAX_VALUE)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 667, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 469, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    public void savePerson() {
        try {
            dateBirth = Long.valueOf(tfBirthDate.getText());
        } catch (java.lang.NumberFormatException nfe) {
            dateBirth = 0L;
        }
        try {
            dateDeath = Long.valueOf(tfDeathDate.getText());
        } catch (java.lang.NumberFormatException nfe) {
            dateDeath = 0L;
        }

        person = new Person(person.getUUID(), tfName.getText(), cbAlive.isSelected(), dateBirth, dateDeath, tpAddressPostal.getText(), tfMail.getText(), tpDescription.getText(), cbIsPrivate.isSelected(), SessionData.getUser(), lKey.getText());
        dbModel.storeObject(person);
    }

    public void erasePerson() {
        dbModel.deleteObject(person.getUUID());
    }

    private void cbAliveItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cbAliveItemStateChanged
        if (cbAlive.isSelected()) {
            tfDeathDate.setVisible(false);
            tfDeathPlace.setVisible(false);
            lDeathDate.setVisible(false);
            lDeathPlace.setVisible(false);
        } else {
            tfDeathDate.setVisible(true);
            tfDeathPlace.setVisible(true);
            lDeathDate.setVisible(true);
            lDeathPlace.setVisible(true);
        }
    }//GEN-LAST:event_cbAliveItemStateChanged

    private void tfNameCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tfNameCaretUpdate
        if (!cbKeyLock.isSelected()) {
            lKey.setText(BaseN.updateKeyValueAuthor(lKey.getText(), tfBirthDate.getText(), tfName.getText(), personKeyList));
            lKey.updateUI();
        }
    }//GEN-LAST:event_tfNameCaretUpdate

    private void tfBirthDateCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tfBirthDateCaretUpdate
        if (!cbKeyLock.isSelected()) {
            lKey.setText(BaseN.updateKeyValueAuthor(lKey.getText(), tfBirthDate.getText(), tfName.getText(), personKeyList));
            lKey.updateUI();
        }
    }//GEN-LAST:event_tfBirthDateCaretUpdate

    private void cbAliveStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_cbAliveStateChanged
    }//GEN-LAST:event_cbAliveStateChanged

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        cbIsPrivate.setSelected(!cbIsPrivate.isSelected());
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        cbKeyLock.setSelected(!cbKeyLock.isSelected());
    }//GEN-LAST:event_jButton2ActionPerformed

    private void cbKeyLockItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cbKeyLockItemStateChanged
    }//GEN-LAST:event_cbKeyLockItemStateChanged

    private void cbIsPrivateStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_cbIsPrivateStateChanged
    }//GEN-LAST:event_cbIsPrivateStateChanged

    private void cbIsPrivateItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cbIsPrivateItemStateChanged
        if (!cbIsPrivate.isSelected()) {
            jTabbedPane1.remove(accessManager);
        } else {
            if (accessManager == null) {
                accessManager = new AccessManagerGUI(person);
            }
            jTabbedPane1.add(accessManager, "Access");
        }
    }//GEN-LAST:event_cbIsPrivateItemStateChanged

    private void formFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_formFocusGained
        // System.out.println("PersonEditor gained focus");
        // new EditorEvents().showButtonsPersonEditor(this);
    }//GEN-LAST:event_formFocusGained

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        savePerson();
    }//GEN-LAST:event_jButton3ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox cbAlive;
    private javax.swing.JCheckBox cbIsPrivate;
    private javax.swing.JCheckBox cbKeyLock;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JToolBar.Separator jSeparator2;
    private javax.swing.JToolBar.Separator jSeparator3;
    private javax.swing.JToolBar.Separator jSeparator4;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JLabel lBirthDate;
    private javax.swing.JLabel lBirthPlace;
    private javax.swing.JLabel lDeathDate;
    private javax.swing.JLabel lDeathPlace;
    private javax.swing.JLabel lImage;
    private javax.swing.JLabel lKey;
    private javax.swing.JPanel pAddress;
    private javax.swing.JPanel pBasics;
    private javax.swing.JPanel pBirthDearth;
    private javax.swing.JTextField tfBirthDate;
    private javax.swing.JTextField tfBirthPlace;
    private javax.swing.JTextField tfDeathDate;
    private javax.swing.JTextField tfDeathPlace;
    private javax.swing.JTextField tfMail;
    private javax.swing.JTextField tfName;
    private javax.swing.JTextField tfPlaceCurrent;
    private javax.swing.JTextPane tpAddressPostal;
    private javax.swing.JTextPane tpDescription;
    // End of variables declaration//GEN-END:variables

    /**
     * Creates the default initialization state for the components (enabled, document types, etc...)
     */
    private void initData(Person person) {
        tfBirthDate.setDocument(new DocumentTypeDateLong());
        tfDeathDate.setDocument(new DocumentTypeDateLong());

        lDeathDate.setVisible(false);
        lDeathPlace.setVisible(false);
        tfDeathDate.setVisible(false);
        tfDeathPlace.setVisible(false);

        cbAlive.setSelected(true);

        // set standard icon
        try {
            lImage.setIcon(new ImageIcon(ImageIO.read(new File("src/net/geoadel/bibcite/Images/Person160.png"))));
        } catch (IOException ex) {
            Logger.getLogger(PersonEditor.class.getName()).log(Level.SEVERE, null, ex);
            ex.printStackTrace();
        }

        // if object has uuid, try to get its specific preview icon for displaying (if one exists)
        if (person.getUUID() != null) {
            File fIcon = FileOperations.getFile(person.getUUID(), person.getUUID() + "-medium.png");
            // System.out.println("Icon path: " + fIcon.getPath()+" exists: "+fIcon.exists());
            ImageIcon icon = null;
            if (fIcon != null && fIcon.exists()) {
                // System.out.println("Load it");
                try {
                    lImage.setIcon(new ImageIcon(ImageIO.read(fIcon)));
                    lImage.updateUI();
                    // System.out.println("Loaded image...");
                } catch (IOException ex) {
                    // Logger.getLogger(TreeViewBibliography.class.getName()).log(Level.SEVERE, null, ex);
                    ex.printStackTrace();
                }
            }
        }
        lImage.updateUI();


        lKey.setText(person.getKey());
        cbKeyLock.setSelected(true);
        if (person.getKey().equals("")) {
            cbKeyLock.setSelected(false);
        }

        tfName.setText(person.getName());
        tpDescription.setText(person.getDescription());

        tpAddressPostal.setText(person.getAddressPostal());
        tfMail.setText(person.getAddressMail());
        tfPlaceCurrent.setText(person.getPlaceCurrent());

        tfBirthDate.setText(String.valueOf(person.getDateBirth()));
        tfDeathDate.setText(String.valueOf(person.getDateDeath()));
        tfBirthPlace.setText(person.getPlaceBirth());
        tfDeathPlace.setText(person.getPlaceDeath());
        accessManager = new AccessManagerGUI(person);       
    }

    private void initDefaultState() {
        jTabbedPane1.add(new FileEditorPanel(person.getUUID()), "Files");
        jTabbedPane1.add(new NoteEditorPanel(person.getUUID()), "Notes");
        jTabbedPane1.add(new RelationManager(person), "Relations");
        
        final ComboBoxRevisionAccess cbRevisions = new ComboBoxRevisionAccess((dbModel.getRevisionsOf(person.getUUID())));
        cbRevisions.getModel().setSelectedItem(person);
        cbRevisions.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                initData((Person) cbRevisions.getSelectedItem());
            }
        });
        jToolBar1.add(cbRevisions);
    }
}
