/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * QuotationEditor.java
 *
 * Created on 06.06.2010, 21:41:31
 */
package net.geoadel.bibcite.GUI.ObjectEditors;

import java.awt.Component;
import net.geoadel.bibcite.GUI.ObjectListPanels.NoteEditorPanel;
import net.geoadel.bibcite.GUI.ObjectListPanels.FileEditorPanel;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import net.geoadel.bibcite.Database.db4o;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.UUID;
import javax.swing.DefaultComboBoxModel;
import net.geoadel.bibcite.DocType.*;
import net.geoadel.bibcite.DatabaseObjects.*;
import java.awt.image.BufferedImage;
import java.io.File;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.GUI.AttributeEditors.AccessManagerGUI;
import net.geoadel.bibcite.GUI.AttributeEditors.RelationManager;
import net.geoadel.bibcite.GUI.ObjectListPanels.TextAreaPanel;
import net.geoadel.bibcite.Tools.BaseN;
import net.geoadel.bibcite.Tools.ObjectTools;

/**
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class QuotationEditor extends ObjectEditor implements Comparable<QuotationEditor> {

    Quotation quote;
    boolean isPrivate = false;
    BufferedImage bImage;
    File selectedFile;
    String fileSeparator = System.getProperty("file.separator");
    UUID uuid = UUID.randomUUID();
    UUID bibUUID;
    boolean minimized = false;
    DefaultListModel dlm = new DefaultListModel();
    BibliographyEditor bibEditor;
    AccessManagerGUI accessManager;
    /**
     * Buttons for notes and files
     */
    JButton bNewFile = new JButton(new ImageIcon("images/buttons/fileNew.png"));
    JButton bNewNote = new JButton(new ImageIcon("images/buttons/noteNew.png"));

    /**
     * if the QuotationEditor is used to display an already existing quote, this method is called
     *
     * @param bibUUID       the Bibliography uuid, this quote is related to
     * @param quote         the quote to be displayed
     * @param alRevisions   the revisions of the quote (aka the family)
     * @param user          user that uses this program instance right now
     * @param bibEditor     reference to the BibliographyEditor, necessary for maximizing minimizing QuotationEditor and BibliographyEditor to adjust layout accordingly
     */
    public QuotationEditor(UUID bibUUID, Quotation quote, ArrayList<Long> alRevisions, BibliographyEditor bibEditor) {
        this.quote = quote;
        this.bibEditor = bibEditor;
        this.uuid = quote.getUUID();
        this.isPrivate = quote.isPrivate();
        this.bibUUID = bibUUID;

        accessManager = new AccessManagerGUI(quote);

        initComponents();
        initDefaultState();
        initData(quote);
    }

    /**
     * A quote editor panel for just viewing and not editing quotes - used when a quote was selected in TreeView
     * @param quote     the quote to be displayed
     * @param user      the current user
     */
    public QuotationEditor(Quotation quote) {
        this.quote = quote;

        this.uuid = quote.getUUID();

        initComponents();
        initDefaultState();
        initData(quote);

    }

    public void initDefaultState() {
        jTabbedPane1.add(new NoteEditorPanel(uuid), "Notes");
        jTabbedPane1.add(new FileEditorPanel(uuid), "Files");
        jTabbedPane1.add(new RelationManager(quote), "Relations");

        final ComboBoxRevisionAccess cbRevisions = new ComboBoxRevisionAccess(dbModel.getRevisionsOf(quote.getUUID()));
        cbRevisions.getModel().setSelectedItem(quote);
        cbRevisions.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                initData((Quotation) cbRevisions.getSelectedItem());
            }
        });
        jToolBar1.add(cbRevisions);
    }

    public void initData(Quotation quote) {
        this.isPrivate = quote.isPrivate();
        cbIsPrivate.setSelected(quote.isPrivate());

        tfStart.setDocument(new DocumentTypeEightDigit());
        tfEnd.setDocument(new DocumentTypeEightDigit());

        tfStart.setText(quote.getPageStart() + "");
        tfEnd.setText(quote.getPageEnd() + "");
        // TODO implement tags display
        ArrayList<String> tags = quote.getTags();
        String sTags = "";
        // System.out.println("Sorting through quote tags:");
        for (int i = 0; i < tags.size(); i++) {
            // System.out.println(tags.get(i));
            sTags = sTags.concat(tags.get(i));
            if (i < tags.size() - 1) {
                sTags = sTags.concat(", ");
            }
        }
        tfTags.setText(sTags);
        tfTags.updateUI();
        tpGist.setText(quote.getGist());
        tpQuotes.setText(quote.getQuote());

        tfEnd.setDocument(new DocumentTypeEightDigit());
        tfStart.setDocument(new DocumentTypeEightDigit());

        tfEnd.setText(quote.getPageEnd() + "");
        tfStart.setText(quote.getPageStart() + "");

        tbMaxMin.setVisible(false);
        bQuoteSave.setVisible(true);

        if (!tfKeyValue.getText().isEmpty()) {
            cbKeyFixed.setSelected(true);
        } else {
            updateKey();
        }

        accessManager = new AccessManagerGUI(quote);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jToolBar1 = new javax.swing.JToolBar();
        tbMaxMin = new javax.swing.JToggleButton();
        tfKeyValue = new javax.swing.JTextField();
        cbKeyFixed = new javax.swing.JCheckBox();
        bSetKeyLock = new javax.swing.JButton();
        jSeparator4 = new javax.swing.JToolBar.Separator();
        cbIsPrivate = new javax.swing.JCheckBox();
        bSetPrivate = new javax.swing.JButton();
        jSeparator3 = new javax.swing.JToolBar.Separator();
        bQuoteSave = new javax.swing.JButton();
        bQuoteErase = new javax.swing.JButton();
        jSeparator2 = new javax.swing.JToolBar.Separator();
        lRevisions = new javax.swing.JLabel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        spBottom = new javax.swing.JScrollPane();
        panelBottom = new javax.swing.JPanel();
        lGist = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        tpGist = new javax.swing.JTextPane();
        jLabel4 = new javax.swing.JLabel();
        jScrollPane5 = new javax.swing.JScrollPane();
        tpQuotes = new javax.swing.JTextPane();
        jLabel5 = new javax.swing.JLabel();
        tfTags = new javax.swing.JTextField();
        jSeparator1 = new javax.swing.JSeparator();
        lQuoteStart = new javax.swing.JLabel();
        tfStart = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        tfEnd = new javax.swing.JTextField();

        setBorder(javax.swing.BorderFactory.createEtchedBorder());
        setMaximumSize(new java.awt.Dimension(32771, 380));
        setMinimumSize(new java.awt.Dimension(200, 30));
        setPreferredSize(new java.awt.Dimension(600, 340));
        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.PAGE_AXIS));

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);
        jToolBar1.setMaximumSize(new java.awt.Dimension(32767, 27));
        jToolBar1.setMinimumSize(new java.awt.Dimension(243, 27));
        jToolBar1.setPreferredSize(new java.awt.Dimension(100, 27));

        tbMaxMin.setText("+");
        tbMaxMin.setToolTipText("Expand or minimize this quote editor");
        tbMaxMin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tbMaxMinActionPerformed(evt);
            }
        });
        jToolBar1.add(tbMaxMin);

        tfKeyValue.setToolTipText("Reference key, useful for tracking quotes in external documents like LaTex or LibreOffice");
        tfKeyValue.setPreferredSize(new java.awt.Dimension(180, 28));
        jToolBar1.add(tfKeyValue);

        cbKeyFixed.setToolTipText("If enabled, a lock is in place, preventing changes to the key by either changes to author or year. Only change this status manually if you are certain you want it to be changed.");
        cbKeyFixed.setFocusable(false);
        cbKeyFixed.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        cbKeyFixed.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        cbKeyFixed.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                cbKeyFixedStateChanged(evt);
            }
        });
        jToolBar1.add(cbKeyFixed);

        bSetKeyLock.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/lock.png"))); // NOI18N
        bSetKeyLock.setFocusable(false);
        bSetKeyLock.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bSetKeyLock.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bSetKeyLock.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bSetKeyLockActionPerformed(evt);
            }
        });
        jToolBar1.add(bSetKeyLock);
        jToolBar1.add(jSeparator4);

        cbIsPrivate.setSelected(quote.isPrivate());
        cbIsPrivate.setToolTipText("set Quote to private");
        cbIsPrivate.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbIsPrivateItemStateChanged(evt);
            }
        });
        jToolBar1.add(cbIsPrivate);

        bSetPrivate.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/privateKey24.png"))); // NOI18N
        bSetPrivate.setToolTipText("set Quote to private");
        bSetPrivate.setBorderPainted(false);
        bSetPrivate.setFocusPainted(false);
        bSetPrivate.setFocusable(false);
        bSetPrivate.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bSetPrivate.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bSetPrivate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bSetPrivateActionPerformed(evt);
            }
        });
        jToolBar1.add(bSetPrivate);

        jSeparator3.setSeparatorSize(new java.awt.Dimension(20, 12));
        jToolBar1.add(jSeparator3);

        bQuoteSave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/quoteSave.png"))); // NOI18N
        bQuoteSave.setToolTipText("Save quote");
        bQuoteSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bQuoteSaveActionPerformed(evt);
            }
        });
        jToolBar1.add(bQuoteSave);

        bQuoteErase.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/quoteDelete.png"))); // NOI18N
        bQuoteErase.setToolTipText("Delete quote");
        bQuoteErase.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bQuoteEraseActionPerformed(evt);
            }
        });
        jToolBar1.add(bQuoteErase);

        jSeparator2.setSeparatorSize(new java.awt.Dimension(20, 12));
        jToolBar1.add(jSeparator2);

        lRevisions.setText("Revisions");
        jToolBar1.add(lRevisions);

        add(jToolBar1);

        jTabbedPane1.setMinimumSize(new java.awt.Dimension(58, 310));
        jTabbedPane1.setPreferredSize(new java.awt.Dimension(514, 310));
        jTabbedPane1.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jTabbedPane1StateChanged(evt);
            }
        });

        panelBottom.setMinimumSize(new java.awt.Dimension(500, 260));
        panelBottom.setPreferredSize(new java.awt.Dimension(500, 260));

        lGist.setText("Gist");

        tpGist.setToolTipText("Summarize the quote in own words");
        tpGist.setName("gist"); // NOI18N
        tpGist.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                tpGistMouseClicked(evt);
            }
        });
        tpGist.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tpGistCaretUpdate(evt);
            }
        });
        jScrollPane3.setViewportView(tpGist);

        jLabel4.setText("Quote");

        tpQuotes.setToolTipText("The original quote");
        tpQuotes.setName("quote"); // NOI18N
        tpQuotes.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                tpQuotesMouseClicked(evt);
            }
        });
        tpQuotes.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tpQuotesCaretUpdate(evt);
            }
        });
        jScrollPane5.setViewportView(tpQuotes);

        jLabel5.setText("Tags");

        tfTags.setToolTipText("Tags separated by commata (e.g. tag1, tag2, tag3)");
        tfTags.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tfTagsCaretUpdate(evt);
            }
        });

        lQuoteStart.setText("Start:");

        tfStart.setToolTipText("Page on which the quote begins");
        tfStart.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tfStartCaretUpdate(evt);
            }
        });

        jLabel3.setText("End:");

        tfEnd.setToolTipText("Page on which the quote ends");
        tfEnd.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tfEndCaretUpdate(evt);
            }
        });

        javax.swing.GroupLayout panelBottomLayout = new javax.swing.GroupLayout(panelBottom);
        panelBottom.setLayout(panelBottomLayout);
        panelBottomLayout.setHorizontalGroup(
            panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelBottomLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addComponent(lGist)
                        .addComponent(jLabel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(jLabel5))
                    .addComponent(lQuoteStart))
                .addGap(18, 18, 18)
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(panelBottomLayout.createSequentialGroup()
                        .addComponent(tfStart, javax.swing.GroupLayout.PREFERRED_SIZE, 95, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(jLabel3)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(tfEnd, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jSeparator1, javax.swing.GroupLayout.DEFAULT_SIZE, 539, Short.MAX_VALUE)
                    .addComponent(tfTags, javax.swing.GroupLayout.DEFAULT_SIZE, 539, Short.MAX_VALUE)
                    .addComponent(jScrollPane5, javax.swing.GroupLayout.DEFAULT_SIZE, 539, Short.MAX_VALUE)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 539, Short.MAX_VALUE))
                .addContainerGap())
        );
        panelBottomLayout.setVerticalGroup(
            panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelBottomLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(lGist)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 58, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addComponent(jScrollPane5, javax.swing.GroupLayout.DEFAULT_SIZE, 99, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(tfTags, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelBottomLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lQuoteStart)
                    .addComponent(tfStart, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3)
                    .addComponent(tfEnd, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(19, 19, 19))
        );

        spBottom.setViewportView(panelBottom);

        jTabbedPane1.addTab("Quote", spBottom);

        add(jTabbedPane1);
    }// </editor-fold>//GEN-END:initComponents

    /**
     * if bQuoteSave was pressed, save the quotation in the database
     * @param evt
     */
    private void bQuoteSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bQuoteSaveActionPerformed
        saveQuotation();
    }//GEN-LAST:event_bQuoteSaveActionPerformed

    public void saveQuotation() {

        String[] tags = tfTags.getText().split(", ");
        ArrayList<String> arrayTags = new ArrayList<String>();
        arrayTags.addAll(Arrays.asList(tags));

        /**
         * get current date as specified SimpleDateFormat
         * denoting year (four digits), month (two digits), day of month (two digits),
         * hour of day (two digits), minutes of hour (two digits) and seconds (two digits)
         */
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        Date dt = new Date();
        Long dateEdited = Long.valueOf(format.format(dt));

        quote = new Quotation(uuid, bibUUID, arrayTags, tfKeyValue.getText(), tpQuotes.getText(), tpGist.getText(), tfStart.getText(), tfEnd.getText(), dateEdited, cbIsPrivate.isSelected());
        dbModel.storeObject(quote);
        bQuoteSave.setEnabled(false);
    }
    private void tfEndCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tfEndCaretUpdate
        // tfStartCaretUpdate(evt); // calling this would cause a loop of events

        updateKey();
        bQuoteSave.setEnabled(true);
}//GEN-LAST:event_tfEndCaretUpdate

    private void updateKey() {
        if (!cbKeyFixed.isSelected()) {
            if (!tfStart.getText().isEmpty()) {
                tfKeyValue.setText(tfStart.getText() + " - " + tfEnd.getText());
                tfKeyValue.setText(BaseN.updateKeyValueQuotation(tfKeyValue.getText(), uuid.toString(), dbModel.getObjectKeys()));
            } else {
                tfKeyValue.setText(BaseN.updateKeyValueQuotation(tfKeyValue.getText(), uuid.toString(), dbModel.getObjectKeys()));
            }
        }
    }

    /**
     * copy digits in tfStart to tfEnd - makes data input easier and faster
     * since tfStart should always be equal or lower than tfEnd
     * @param evt
     */
    private void tfStartCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tfStartCaretUpdate
        try {
            if (tfEnd.getText().equals("")) {
                tfEnd.setText(tfStart.getText());
            }
            if (Integer.valueOf(tfEnd.getText()) < Integer.valueOf(tfStart.getText())) {
                tfEnd.setText(tfStart.getText());
            }
        } catch (java.lang.NumberFormatException nfe) {
        }
        updateKey();
        bQuoteSave.setEnabled(true);
    }//GEN-LAST:event_tfStartCaretUpdate

    /**
     * if tpGist text is changed, adapt head label accordingly
     * @param evt
     */
    private void tpGistCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tpGistCaretUpdate

        bQuoteSave.setEnabled(true);
}//GEN-LAST:event_tpGistCaretUpdate

    /**
     * if JCheckBox cbIsPrivate selection status is changed, adjust boolean isPrivate
     * @param evt
     */
    private void cbIsPrivateItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cbIsPrivateItemStateChanged
        if (!cbIsPrivate.isSelected()) {
            jTabbedPane1.remove(accessManager);
            isPrivate = cbIsPrivate.isSelected();
        } else {
            jTabbedPane1.add(accessManager, "Access");
        }
        bQuoteSave.setEnabled(true);
    }//GEN-LAST:event_cbIsPrivateItemStateChanged

    /**
     * if tpQuotes was double clicked by mouse, open more comfortable instance of
     * TextAreaPanel for easier editing
     * @param evt
     */
    private void tpQuotesMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tpQuotesMouseClicked
        if (evt.isControlDown() & evt.getClickCount() > 1) {
            JFrame frame = new JFrame("Texteditor");
            TextAreaPanel tap = new TextAreaPanel();
            String text = tap.TextAreaPanel(tpQuotes);
            frame.add(tap);
            frame.pack();
            frame.setVisible(true);
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            tpQuotes.setText(text);
        }
    }//GEN-LAST:event_tpQuotesMouseClicked

    /**
     * if tpGist was double clicked by mouse, open more comfortable instance of
     * TextAreaPanel for easier editing
     * @param evt
     */
    private void tpGistMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tpGistMouseClicked
        if (evt.isControlDown() & evt.getClickCount() > 1) {
            JFrame frame = new JFrame("Texteditor");
            TextAreaPanel tap = new TextAreaPanel();
            String text = tap.TextAreaPanel(tpGist);
            frame.add(tap);
            frame.pack();
            frame.setVisible(true);
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            tpGist.setText(text);
        }
    }//GEN-LAST:event_tpGistMouseClicked

    private void bQuoteEraseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bQuoteEraseActionPerformed
        dbModel.deleteObject(uuid);
        this.setVisible(false);
    }//GEN-LAST:event_bQuoteEraseActionPerformed

    private void tfTagsCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tfTagsCaretUpdate
        bQuoteSave.setEnabled(true);
    }//GEN-LAST:event_tfTagsCaretUpdate

    private void jTabbedPane1StateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jTabbedPane1StateChanged
        if (jTabbedPane1.getSelectedComponent().getName() != null) {
            if (jTabbedPane1.getSelectedComponent().getName().equals("Quote")) {
                //jToolBar1.add(bNewQuote, 4);
            }
            if (jTabbedPane1.getSelectedComponent().getName().equals("Notes")) {
                jToolBar1.add(bNewNote, 4);
            }
            if (jTabbedPane1.getSelectedComponent().getName().equals("Files")) {
                jToolBar1.add(bNewFile, 4);
            }
        }
    }//GEN-LAST:event_jTabbedPane1StateChanged

    private void bSetPrivateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bSetPrivateActionPerformed
        cbIsPrivate.setSelected(!cbIsPrivate.isSelected());
    }//GEN-LAST:event_bSetPrivateActionPerformed

    private void cbKeyFixedStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_cbKeyFixedStateChanged
        tfKeyValue.setEditable(!cbKeyFixed.isSelected());
        bQuoteSave.setEnabled(true);
}//GEN-LAST:event_cbKeyFixedStateChanged

    private void bSetKeyLockActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bSetKeyLockActionPerformed
        cbKeyFixed.doClick();
    }//GEN-LAST:event_bSetKeyLockActionPerformed

    private void tpQuotesCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tpQuotesCaretUpdate
        bQuoteSave.setEnabled(true);
    }//GEN-LAST:event_tpQuotesCaretUpdate

    private void tbMaxMinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tbMaxMinActionPerformed
        toggleMinMaxEditor();
    }//GEN-LAST:event_tbMaxMinActionPerformed

    // Allow the quotation editors to be sorted according to the containing quotation page start and end
    public int compareTo(QuotationEditor q) {
        int retValue = 0;
        try {
            if (Integer.valueOf(this.quote.getPageStart()) < Integer.valueOf(q.quote.getPageStart())) {
                retValue = -1;
            }
            if (Integer.valueOf(this.quote.getPageStart()) > Integer.valueOf(q.quote.getPageStart())) {
                retValue = 1;
            }
            if (Integer.valueOf(this.quote.getPageStart()) == Integer.valueOf(q.quote.getPageStart())) {
                if (Integer.valueOf(this.quote.getPageEnd()) < Integer.valueOf(q.quote.getPageEnd())) {
                    retValue = -1;
                }
                if (Integer.valueOf(this.quote.getPageEnd()) > Integer.valueOf(q.quote.getPageEnd())) {
                    retValue = 1;
                }
                if (Integer.valueOf(this.quote.getPageEnd()) == Integer.valueOf(q.quote.getPageEnd())) {
                    if (this.quote.getDateEdited() < q.quote.getDateEdited()) {
                        retValue = -1;
                    } else {
                        retValue = 1;
                    }
                }
            }
        } catch (NumberFormatException nfe) {
            retValue = -1;
        }

        return retValue;
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bQuoteErase;
    public javax.swing.JButton bQuoteSave;
    private javax.swing.JButton bSetKeyLock;
    private javax.swing.JButton bSetPrivate;
    private javax.swing.JCheckBox cbIsPrivate;
    private javax.swing.JCheckBox cbKeyFixed;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JToolBar.Separator jSeparator2;
    private javax.swing.JToolBar.Separator jSeparator3;
    private javax.swing.JToolBar.Separator jSeparator4;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JLabel lGist;
    private javax.swing.JLabel lQuoteStart;
    private javax.swing.JLabel lRevisions;
    private javax.swing.JPanel panelBottom;
    private javax.swing.JScrollPane spBottom;
    private javax.swing.JToggleButton tbMaxMin;
    private javax.swing.JTextField tfEnd;
    private javax.swing.JTextField tfKeyValue;
    private javax.swing.JTextField tfStart;
    private javax.swing.JTextField tfTags;
    private javax.swing.JTextPane tpGist;
    private javax.swing.JTextPane tpQuotes;
    // End of variables declaration//GEN-END:variables
}
