/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/*
 * FileEditorPanel.java
 *
 * Created on Aug 9, 2010, 2:43:40 PM
 */
package net.geoadel.bibcite.GUI.ObjectListPanels;

import java.awt.Component;
import java.awt.Desktop;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.UUID;
import net.geoadel.bibcite.DatabaseObjects.dbFile;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.GUI.ObjectEditors.FileEditor;
import net.geoadel.bibcite.Tools.ObjectTools;

/**
 * Contains a JPanel to display many fileEditors at once top-down
 * @author Georg Adelmann
 */
public class FileEditorPanel extends javax.swing.JPanel {

    private static UUID sourceUUID;

    /** Creates new form FileEditorPanel */
    public FileEditorPanel(UUID sourceUUID) {
        FileEditorPanel.sourceUUID = sourceUUID;

        initComponents();

        ArrayList<dbFile> alFiles = dbModel.getFilesRelatedToUUID(sourceUUID);
        
        ArrayList<UUID> families = ObjectTools.filterFamiliesByUUID(ObjectTools.fileToDBObject(alFiles));
        Iterator<UUID> iFamilies = families.iterator();

        // ArrayList<Bibliography> newestFamilyMembers = new ArrayList<Bibliography>();
        while (iFamilies.hasNext()) {
            UUID quoteUUID = iFamilies.next();
            ArrayList<dbFile> family = new ArrayList<dbFile>();
            ArrayList<Long> alRevisions = new ArrayList<Long>();
            // find all members of a family and put them in ArrayList family
            for (int i = 0; i < alFiles.size(); i++) {
                if (alFiles.get(i).getUUID().equals(quoteUUID)) {
                    family.add(alFiles.get(i));
                    alRevisions.add(alFiles.get(i).getDateEdited());
                }
            }

            // now filter these for the newest entry
            int iYoungest = ObjectTools.filterNewest(ObjectTools.fileToDBObject(family));
            panelFiles.add(new FileEditor(family.get(iYoungest)));
        }
        panelFiles.updateUI();

        // if Desktop is not supported by Java Desktop class (e.g. KDE but a lot of others as well), disable the bCreateNewFile button
        if (!Desktop.isDesktopSupported()) {
            bCreateFileNew.setEnabled(false);
            // bCreateFileNew.setTool
        }
    }

    /** Creates new form FileEditorPanel */
    public FileEditorPanel(dbFile file) {
        sourceUUID = file.getUUID();

        initComponents();

        // System.out.println("Testing FileEditorPanel");
        // System.out.println(file.getName());
        // System.out.println(file.getUUID());
        // System.out.println(SessionData.getUser().getUUID());
        panelFiles.add(new FileEditor(file));

        panelFiles.updateUI();

        // if Desktop is not supported by Java Desktop class (e.g. KDE but a lot of others as well), disable the bCreateNewFile button
        if (!Desktop.isDesktopSupported()) {
            bCreateFileNew.setEnabled(false);
            // bCreateFileNew.setTool
        }
    }

    public UUID getSourceUUID() {
        return sourceUUID;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        bCreateFileNew = new javax.swing.JButton();
        jToolBar1 = new javax.swing.JToolBar();
        bAddFile = new javax.swing.JButton();
        bMinMax = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        panelFiles = new javax.swing.JPanel();

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 527, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 494, Short.MAX_VALUE)
        );

        bCreateFileNew.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/fileNew.png"))); // NOI18N
        bCreateFileNew.setToolTipText("Create new file");
        bCreateFileNew.setEnabled(false);
        bCreateFileNew.setFocusable(false);
        bCreateFileNew.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bCreateFileNew.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bCreateFileNew.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bCreateFileNewActionPerformed(evt);
            }
        });

        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.PAGE_AXIS));

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);
        jToolBar1.setMaximumSize(new java.awt.Dimension(32767, 27));
        jToolBar1.setMinimumSize(new java.awt.Dimension(0, 27));
        jToolBar1.setPreferredSize(new java.awt.Dimension(74, 26));

        bAddFile.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/fileImport.png"))); // NOI18N
        bAddFile.setToolTipText("Import file");
        bAddFile.setFocusable(false);
        bAddFile.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bAddFile.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bAddFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bAddFileActionPerformed(evt);
            }
        });
        jToolBar1.add(bAddFile);

        bMinMax.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/editorSize.png"))); // NOI18N
        bMinMax.setToolTipText("Toggle editor size");
        bMinMax.setFocusable(false);
        bMinMax.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bMinMax.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bMinMax.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bMinMaxActionPerformed(evt);
            }
        });
        jToolBar1.add(bMinMax);

        add(jToolBar1);

        panelFiles.setLayout(new javax.swing.BoxLayout(panelFiles, javax.swing.BoxLayout.PAGE_AXIS));
        jScrollPane1.setViewportView(panelFiles);

        jScrollPane2.setViewportView(jScrollPane1);

        add(jScrollPane2);
    }// </editor-fold>//GEN-END:initComponents

    private void bAddFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bAddFileActionPerformed
        dbFile f = dbModel.importFile(sourceUUID);
        if (f != null) {
            FileEditor fileEditor = new FileEditor(f);
            addFileEditor(fileEditor);
        }
}//GEN-LAST:event_bAddFileActionPerformed

    private void bCreateFileNewActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bCreateFileNewActionPerformed
        dbFile file = dbModel.createFile(sourceUUID);
        if (file != null) {
            ArrayList<UUID> alRelations = new ArrayList<UUID>();
            alRelations.add(sourceUUID);
            dbFile dbfile = dbModel.importFile(sourceUUID);
            dbfile.addRelatedObject(sourceUUID);
            FileEditor fileEditor = new FileEditor(dbfile);
            addFileEditor(fileEditor);
        }
    }//GEN-LAST:event_bCreateFileNewActionPerformed

    private void bMinMaxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bMinMaxActionPerformed
        minMaxFileEditors();
}//GEN-LAST:event_bMinMaxActionPerformed

    public void addFileEditor(FileEditor fileEditor) {
        panelFiles.add(fileEditor);
        panelFiles.updateUI();
    }

    public void minMaxFileEditors() {
        for (Component component : panelFiles.getComponents()) {
            if (component instanceof FileEditor) {
                ((FileEditor) component).toggleMinMaxEditor();
            }
        }
    }

    public void revalidateFilePanel() {
        revalidate();
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bAddFile;
    private javax.swing.JButton bCreateFileNew;
    private javax.swing.JButton bMinMax;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JPanel panelFiles;
    // End of variables declaration//GEN-END:variables
}
