/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/*
 * NoteEditorPanel.java
 *
 * Created on Jul 28, 2010, 5:09:00 PM
 */
package net.geoadel.bibcite.GUI.ObjectListPanels;

import java.awt.Component;
import java.util.ArrayList;
import java.util.UUID;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.DatabaseObjects.Note;
import net.geoadel.bibcite.GUI.ObjectEditors.NoteEditor;


/**
 * Provides a JPanel able to display several note editors in a column
 * @author Georg Adelmann
 */
public class NoteEditorPanel extends javax.swing.JPanel {

    private UUID sourceUUID; // uuid of the object, that this noteeditorpanel belongs to

    /** Creates new form NoteEditorPanel for displaying all notes related to uuid */
    public NoteEditorPanel(UUID sourceUUID) {
        // System.out.println("NoteEditorPanel for " + sourceUUID);
        this.sourceUUID = sourceUUID;

        initComponents();
        ArrayList<Note> notes = dbModel.getNotesNewestVersions();
        for (Note note : notes) {
            panelNotes.add(new NoteEditor(note));
        }

        panelNotes.updateUI();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jToolBar1 = new javax.swing.JToolBar();
        bAddNote = new javax.swing.JButton();
        bMinMax = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        panelNotes = new javax.swing.JPanel();

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);

        bAddNote.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/noteNew.png"))); // NOI18N
        bAddNote.setToolTipText("Add note");
        bAddNote.setFocusable(false);
        bAddNote.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bAddNote.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bAddNote.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bAddNoteActionPerformed(evt);
            }
        });
        jToolBar1.add(bAddNote);

        bMinMax.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/editorSize.png"))); // NOI18N
        bMinMax.setToolTipText("Toggle Min Max");
        bMinMax.setFocusable(false);
        bMinMax.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bMinMax.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bMinMax.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bMinMaxActionPerformed(evt);
            }
        });
        jToolBar1.add(bMinMax);

        panelNotes.setLayout(new javax.swing.BoxLayout(panelNotes, javax.swing.BoxLayout.PAGE_AXIS));
        jScrollPane1.setViewportView(panelNotes);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 433, Short.MAX_VALUE)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 433, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void bAddNoteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bAddNoteActionPerformed
        Note note = new Note();
        note.addRelatedObject(sourceUUID);
        NoteEditor noteEditor = new NoteEditor(note);
        // System.out.println("Added new NoteEditor for " + getSourceUUID() + " = " + noteEditor.getSourceUUID());
        panelNotes.add(noteEditor);
        panelNotes.updateUI();
        revalidateNotePanel();
    }//GEN-LAST:event_bAddNoteActionPerformed

    private void bMinMaxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bMinMaxActionPerformed
        minMaxNoteEditorsTest();
}//GEN-LAST:event_bMinMaxActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bAddNote;
    private javax.swing.JButton bMinMax;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JPanel panelNotes;
    // End of variables declaration//GEN-END:variables

    public void revalidateNotePanel() {
        panelNotes.revalidate();
    }

    public UUID getSourceUUID() {
        // System.out.println("Returning " + sourceUUID);
        return sourceUUID;
    }

    private void minMaxNoteEditorsTest() {
        for (Component component : panelNotes.getComponents()) {
            if (component instanceof NoteEditor) {
                ((NoteEditor) component).toggleMinMaxEditor();
            }
        }
    }
}
