/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * QuotationEditorPanel.java
 *
 * Created on Aug 7, 2010, 3:59:30 PM
 */
package net.geoadel.bibcite.GUI.ObjectListPanels;

import java.awt.Component;
import net.geoadel.bibcite.GUI.ObjectEditors.QuotationEditor;
import net.geoadel.bibcite.GUI.ObjectEditors.BibliographyEditor;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.FlavorEvent;
import java.awt.datatransfer.FlavorListener;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import net.geoadel.bibcite.DatabaseObjects.Quotation;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.Tools.ObjectTools;

/**
 *
 * @author Georg Adelmann
 */
public class QuotationEditorPanel extends javax.swing.JPanel {

    BibliographyEditor bibEditor;
    UUID sourceUUID;

    ArrayList<QuotationEditor> editors = new ArrayList<QuotationEditor>();

    /** Creates new form QuotationEditorPanel */
    public QuotationEditorPanel(BibliographyEditor bibEditor, UUID sourceUUID) {
        this.bibEditor = bibEditor;
        this.sourceUUID = sourceUUID;
        initComponents();

        ArrayList<Quotation> alQuotes = dbModel.getBibliographyQuotes(bibEditor.bib);
        Collections.sort(alQuotes);

        ArrayList<UUID> families = ObjectTools.filterFamiliesByUUID(ObjectTools.quoteToDBObject(alQuotes));
        Iterator<UUID> iFamilies = families.iterator();



        // ArrayList<Bibliography> newestFamilyMembers = new ArrayList<Bibliography>();
        while (iFamilies.hasNext()) {
            UUID quoteUUID = iFamilies.next();
            ArrayList<Quotation> family = new ArrayList<Quotation>();
            ArrayList<Long> alRevisions = new ArrayList<Long>();
            // find all members of a family and put them in ArrayList family
            for (int i = 0; i < alQuotes.size(); i++) {
                if (alQuotes.get(i).getUUID().equals(quoteUUID)) {
                    family.add(alQuotes.get(i));
                    alRevisions.add(alQuotes.get(i).getDateEdited());
                }
            }

            // now filter these for the newest entry
            int iYoungest = ObjectTools.filterNewest(ObjectTools.quoteToDBObject(family));
            QuotationEditor editor = new QuotationEditor(sourceUUID, family.get(iYoungest), alRevisions, bibEditor);
            editors.add(editor);
        }

        // sort the quotation editors
        Collections.sort(editors);

        for (QuotationEditor editor : editors) {
            panelQuotes.add(editor);
        }

        panelQuotes.updateUI();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jToolBar1 = new javax.swing.JToolBar();
        bAddQuote = new javax.swing.JButton();
        tbClipboardScan = new javax.swing.JToggleButton();
        bMinMax = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        panelQuotes = new javax.swing.JPanel();

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);
        jToolBar1.setMinimumSize(new java.awt.Dimension(118, 50));
        jToolBar1.setPreferredSize(new java.awt.Dimension(110, 32));

        bAddQuote.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/quoteNew.png"))); // NOI18N
        bAddQuote.setToolTipText("Add quote");
        bAddQuote.setFocusable(false);
        bAddQuote.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bAddQuote.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bAddQuote.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bAddQuoteActionPerformed(evt);
            }
        });
        jToolBar1.add(bAddQuote);

        tbClipboardScan.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/quoteScan.png"))); // NOI18N
        tbClipboardScan.setToolTipText("Scan clipboard for quotes");
        tbClipboardScan.setFocusable(false);
        tbClipboardScan.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        tbClipboardScan.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        tbClipboardScan.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tbClipboardScanActionPerformed(evt);
            }
        });
        jToolBar1.add(tbClipboardScan);

        bMinMax.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/editorSize.png"))); // NOI18N
        bMinMax.setToolTipText("Toggle editor size");
        bMinMax.setFocusable(false);
        bMinMax.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bMinMax.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bMinMax.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bMinMaxActionPerformed(evt);
            }
        });
        jToolBar1.add(bMinMax);

        panelQuotes.setLayout(new javax.swing.BoxLayout(panelQuotes, javax.swing.BoxLayout.PAGE_AXIS));
        jScrollPane2.setViewportView(panelQuotes);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 527, Short.MAX_VALUE)
            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 527, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 473, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Button bAddQuote was pressed, delegated to EditorEvents
     */
    private void bAddQuoteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bAddQuoteActionPerformed
        dbModel.createQuoteEditor(this);
}//GEN-LAST:event_bAddQuoteActionPerformed

    /**
     * Called by EditorEvents, to give this quotationEditorPanel the
     * hint to create a new quotationEditor
     */
    public void addQuotation() {
        QuotationEditor qEditor = new QuotationEditor(sourceUUID, new Quotation(), new ArrayList<Long>(), bibEditor);
        panelQuotes.add(qEditor);
        panelQuotes.updateUI();
        qEditor.revalidate();
        panelQuotes.revalidate();
        jScrollPane2.revalidate();
    }
    private void tbClipboardScanActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tbClipboardScanActionPerformed
        // System.out.println("Started looking for quotes in the clipboard");

        final Clipboard sysClip = Toolkit.getDefaultToolkit().getSystemClipboard();

        /**
         * Create a listener for changes to the clipboard
         * While Java does not allow for listening to general changes easily,
         * it allows for listening for changes in the type of information stored (e.g. from text to image, etc.)
         *
         * I use it but at the end overwrite the clipboard content so that afterwards the next text
         * copy is again noticed
         */
        FlavorListener fl = new FlavorListener() {

            public void flavorsChanged(FlavorEvent e) {

                Transferable contentClipboard = sysClip.getContents(null);
                if (contentClipboard.isDataFlavorSupported(DataFlavor.stringFlavor)) {
                    try {
                        String content = (String) (contentClipboard.getTransferData(DataFlavor.stringFlavor));
                        // System.out.println(content);
                        // System.out.println();
                        // look for numbers followed either by text or by whitespace minus whitespace and numbers and then text
                        // System.out.println(content.matches("([0-9]+|[0-9]+\\s-\\s[0-9]+)\\s\\w+.*"));
                        Pattern pattern = Pattern.compile("([0-9]+|[0-9]+\\s-\\s[0-9]+)\\s\"\\w+.*?\"", Pattern.DOTALL);
                        Matcher matcher = pattern.matcher(content);
                        /**
                         * For each match, try to add it as a quotationEditor and save the quote
                         */
                        while (matcher.find()) {
                            String quote = matcher.group();
                            // System.out.println("");
                            // System.out.println(" NEXT QUOTE ");
                            // System.out.println(group);
                            String pages = quote.substring(0, quote.indexOf("\""));

                            int pageStart = 0;
                            int pageEnd = 0;
                            /**
                             * Find the start and end page of the quotation
                             */
                            if (pages.contains(" - ")) { // if text contains -, there is a start and end page
                                String[] pageArray = pages.split(" - ");
                                pageStart = Integer.parseInt(pageArray[0].trim());
                                pageEnd = Integer.parseInt(pageArray[1].trim());
                            } else { // else there is only one page number used to be for start and end
                                pageStart = Integer.parseInt(pages.trim());
                                pageEnd = pageStart;
                            }

                            String quoteContent = quote.substring(quote.indexOf("\"") + 1, quote.lastIndexOf("\""));

                            Quotation quotEdit = new Quotation();
                            quotEdit.setQuote(quoteContent);
                            quotEdit.addRelatedObject(sourceUUID);
                            quotEdit.setPageStart(String.valueOf(pageStart));
                            quotEdit.setPageEnd(String.valueOf(pageEnd));
                            QuotationEditor quoteEditor = new QuotationEditor(sourceUUID, quotEdit, new ArrayList<Long>(), bibEditor);

                            // add the quotationEditor to the panel containing all the quotation editors
                            panelQuotes.add(quoteEditor);

                            // store the quote in the database
                            quoteEditor.saveQuotation();

                            panelQuotes.updateUI();
                            quoteEditor.revalidate();
                            panelQuotes.revalidate();
                            jScrollPane2.revalidate();

                            // System.out.println("Added quote " + quoteContent);
                        }
                    } catch (UnsupportedFlavorException ex) {
                        Logger.getLogger(QuotationEditorPanel.class.getName()).log(Level.SEVERE, null, ex);
                    } catch (IOException ex) {
                        Logger.getLogger(QuotationEditorPanel.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }

                // basically sets a new flavor so that flavorsChanged works
                // again next time something is copied into the clipboard
                StringSelection test = new StringSelection("");
                sysClip.setContents(test, null);
            }
        };

        if (tbClipboardScan.isSelected()) {
            // System.out.println("Looking for clipboard changes...");
            sysClip.addFlavorListener(fl);

        } else {
            // System.out.println("Stopped looking for clipboard changes!");
            sysClip.removeFlavorListener(fl); // for some reason does not work
            while (sysClip.getFlavorListeners().length > 0) { // this works
                sysClip.removeFlavorListener(sysClip.getFlavorListeners()[0]);
            }
        }
        // basically sets a new flavor so that flavorsChanged works
        // again next time something is copied into the clipboard
        StringSelection test = new StringSelection("");
        sysClip.setContents(test, null);
    }//GEN-LAST:event_tbClipboardScanActionPerformed

    private void bMinMaxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bMinMaxActionPerformed
        minMaxQuoteEditors();
    }//GEN-LAST:event_bMinMaxActionPerformed

    public void revalidateQuotePanel() {
        panelQuotes.revalidate();
    }

    /**
     * toggle the minMax buttons of all quotation editors on the JPanel panelQuotes, to minimize / maximize them
     */
    public void minMaxQuoteEditors() {
        for (Component component : panelQuotes.getComponents()) {
            if (component instanceof QuotationEditor) {
                ((QuotationEditor) component).toggleMinMaxEditor();
            }
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bAddQuote;
    private javax.swing.JButton bMinMax;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JPanel panelQuotes;
    private javax.swing.JToggleButton tbClipboardScan;
    // End of variables declaration//GEN-END:variables
}
