    /* The program is supposed to work as a bibliography library exporting and importing to bibTex format
Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * PreferencesUserSettings.java
 *
 * Created on Aug 24, 2010, 10:45:14 PM
 */
package net.geoadel.bibcite.GUI.Preferences;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.swing.DefaultComboBoxModel;
import net.geoadel.bibcite.DataFields.Languages;
import net.geoadel.bibcite.DataFields.SessionData;

/**
 *
 * @author Georg Adelmann
 */
public class PreferencesUserSettings extends javax.swing.JPanel {

    /** Creates new form PreferencesUserSettings */
    public PreferencesUserSettings() {
        initComponents();
        ArrayList<String> languages = Languages.getLanguages();
        if (!languages.contains(SessionData.getUser().getPreferredLanguage())) {
            languages.add(SessionData.getUser().getPreferredLanguage());
        }
        cbLanguage.setModel(new DefaultComboBoxModel(languages.toArray()));
        cbLanguage.setEditable(true);
        cbLanguage.setSelectedItem(SessionData.getUser().getPreferredLanguage());


        ArrayList<String> supportedOperatingSystems = new ArrayList<String>();
        supportedOperatingSystems.add(""); // add an empty as field 0 to allow easy adding of new OS
        for (String s : net.geoadel.bibcite.DataFields.OperatingSystems.supported) {
            // to avoid double entries
            if (!supportedOperatingSystems.contains(s.toLowerCase())) {
                supportedOperatingSystems.add(s.toLowerCase());
            }
        }

        if (!supportedOperatingSystems.contains(SessionData.getUser().getDefaultOperatingSystem())) {
            supportedOperatingSystems.add(SessionData.getUser().getDefaultOperatingSystem());
        }

        cbOS.setModel(new DefaultComboBoxModel(supportedOperatingSystems.toArray()));

        if (SessionData.getUser().getDefaultOperatingSystem() != null) {
            cbOS.setSelectedItem(SessionData.getUser().getDefaultOperatingSystem());
        } else {
            cbOS.setSelectedIndex(0);
        }


        bPasswordNew.setEnabled(false);
        pfNew.setEnabled(false);
        pfNewRepeat.setEnabled(false);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        pfCurrent = new javax.swing.JPasswordField();
        jLabel4 = new javax.swing.JLabel();
        pfNew = new javax.swing.JPasswordField();
        jLabel5 = new javax.swing.JLabel();
        pfNewRepeat = new javax.swing.JPasswordField();
        bPasswordNew = new javax.swing.JButton();
        cbPasswordNew = new javax.swing.JCheckBox();
        cbPasswordNewRepeat = new javax.swing.JCheckBox();
        jPanel2 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        cbLanguage = new javax.swing.JComboBox();
        jPanel3 = new javax.swing.JPanel();
        cbOS = new javax.swing.JComboBox();
        jLabel7 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        cbFileType = new javax.swing.JComboBox();
        jLabel8 = new javax.swing.JLabel();
        tfProgram = new javax.swing.JTextField();
        jButton1 = new javax.swing.JButton();

        jLabel2.setText("<html><b>Change password</b></html>");

        jLabel3.setText("Current password");

        pfCurrent.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                pfCurrentCaretUpdate(evt);
            }
        });

        jLabel4.setText("New password");

        pfNew.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                pfNewCaretUpdate(evt);
            }
        });

        jLabel5.setText("Repeat");

        pfNewRepeat.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                pfNewRepeatCaretUpdate(evt);
            }
        });

        bPasswordNew.setText("Confirm changing password");
        bPasswordNew.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bPasswordNewActionPerformed(evt);
            }
        });

        cbPasswordNew.setFocusable(false);

        cbPasswordNewRepeat.setFocusable(false);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel3)
                            .addComponent(jLabel5))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(pfCurrent, javax.swing.GroupLayout.DEFAULT_SIZE, 429, Short.MAX_VALUE)
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                    .addComponent(pfNewRepeat, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 401, Short.MAX_VALUE)
                                    .addComponent(pfNew, javax.swing.GroupLayout.DEFAULT_SIZE, 401, Short.MAX_VALUE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(cbPasswordNewRepeat)
                                    .addComponent(cbPasswordNew)))))
                    .addComponent(bPasswordNew, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(44, 44, 44))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(pfCurrent, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(pfNew, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(cbPasswordNew))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(pfNewRepeat, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(cbPasswordNewRepeat)
                            .addComponent(jLabel5))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(bPasswordNew)
                .addContainerGap())
        );

        jLabel1.setText("Standard Language");

        cbLanguage.setEditable(true);
        cbLanguage.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        cbLanguage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbLanguageActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(cbLanguage, 0, 452, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(cbLanguage, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        cbOS.setEditable(true);
        cbOS.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        cbOS.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbOSItemStateChanged(evt);
            }
        });
        cbOS.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbOSActionPerformed(evt);
            }
        });

        jLabel7.setText("Active Operating System Settings");

        jLabel6.setText("File type");

        cbFileType.setEditable(true);
        cbFileType.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        cbFileType.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbFileTypeActionPerformed(evt);
            }
        });

        jLabel8.setText("Program");

        tfProgram.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tfProgramCaretUpdate(evt);
            }
        });

        jButton1.setText("Add type");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel7)
                    .addGroup(jPanel3Layout.createSequentialGroup()
                        .addComponent(jLabel6)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cbFileType, javax.swing.GroupLayout.PREFERRED_SIZE, 135, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(jPanel3Layout.createSequentialGroup()
                        .addComponent(jLabel8)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(tfProgram, javax.swing.GroupLayout.DEFAULT_SIZE, 207, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButton1))
                    .addComponent(cbOS, javax.swing.GroupLayout.Alignment.LEADING, 0, 358, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cbOS, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel7))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(cbFileType, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(tfProgram, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton1)
                    .addComponent(jLabel8))
                .addContainerGap(36, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void cbLanguageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbLanguageActionPerformed
        SessionData.getUser().setPreferredLanguage((String) cbLanguage.getSelectedItem());
    }//GEN-LAST:event_cbLanguageActionPerformed

    private void bPasswordNewActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bPasswordNewActionPerformed
        SessionData.getUser().changeLoginPassword(String.valueOf(pfNew.getPassword()));
    }//GEN-LAST:event_bPasswordNewActionPerformed

    private void pfNewCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_pfNewCaretUpdate
        if (pfNew.getPassword().length > 5) {
            cbPasswordNew.setSelected(true);
        } else {
            cbPasswordNew.setSelected(false);
            bPasswordNew.setEnabled(false);
        }
        if (pfNewRepeat.getPassword().length > 0) {
            // check if they are equal, if not disable button and cbPasswordNewRepeat
            if (Arrays.equals(pfNewRepeat.getPassword(), pfNew.getPassword())) {
                cbPasswordNewRepeat.setSelected(true);
                if (cbPasswordNew.isSelected()) {
                    bPasswordNew.setEnabled(true);
                } else {
                    bPasswordNew.setEnabled(false);
                }
            } else {
                cbPasswordNewRepeat.setSelected(false);
                bPasswordNew.setEnabled(false);
            }
        }
    }//GEN-LAST:event_pfNewCaretUpdate

    private void pfNewRepeatCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_pfNewRepeatCaretUpdate
        if (Arrays.equals(pfNewRepeat.getPassword(), pfNew.getPassword())) {
            cbPasswordNewRepeat.setSelected(true);
            if (cbPasswordNew.isSelected()) {
                bPasswordNew.setEnabled(true);
            } else {
                bPasswordNew.setEnabled(false);
            }
        } else {
            cbPasswordNewRepeat.setSelected(false);
            bPasswordNew.setEnabled(false);
        }
    }//GEN-LAST:event_pfNewRepeatCaretUpdate

    private void pfCurrentCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_pfCurrentCaretUpdate
        // System.out.println("PreferencesUserSettings entered password "+String.valueOf(pfCurrent.getPassword()));
        if (String.valueOf(pfCurrent.getPassword()).equals(SessionData.getPasswordUserLogin())) {
            pfNew.setEnabled(true);
            pfNewRepeat.setEnabled(true);
            cbPasswordNew.setSelected(false);
            cbPasswordNewRepeat.setSelected(false);
        } else {
            pfNew.setEnabled(false);
            pfNewRepeat.setEnabled(false);
            bPasswordNew.setEnabled(false);
        }
    }//GEN-LAST:event_pfCurrentCaretUpdate

    private void cbOSItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cbOSItemStateChanged
        if (SessionData.getUser().hasOperatingSystemSettings((String) cbOS.getSelectedItem())) {
            ArrayList<String> list = new ArrayList<String>();
            Iterator<String> iterator = SessionData.getUser().getOperatingSystemSettings((String) cbOS.getSelectedItem()).keySet().iterator();
            while (iterator.hasNext()) {
                String element = iterator.next();
                list.add(element);
            }
            System.out.println(list);
            cbFileType.setModel(new DefaultComboBoxModel(list.toArray()));
            cbFileType.setSelectedIndex(0);
        } else {
            String[] array = new String[1];
            cbFileType.setModel(new DefaultComboBoxModel(array));
            cbFileType.setSelectedIndex(0);
        }
        SessionData.getUser().setDefaultOperatingSystem((String) cbOS.getSelectedItem());
}//GEN-LAST:event_cbOSItemStateChanged

    private void cbOSActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbOSActionPerformed
        if (SessionData.getUser().hasOperatingSystemSettings((String) cbOS.getSelectedItem())) {
            ArrayList<String> list = new ArrayList<String>();
            Iterator<String> iterator = SessionData.getUser().getOperatingSystemSettings((String) cbOS.getSelectedItem()).keySet().iterator();
            //  list.add(0, "");
            while (iterator.hasNext()) {
                list.add(iterator.next());
            }
            cbFileType.setModel(new DefaultComboBoxModel(list.toArray()));
            cbFileType.setSelectedIndex(0);
        } else {
            String[] array = new String[1];
            cbFileType.setModel(new DefaultComboBoxModel(array));
        }
        SessionData.getUser().setDefaultOperatingSystem((String) cbOS.getSelectedItem());
}//GEN-LAST:event_cbOSActionPerformed

    private void tfProgramCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tfProgramCaretUpdate
    }//GEN-LAST:event_tfProgramCaretUpdate

    private void cbFileTypeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbFileTypeActionPerformed
        if (SessionData.getUser().getOperatingSystemDefaultProgram((String) cbOS.getSelectedItem(), (String) cbFileType.getSelectedItem()) != null) {
            tfProgram.setText(SessionData.getUser().getOperatingSystemDefaultProgram((String) cbOS.getSelectedItem(), (String) cbFileType.getSelectedItem()));
        } else {
            tfProgram.setText("");
        }
    }//GEN-LAST:event_cbFileTypeActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        if (cbFileType != null && cbFileType.getItemCount() > 0) {
            if (((String) cbFileType.getSelectedItem()) != null) {
                if (((String) cbFileType.getSelectedItem()).length() > 0) {
                    SessionData.getUser().setOperatingSystemDefaultProgram((String) cbOS.getSelectedItem(), (String) cbFileType.getSelectedItem(), tfProgram.getText());
                }
            }
        }

        ArrayList<String> list = new ArrayList<String>();
        Iterator<String> iterator = SessionData.getUser().getOperatingSystemSettings((String) cbOS.getSelectedItem()).keySet().iterator();
        list.add(0, "");
        while (iterator.hasNext()) {
            list.add(iterator.next());
        }
        cbFileType.setModel(new DefaultComboBoxModel(list.toArray()));
        cbFileType.setSelectedItem("");
    }//GEN-LAST:event_jButton1ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bPasswordNew;
    private javax.swing.JComboBox cbFileType;
    private javax.swing.JComboBox cbLanguage;
    private javax.swing.JComboBox cbOS;
    private javax.swing.JCheckBox cbPasswordNew;
    private javax.swing.JCheckBox cbPasswordNewRepeat;
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPasswordField pfCurrent;
    private javax.swing.JPasswordField pfNew;
    private javax.swing.JPasswordField pfNewRepeat;
    private javax.swing.JTextField tfProgram;
    // End of variables declaration//GEN-END:variables
}
