/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 *
 * TreeObjectSelectorGUI.java
 *
 * Created on 08.05.2010, 10:22:21
 */
package net.geoadel.bibcite.GUI;

import net.geoadel.bibcite.Events.TreeViewSelectionEvent;
import net.geoadel.bibcite.Events.TreeViewSelectionListener;
import net.geoadel.bibcite.Tools.FileOperations;
import java.util.Arrays;
import java.util.Iterator;
import java.util.UUID;
import java.util.ArrayList;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import net.geoadel.bibcite.DatabaseObjects.*;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.HashMap;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.dbModel;

/**
 * Provides JTree for three views for selecting specific bibliography entries to edit and view them and their quotations
 * by bibliography key (could be changed to bibliography name actually)
 * by bibliography authors (all authors on their own)
 * by tags (all tags of related quotations lead to the bibliographies of the quotes)
 * A click on each bibliography entry loads the corresponding bibliography into the BibliographyEditor
 *
 * @DocumentationStatus finished
 * @GPLStatus clean
 * @author Georg Adelmann
 */
public class TreeObjectSelectorGUI extends javax.swing.JPanel {

    static User user = SessionData.getUser(); // the current user of this instance of the program
    private transient ArrayList listeners; // a ArrayList containing listeners for the selection event
    // the tree nodes with uuid
    DefaultMutableTreeNodeWithUUID topNode;
    DefaultMutableTreeNodeWithUUID topBibs;
    DefaultMutableTreeNodeWithUUID topPeople;
    DefaultMutableTreeNodeWithUUID topTags;
    DefaultMutableTreeNodeWithUUID topNotes;
    DefaultMutableTreeNodeWithUUID topFiles;
    DefaultMutableTreeNodeWithUUID topQuotes;
    /**
     * The buttons for creating, saving, deleting database elements
     */
    static JButton bCreateBib = new JButton();
    static JButton bCreateNote = new JButton();
    static JButton bCreatePerson = new JButton();
    static JButton bCreateFile = new JButton();
    static JButton bCreateQuote = new JButton();
    static JButton bSaveBib = new JButton();
    static JButton bSaveNote = new JButton();
    static JButton bSavePerson = new JButton();
    static JButton bSaveFile = new JButton();
    static JButton bSaveQuote = new JButton();
    static JButton bDeleteBib = new JButton();
    static JButton bDeleteNote = new JButton();
    static JButton bDeletePerson = new JButton();
    static JButton bDeleteFile = new JButton();
    static JButton bDeleteQuote = new JButton();
    static JButton bImportFile = new JButton();
    /**
     * The data
     */
    static ArrayList<Bibliography> alBibs;
    static ArrayList<Quotation> alQuotes;
    static ArrayList<Note> alNotes;
    static ArrayList<dbFile> alFiles;
    static ArrayList<Person> alPersons;
    ArrayList<dbObject> alObjects = new ArrayList<dbObject>();

    /** Creates new form TreeObjectSelectorGUI */
    JPanel TreeView(ArrayList<Bibliography> alBibs, ArrayList<Quotation> alQuotes, ArrayList<Note> alNotes, ArrayList<dbFile> alFiles, ArrayList<Person> alPersons, User user) {
        TreeObjectSelectorGUI.user = user;

        TreeObjectSelectorGUI.alBibs = alBibs;
        TreeObjectSelectorGUI.alFiles = alFiles;
        TreeObjectSelectorGUI.alNotes = alNotes;
        TreeObjectSelectorGUI.alQuotes = alQuotes;
        TreeObjectSelectorGUI.alPersons = alPersons;

        // Sort the data
        Collections.sort(TreeObjectSelectorGUI.alBibs);
        Collections.sort(TreeObjectSelectorGUI.alQuotes);
        Collections.sort(TreeObjectSelectorGUI.alNotes);
        Collections.sort(TreeObjectSelectorGUI.alFiles);
        Collections.sort(TreeObjectSelectorGUI.alPersons);

        // Add the individual arrayLists to general alObjects
        alObjects.addAll(alBibs);
        alObjects.addAll(alFiles);
        alObjects.addAll(alNotes);
        alObjects.addAll(alQuotes);
        alObjects.addAll(alPersons);

        bCreateBib.setIcon(new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/bibNew.png")));
        bCreateNote.setIcon(new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/noteNew.png")));
        bCreateQuote.setIcon(new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/quoteNew.png")));
        bCreateFile.setIcon(new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/fileNew.png")));
        bCreatePerson.setIcon(new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/personNew.png")));

        bImportFile.setIcon(new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/fileImport.png")));


        bCreateBib.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dbModel.createBibEditor();
            }
        });

        bCreatePerson.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dbModel.createPersonEditor();
            }
        });

        bCreateNote.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dbModel.createNoteEditor();
            }
        });

        bCreateFile.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dbModel.createFile();
            }
        });

        bImportFile.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dbModel.importFile();
            }
        });

        initComponents();

        // clear tree of old data and then fill it with new data
        tree.removeAll();
        tree.setModel(new DefaultTreeModel(createNodes()));


        // initiate the custom nodePanel (which enables preview icons next to dbObject entries, if they
        // have a file image (only png for now) with their uuid (can be set via GUI by calling 'setIcon')
        TreeCellRenderer renderer = new CustomCellRenderer();

        // tell the JTrees to use the custom renderer
        tree.setCellRenderer(renderer);

        // needed for WinXP Java to adapt the row height to the nodes - without it they were not high enough
        tree.setRowHeight(0);

        // System.out.println("Number of root nodes: " + topNode.getChildCount());
        tree.setEditable(false);

        tree.updateUI();

        // System.out.println("Created trees with "+alBibs.size()+" bibs and "+alQuotes.size()+" quotes.");
        return panelTrees;
    }

    private DefaultMutableTreeNodeWithUUID createNodes() {

        topNode = new DefaultMutableTreeNodeWithUUID(new dbObject("Data", SessionData.getUser()));
        topNode.removeAllChildren();
        topNode = createNodes(topNode);

        topPeople = createNodesByPerson(topPeople);

        topNode.add(topPeople);

        return topNode;
    }

    /**
     * creates nodes for people and authors (not specifically stored as people but authors of bib-entries)
     * @param rootNode  the root node, the author nodes should be added to
     * @param alBibs    all bibliographies this user has access to
     * @return  root node with all subnodes attached
     */
    private DefaultMutableTreeNodeWithUUID createNodesByPerson(DefaultMutableTreeNodeWithUUID rootNode) {
        // System.out.println("createNodesByAuthor: ArrayList bibs contains " + alBibs.size() + " elements");
        HashMap<String, ArrayList<Bibliography>> hmAuthors = new HashMap<String, ArrayList<Bibliography>>();
        for (Bibliography bib : alBibs) {
            String bibAuthors = bib.getAuthorNameAsText();
            String NEWLINE = System.getProperty("line.separator");
            String[] authorArray = bibAuthors.split(NEWLINE);
            ArrayList<String> authors = new ArrayList<String>();
            authors.addAll(Arrays.asList(authorArray));
            Collections.sort(authors);

            Iterator<String> iAuth = authors.iterator();
            while (iAuth.hasNext()) {
                String author = iAuth.next();
                ArrayList<Bibliography> tempBibs = hmAuthors.get(author);
                if (tempBibs == null) {
                    tempBibs = new ArrayList<Bibliography>();
                }
                tempBibs.add(bib);
                hmAuthors.put(author, tempBibs);
            }
        }

        for (Person person : alPersons) {
            DefaultMutableTreeNodeWithUUID personNode = new DefaultMutableTreeNodeWithUUID(person);
            if (hmAuthors.get(person.getName()) != null) {
                for (Bibliography bib : hmAuthors.get(person.getName())) {
                    personNode.add(new DefaultMutableTreeNodeWithUUID(bib));
                }
            }

            rootNode.add(personNode);
        }

        return rootNode;
    }

    /**
     * creates nodes for bibliographies by their titles
     * @param rootNode  the root node, the tag nodes should be added to
     * @param alBibs    all bibliographies this user has access to
     * @return  root node with all subnodes attached
     */
    private DefaultMutableTreeNodeWithUUID createNodes(DefaultMutableTreeNodeWithUUID rootNode) {
        // Instantiate DefaultTreeNodeWithUUID for bibliographies, authors and tags
        topBibs = new DefaultMutableTreeNodeWithUUID(new dbObject("Bibliographies", user));
        topPeople = new DefaultMutableTreeNodeWithUUID(new dbObject("People", user));
        topTags = new DefaultMutableTreeNodeWithUUID(new dbObject("Tags", user));
        topNotes = new DefaultMutableTreeNodeWithUUID(new dbObject("Notes", user));
        topFiles = new DefaultMutableTreeNodeWithUUID(new dbObject("Files", user));
        topQuotes = new DefaultMutableTreeNodeWithUUID(new dbObject("Quotes", user));

        // find and store tags and their corresponding objects
        HashMap<String, ArrayList<dbObject>> tagMap = new HashMap<String, ArrayList<dbObject>>();

        for (dbObject object : alObjects) {
            if (!object.getTags().isEmpty()) {
                for (String tag : object.getTags()) {
                    if (tagMap.containsKey(tag)) {
                        ArrayList<dbObject> tagObjects = tagMap.get(tag);
                        tagObjects.add(object);
                        tagMap.put(tag, tagObjects);
                    } else {
                        ArrayList<dbObject> tagObjects = new ArrayList<dbObject>();
                        tagObjects.add(object);
                        tagMap.put(tag, tagObjects);
                    }
                }
            }
            DefaultMutableTreeNodeWithUUID node = new DefaultMutableTreeNodeWithUUID(object);
            for (dbObject relation : alObjects) {
                if (relation.getRelatedObjects().contains(object.getUUID())) {
                    node.add(new DefaultMutableTreeNodeWithUUID(relation));
                }
            }

            if (object instanceof Bibliography) {
                // System.out.println("Bibliography: "+object.getName());
                topBibs.add(node);
            }
            if (object instanceof Note) {
                topNotes.add(node);
            }
            if (object instanceof dbFile) {
                topFiles.add(node);
            }
            if (object instanceof Quotation) {
                topQuotes.add(node);
            }

        }

        for (String key : tagMap.keySet()) {
            ArrayList<dbObject> objects = tagMap.get(key);
            DefaultMutableTreeNodeWithUUID node = new DefaultMutableTreeNodeWithUUID(new dbObject(key, SessionData.getUser()));
            for (dbObject object : objects) {
                node.add(new DefaultMutableTreeNodeWithUUID(object));
            }
            topTags.add(node);
        }

        rootNode.add(topBibs);
        rootNode.add(topQuotes);
        rootNode.add(topNotes);
        rootNode.add(topPeople);
        rootNode.add(topFiles);
        rootNode.add(topTags);

        return rootNode;
    }

    /**
     * This class helps combining the informations of the database elements (mainly the UUID and name) in the JTree and makes it retrievable
     */
    class DefaultMutableTreeNodeWithUUID extends DefaultMutableTreeNode implements Comparable<DefaultMutableTreeNodeWithUUID> {

        private UUID uuid;
        private String name;

        public DefaultMutableTreeNodeWithUUID(dbObject object) {
            setName(object.getName());
            setUUID(object.getUUID());
            setUserObject(object);
        }

        private void setUUID(UUID uuid) {
            this.uuid = uuid;
        }

        private UUID getUUID() {
            return uuid;
        }

        private void setName(String name) {
            this.name = name;
        }

        private String getName() {
            return name;
        }

        /**
         * This is important, because JTrees use toString for the String that is visible.
         * So by overriding this I make sure, the value for name is visible and just
         * have to make sure, that the name String contains the informations I want to be displayed
         * @return
         */
        @Override
        public String toString() {
            return name;
        }

        public int compareTo(DefaultMutableTreeNodeWithUUID node) {
            int retValue = getName().compareToIgnoreCase(node.getName());
            return retValue;
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        panelTrees = new javax.swing.JPanel();
        toolBarCategories = new javax.swing.JToolBar();
        jScrollPane1 = new javax.swing.JScrollPane();
        tree = new javax.swing.JTree();
        tree.setRootVisible(false);
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        tfSearch = new javax.swing.JTextField();

        panelTrees.setMaximumSize(new java.awt.Dimension(200, 32767));
        panelTrees.setPreferredSize(new java.awt.Dimension(200, 407));

        toolBarCategories.setFloatable(false);
        toolBarCategories.setRollover(true);

        tree.setEditable(true);
        tree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                treeValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(tree);

        jLabel1.setText("Search");

        tfSearch.setText("search text");
        tfSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tfSearchActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(tfSearch, javax.swing.GroupLayout.DEFAULT_SIZE, 102, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(tfSearch, javax.swing.GroupLayout.PREFERRED_SIZE, 24, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout panelTreesLayout = new javax.swing.GroupLayout(panelTrees);
        panelTrees.setLayout(panelTreesLayout);
        panelTreesLayout.setHorizontalGroup(
            panelTreesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(toolBarCategories, javax.swing.GroupLayout.DEFAULT_SIZE, 174, Short.MAX_VALUE)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 174, Short.MAX_VALUE)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        panelTreesLayout.setVerticalGroup(
            panelTreesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelTreesLayout.createSequentialGroup()
                .addComponent(toolBarCategories, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 284, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(panelTrees, javax.swing.GroupLayout.PREFERRED_SIZE, 174, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(panelTrees, javax.swing.GroupLayout.DEFAULT_SIZE, 381, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    /** Register a listener for TreeObjectSelectorGUI */
    synchronized public void addTreeViewListener(TreeViewSelectionListener l) {
        if (listeners == null) {
            listeners = new ArrayList();
        }
        listeners.add(l);
    }

    /** Remove a listener for TreeObjectSelectorGUI */
    synchronized public void removeTreeViewListener(TreeViewSelectionListener l) {
        if (listeners == null) {
            listeners = new ArrayList();
        }
        listeners.remove(l);
    }

    /** Fire a selectionChanged to all registered listeners */
    protected void selectionChanged(boolean selection, Object object) {
        // System.out.println("selectionChanged");
        // if we have no listeners, do nothing...
        if (object != null & (listeners != null && !listeners.isEmpty())) {
            // create the event object to send
            TreeViewSelectionEvent event = new TreeViewSelectionEvent(object, selection);

            // make a copy of the listener list in case
            //   anyone adds/removes listeners
            ArrayList targets;
            synchronized (this) {
                targets = (ArrayList) listeners.clone();
            }

            // walk through the listener list and
            // call the selectionChanged method in each
            Iterator e = targets.iterator();
            while (e.hasNext()) {
                // System.out.println("Selection changed - informing others");
                TreeViewSelectionListener l = (TreeViewSelectionListener) e.next();
                l.selectionChanged(event);
            }
        }
    }

    public static void showButtonsPerson() {
        toolBarCategories.removeAll();
        toolBarCategories.add(bCreatePerson);
        // toolBarCategories.add(bSavePerson);
        // toolBarCategories.add(bDeletePerson);

        // bSavePerson.setEnabled(false);
    }

    public static void showButtonsQuote() {
        toolBarCategories.removeAll();
        toolBarCategories.add(bCreateQuote);
        // toolBarCategories.add(bSaveQuote);
        // toolBarCategories.add(bDeleteQuote);

        // bSaveQuote.setEnabled(false);
    }

    public static void showButtonsFile() {
        toolBarCategories.removeAll();
        toolBarCategories.add(bImportFile);
        // toolBarCategories.add(bCreateFile);
        // toolBarCategories.add(bSaveFile);
        // toolBarCategories.add(bDeleteFile);

        // bSaveFile.setEnabled(false);
    }

    public static void showButtonsNote() {
        toolBarCategories.removeAll();
        toolBarCategories.add(bCreateNote);
        // toolBarCategories.add(bSaveNote);
        // toolBarCategories.add(bDeleteNote);

        // bSaveNote.setEnabled(false);
    }

    public static void showButtonsBib() {
        toolBarCategories.removeAll();
        toolBarCategories.add(bCreateBib);
        // toolBarCategories.add(bSaveBib);
        // toolBarCategories.add(bDeleteBib);

        // bSaveBib.setEnabled(false);
    }

    public static void enableSaveButtons() {
        // bSaveBib.setEnabled(true);
        // bSaveFile.setEnabled(true);
        // bSaveNote.setEnabled(true);
        // bSavePerson.setEnabled(true);
        // bSaveQuote.setEnabled(true);
    }

    /**
     * If Enter was pressed in the search JTextField, convert the search parameters into an ArrayList and give it to the database
     * then update the JTree View only displaying the search results
     * @param evt
     */
    private void tfSearchActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tfSearchActionPerformed
        ArrayList<String> alSearch = new ArrayList<String>();
        if (tfSearch.getText().contains(", ")) {
            String[] searchParameters = tfSearch.getText().split(", ");
            alSearch.addAll(Arrays.asList(searchParameters));
        } else {
            alSearch.add(tfSearch.getText());
        }

        int i = 0;
        Object[] object = dbModel.returnSearchResults(alSearch);
        new BibCiteGUI().updateTree((ArrayList<Bibliography>) object[0], (ArrayList<Quotation>) object[1], (ArrayList<Note>) object[2], (ArrayList<dbFile>) object[3], (ArrayList<Person>) object[4]);
    }//GEN-LAST:event_tfSearchActionPerformed

    private void treeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_treeValueChanged
        // System.out.println("treeValueChanged()");
        if (tree.getSelectionPath() != null) {

            // System.out.println("TreeSelectionPath: " + ((DefaultMutableTreeNodeWithUUID) tree.getSelectionPath().getPathComponent(1)).getName());
            toolBarCategories.removeAll();
            Object object = ((DefaultMutableTreeNodeWithUUID) tree.getSelectionPath().getLastPathComponent()).getUserObject();
            DefaultMutableTreeNodeWithUUID node = ((DefaultMutableTreeNodeWithUUID) tree.getSelectionPath().getPathComponent(1));
            if (object instanceof Bibliography || node.getName().equals("Bibliographies")) {
                showButtonsBib();
            }
            if (object instanceof Note || node.getName().equals("Notes")) {
                showButtonsNote();
            }
            if (object instanceof dbFile || node.getName().equals("Files")) {
                showButtonsFile();
            }
            if (object instanceof Person || node.getName().equals("People")) {
                showButtonsPerson();
            }
            if (object instanceof Quotation || node.getName().equals("Quotations")) {
                showButtonsQuote();
            }
            toolBarCategories.updateUI();

            /**
             * Call the appropriate editor
             */
            selectionChanged(true, ((DefaultMutableTreeNodeWithUUID) evt.getPath().getLastPathComponent()).getUserObject());
        }

    }//GEN-LAST:event_treeValueChanged
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JPanel panelTrees;
    private javax.swing.JTextField tfSearch;
    private static javax.swing.JToolBar toolBarCategories;
    private javax.swing.JTree tree;
    // End of variables declaration//GEN-END:variables

    /**
     * This inner class provides a custom cell nodePanel, that has support for
     * background colors based on selection state and icons
     */
    class CustomCellRenderer implements TreeCellRenderer {

        JLabel name = new JLabel("");
        JPanel nodePanel = new JPanel();
        DefaultTreeCellRenderer defaultRenderer = new DefaultTreeCellRenderer();
        Color backgroundSelectionColor;
        Color backgroundNonSelectionColor;

        /**
         * This instantiates the general CustomCellRenderer, all the settings in here apply to ALL nodes
         */
        public CustomCellRenderer() {
            name.setForeground(Color.BLACK);
            nodePanel.add(name);
            // nodePanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
            backgroundSelectionColor = defaultRenderer.getBackgroundSelectionColor();
            backgroundNonSelectionColor = defaultRenderer.getBackgroundNonSelectionColor();
        }

        /**
         * This getTreeCellRendererComponent is run for all elements of the JTree
         * @param tree  the tree, those components are part of
         * @param value the individual node within the JTree
         * @param selected  boolean, whether the node is selected or not
         * @param expanded  boolean, whether it is expanded or not
         * @param leaf  boolean, whether it is a leaf
         * @param row   row number
         * @param hasFocus  whether the component has focus
         * @return  component to be displayed as defined in this method
         */
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected,
                boolean expanded, boolean leaf, int row, boolean hasFocus) {
            Component returnValue = null;
            if ((value != null) && (value instanceof DefaultMutableTreeNode)) {
                Object userObject = ((DefaultMutableTreeNode) value).getUserObject();

                dbObject object = (dbObject) userObject;
                name.setText("<html><font-size=\"8\">" + object.getName());

                // System.out.println("Objectname: " + object.getName());
                if (selected) {
                    nodePanel.setBackground(backgroundSelectionColor);
                } else {
                    nodePanel.setBackground(backgroundNonSelectionColor);
                }

                // set the standard icon, if this entry does not have a small icon under its own uuid files

                ImageIcon f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/file.png"));
                // adjust icon according to type
                if (userObject instanceof Bibliography) {
                    f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Book32.png"));
                }
                if (userObject instanceof Person) {
                    f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Person32.png"));
                }
                if (userObject instanceof dbFile) {
                    f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/File32.png"));
                }
                if (userObject instanceof Note) {
                    f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Note32.png"));
                }
                if (userObject instanceof Quotation) {
                    f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Quote32.png"));
                }
                if (userObject instanceof dbObject) {
                    if (((dbObject) userObject).getName().equals("Bibliographies")) {
                        f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Book48.png"));
                    }
                    if (((dbObject) userObject).getName().equals("Files")) {
                        f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/File48.png"));
                    }
                    if (((dbObject) userObject).getName().equals("Notes")) {
                        f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Note48.png"));
                    }
                    if (((dbObject) userObject).getName().equals("People")) {
                        f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Person48.png"));
                    }
                    if (((dbObject) userObject).getName().equals("Tags")) {
                        f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Tag48.png"));
                    }
                    if (((dbObject) userObject).getName().equals("Quotes")) {
                        f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Quote48.png"));
                    }
                    if (((DefaultMutableTreeNodeWithUUID) value).getParent() != null) {
                        if (((DefaultMutableTreeNodeWithUUID) ((DefaultMutableTreeNodeWithUUID) value).getParent()).getName().equals("Tags")) {
                            f = new ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/Tag32.png"));
                        }
                    }
                }
                // System.out.println("^^^  Path of standard image: "+f.getAbsolutePath());
                name.setIcon(f);

                // if object has uuid, try to get its specific preview icon for displaying (if one exists)
                if (object.getUUID() != null) {
                    if (FileOperations.hasPreviewSmall(object.getUUID())) {
                        name.setIcon(new ImageIcon(FileOperations.loadPreviewImageSmall(object.getUUID())));
                    }
                }
                nodePanel.setEnabled(tree.isEnabled());
                returnValue = nodePanel;
            }
            if (returnValue == null) {
                returnValue = defaultRenderer.getTreeCellRendererComponent(tree, value, selected, expanded,
                        leaf, row, hasFocus);
            }
            return returnValue;
        }
    }
}
