/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.ImportExport.DOIImport.Filters;

import java.util.ArrayList;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.DatabaseObjects.dbObject;
import net.geoadel.bibcite.Tools.InternetTools;

/**
 *
 * @author Georg Adelmann
 */
public class filterSpringerLink {

    public static Bibliography filterJournal(String text) {
        String journalTitle = "";
        String volume = "";
        String number = "";
        String articleTitle = "";
        String authors = "";
        String pages = "";
        String datePublished = "";
        String publisher = "Springer";

        String month = "";
        String year = "";
        // System.out.println("Filtering for journal article...");

        // System.out.println("Filtering for SpringerLink journal article ...");
        // System.out.println(text);

        if (text.contains(", Volume ")) {
            journalTitle = text.substring(text.indexOf(" - ") + 3, text.indexOf(", Vol")).trim();
            volume = text.substring(text.indexOf("Volume ") + 7, text.indexOf(", Number")).trim();
            number = text.substring(text.indexOf(", Number") + 9, text.indexOf("</title>")).trim();
        } else {
            if(text.contains("Online First")){
                journalTitle = text.substring(text.indexOf(" - ") + 3, text.indexOf(", Online")).trim();
                volume = "";
                number = "";
            }
        }


        articleTitle = text.substring(text.indexOf("<h1>") + 4, text.indexOf("</h1><p class=\"authors\">")).trim();
        String pagesStart = text.substring(text.indexOf("startPage=") + 10, text.indexOf("&amp", text.indexOf("startPage=")));
        String pagesEnd = text.substring(text.indexOf("endPage=") + 8, text.indexOf("&amp", text.indexOf("endPage=")));
        pages = pagesStart + " - " + pagesEnd;
        datePublished = text.substring(text.indexOf("publicationDate=") + 16, text.indexOf("&amp;", text.indexOf("publicationDate="))).replaceAll("%2f", "-").trim();

        if (datePublished.length() > 4) {
            month = datePublished.substring(datePublished.indexOf("-"), datePublished.lastIndexOf("-"));
            year = datePublished.substring(datePublished.lastIndexOf("-"));
        } else {
            year = datePublished;
        }

        // System.out.println("Found date " + month + " in " + year);

        int authorsAreaStart = text.indexOf("</h1><p class=\"authors\">");
        String authorsArea = text.substring(authorsAreaStart, text.indexOf("<!-- Clear floating elements -->"));

        Pattern pattern = Pattern.compile("View content where Author is .+? href=", Pattern.DOTALL);
        Matcher matcher = pattern.matcher(authorsArea);

        ArrayList<String> authorsList = new ArrayList<String>();

        while (matcher.find()) {
            String entry = matcher.group();

            entry = entry.substring(entry.indexOf("is ") + 3, entry.indexOf("\" href="));
            authorsList.add(entry);
        }

        // System.out.println(articleTitle + " published in " + journalTitle + ", Volume " + volume + ", Number " + number + ", Pages " + pages);
        // System.out.println("Published " + datePublished + " by " + publisher);
        // System.out.println("Authors: " + authors);

        ArrayList<String> editorsList = new ArrayList<String>();
        Bibliography bib = new Bibliography(UUID.randomUUID(), "", "", "", "", "", "", journalTitle, "", "", publisher, "", "", articleTitle, "article", "", "", month, year, number, pages, volume, authorsList, editorsList, "", "", "", "", 0L, SessionData.getUser(), "", new dbObject().setDateEdited(), false, SessionData.getUser().getPreferredLanguage(), "", new ArrayList<String>());
        return bib;
    }

    public static Bibliography filterBook(String text) {
        // System.out.println("Looking for springer links book");

        String titleBegin = "class=\"title\">";
        String titleEnd = "</h1>";
        String yearBegin = "Copyright Year</dt><dd>";
        String yearEnd = "</dd><dt>";

        String title = "";
        String subtitle = "";

        String authors = "";
        String editors = "";
        String publisher = "Springer";

        String year = "";

        if (text.contains("SpringerLink - ")) {
            title = text.substring(text.indexOf(titleBegin) + titleBegin.length(), text.indexOf(titleEnd, text.indexOf(titleBegin))).trim();
            if (title.contains("<span class=\"subtitle\">")) {
                subtitle = title.substring(title.indexOf("subtitle\">") + 10, title.indexOf("</span")).trim();
                title = title.substring(0, title.indexOf("<")).trim();
            }
            year = text.substring(text.indexOf(yearBegin) + yearBegin.length(), text.indexOf(yearEnd, text.indexOf(yearBegin))).trim();

            // System.out.println("*********************************");
            // System.out.println(text);
            // System.out.println("*********************************");

            Pattern pattern = Pattern.compile("<a title=\"View content where Author is .+? \"", Pattern.DOTALL);
            Matcher matcher = pattern.matcher(text);

            ArrayList<String> authorsList = new ArrayList<String>();

            while (matcher.find()) {
                String entry = matcher.group();
                entry = entry.substring(entry.indexOf("\">") + 3, entry.indexOf("</a>", entry.indexOf("\">")));
                if (!authorsList.contains(entry)) {
                    authorsList.add(entry);
                }
            }

            ArrayList<String> editorsList = new ArrayList<String>();
            if (text.contains("</h1><p class=\"editors\">")) {
                Pattern patternEditors = Pattern.compile("<a title=\"View content where Editor is .+? \"", Pattern.DOTALL);
                Matcher matcherEditors = patternEditors.matcher(text);

                while (matcherEditors.find()) {
                    String entry = matcherEditors.group();
                    entry = entry.substring(entry.indexOf("is ") + 3, entry.indexOf("\"", entry.indexOf("is ")));
                    // System.out.println(entry);
                    if (!editorsList.contains(entry)) {
                        editorsList.add(entry);
                    }
                }
            }

            UUID uuid = UUID.randomUUID();

            // importFiles(text, uuid); // doesn't work for some reason... java identifies the links as html and therefore does not download the pdf files

            Bibliography bib = new Bibliography(uuid, "", "", "", "", "", "", "", "", "", publisher, "", "", title, "book", "", "", "", year, "", "", "", authorsList, editorsList, "", "", "", "", 0L, SessionData.getUser(), subtitle, new dbObject().setDateEdited(), true, SessionData.getUser().getPreferredLanguage(), "", new ArrayList<String>());
            return bib;
        }

        return null;
    }

    private static void importFiles(String text, UUID bibUUID) {
        // System.out.println("Trying to import springer pdf files");
        // System.out.println(text);
        // Try to import the pdf files as well
        // 1. get the corresponding file url


        Pattern patternFile = Pattern.compile("<li class=\"primitive.+?</a></p></li>", Pattern.DOTALL);
        Matcher matcherFile = patternFile.matcher(text);

        while (matcherFile.find()) {
            String entry = matcherFile.group();
            // System.out.println("Found matches "+entry);

            String name = entry.substring(entry.lastIndexOf("/\">") + 3, entry.indexOf("</a></p></li>"));
            // System.out.println(name);
            String fileIdentifier = entry.substring(entry.indexOf("href=\"/content/") + 15, entry.indexOf("\">", entry.indexOf("href=\"/content/")));
            String url = "http://www.springerlink.com/content/" + fileIdentifier;
            if (!fileIdentifier.contains(".pdf")) {
                url = url + "fulltext.pdf";
            }

            // System.out.println("Name: "+name+" -> URL: "+url);
            InternetTools.getFile(name, url, bibUUID);
        }
    }
}
