/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.ImportExport.DOIImport;

import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.ImportExport.DOIImport.Filters.filterCambridge;
import net.geoadel.bibcite.ImportExport.DOIImport.Filters.filterNature;
import net.geoadel.bibcite.ImportExport.DOIImport.Filters.filterSagePub;
import net.geoadel.bibcite.ImportExport.DOIImport.Filters.filterScienceDirect;
import net.geoadel.bibcite.ImportExport.DOIImport.Filters.filterSpringerLink;
import net.geoadel.bibcite.Tools.InternetTools;

/**
 *
 * @author Georg Adelmann
 */
public class Import_DOI {

    public static Bibliography importByDOI(String doi) {
        Bibliography bib = null;
        String html = InternetTools.getHTML("http://dx.doi.org/" + doi);
        // System.out.println("Getting html for DOI " + doi);
        // System.out.println(html);

        if (html.contains("SpringerLink - ")) {
            String identifierBegin = "\"Save this item\" href=\"/content/";
            String identifierEnd = "/fulltext/";
            if (html.contains("/abstract/?target")) {
                identifierEnd = "/abstract/";
            }
            String identifier = html.substring(html.indexOf(identifierBegin) + identifierBegin.length(), html.indexOf(identifierEnd));

            // find out, whether it is a book or journal article
            if (html.contains(identifier + "/contents/")) {
                html = InternetTools.getHTML("http://www.springerlink.com/content/" + identifier + "/contents/");
                // System.out.println(html);
                if (html.contains("Search Within This Book")) {
                    bib = filterSpringerLink.filterBook(html);
                }
            } else {
                bib = filterSpringerLink.filterJournal(html);
            }
        }

        if (html.contains("SAGE Publications")) {
            bib = filterSagePub.filter(html);
        }

        if (html.contains("Nature Publishing Group")){
            // System.out.println(html);
            bib = filterNature.filter(html);
        }
        
        if(html.contains("Cambridge University Press")){
            bib = filterCambridge.filter(html);
        }
        
        if(html.contains("ScienceDirect")){
            bib = filterScienceDirect.filter(html);
        }
        
        return bib;
    }
}
