/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.ImportExport;

import net.geoadel.bibcite.Database.db4o;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import net.geoadel.bibcite.DatabaseObjects.*;
import java.util.UUID;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.dbModel;

/**
 * Exports bibliography entries and their quotations into external files, depending on the isBackup, all revisions are exported or only the newest entries
 * The format is the bibTex format with a custom field for storing the quotations
 *
 * @DocumentationStatus finished
 * @GPLStatus   clean
 * @author Georg Adelmann
 */
public class Export_bibTex {

    public static void export_bibTex(Bibliography[] bibs, boolean exportPrivate, boolean exportQuotes, boolean exportPersons, boolean exportNotes, boolean separateEntries) {
        PrintStream p = null; // declare a print stream object
        String NEW_LINE = System.getProperty("line.separator");
        try {
            // if all should go into one export-file (separateEntries == FALSE)
            // Create a new file output stream for either backup or export, depending on the selected menu entry
            if (!separateEntries) {
                p = new PrintStream("exports.bib");
                p.println("% This file was created by bibcite " + net.geoadel.bibcite.DataFields.Preferences.version);
                p.println("% Encoding: UTF-8");
            }



            for (Bibliography bib : bibs) {
                // System.out.println("Exporting "+bib.getKey());
                // if all should go in separate files
                if (separateEntries) {
                    p = new PrintStream(bib.getKey() + ".bib");
                    p.println("% This file was created by bibcite " + net.geoadel.bibcite.DataFields.Preferences.version);
                    p.println("% Encoding: UTF-8");
                }
                p.println();
                if (!bib.isPrivate() || exportPrivate) {
                    p.println("@" + bib.getType() + "{" + bib.getKey() + ",");
                    if (!bib.getTitle().isEmpty()) {
                        p.println("title = {" + bib.getTitle() + "},");
                    }
                    if (!bib.getSubtitle().isEmpty()) {
                        p.println("subtitle = {" + bib.getSubtitle() + "},");
                    }
                    p.println("dateedited = {" + bib.getDateEdited() + "},");

                    if (!String.valueOf(bib.getYear()).isEmpty()) {
                        p.println("year = {" + bib.getYear() + "},");
                    }
                    p.println("uuid = {" + bib.getUUID() + "},");
                    if (!bib.getPublisher().isEmpty()) {
                        p.println("publisher = {" + bib.getPublisher() + "},");
                    }
                    if (!bib.getAddress().isEmpty()) {
                        p.println("address = {" + bib.getAddress() + "},");
                    }
                    if (bib.getJournal() != null && !bib.getJournal().isEmpty()) {
                        p.println("journal = {" + bib.getJournal() + "},");
                    }
                    if (bib.getBooktitle() != null && !bib.getBooktitle().isEmpty()) {
                        p.println("booktitle = {" + bib.getBooktitle() + "},");
                    }
                    if (bib.getCrossref() != null && !bib.getCrossref().isEmpty()) {
                        p.println("crossref = {" + bib.getCrossref() + "},");
                    }
                    if (bib.getHowpublished() != null && !bib.getHowpublished().isEmpty()) {
                        p.println("howpublished = {" + bib.getHowpublished() + "},");
                    }
                    if (bib.getInstitution() != null && !bib.getInstitution().isEmpty()) {
                        p.println("institution = {" + bib.getInstitution() + "},");
                    }
                    if (bib.getKey() != null && !bib.getKey().isEmpty()) {
                        p.println("key = {" + bib.getKey() + "},");
                    }
                    if (bib.getOrganization() != null && !bib.getOrganization().isEmpty()) {
                        p.println("organization = {" + bib.getOrganization() + "},");
                    }
                    if (bib.getPages() != null) {
                        p.println("pages = {" + bib.getPages() + "},");
                    }
                    if (bib.getSchool() != null && !bib.getSchool().isEmpty()) {
                        p.println("school = {" + bib.getSchool() + "},");
                    }
                    if (bib.getSeries() != null && !bib.getSeries().isEmpty()) {
                        p.println("series = {" + bib.getSeries() + "},");
                    }
                    if (bib.getBibAbstract() != null && !bib.getBibAbstract().isEmpty()) {
                        p.println("abstract = {" + bib.getBibAbstract() + "},");
                    }

                    if (!bib.getAuthors().isEmpty()) {

                        ArrayList<String> authors = bib.getAuthors();
                        int i = 0;
                        for (String author : authors) {
                            if (i == 0) {
                                p.print("author = { " + author);
                            } else {
                                p.print(" and " + author);
                            }
                            i++;
                        }
                        p.println("},");
                    }

                    if (!bib.getAuthors().isEmpty()) {

                        ArrayList<String> editors = bib.getEditors();
                        int i = 0;
                        for (String editor : editors) {
                            if (i == 0) {
                                p.print("editor = { " + editor);
                            } else {
                                p.print(" and " + editor);
                            }
                            i++;
                        }
                        p.println("},");
                    }

                    if (!bib.getTags().isEmpty()) {
                        ArrayList<String> editors = bib.getEditors();
                        int i = 0;
                        for (String editor : editors) {
                            if (i == 0) {
                                p.print("tags = { " + editor);
                            } else {
                                p.print("; " + editor);
                            }
                            i++;
                        }
                        p.println("},");
                    }

                    if (bib.getChapter() != null && !bib.getChapter().isEmpty()) {
                        p.println("chapter = {" + bib.getChapter() + "},");
                    }
                    if (bib.getEdition() != null && !bib.getEdition().isEmpty()) {
                        p.println("edition = {" + bib.getEdition() + "},");
                    }
                    if (bib.getNumber() != null && !bib.getNumber().isEmpty()) {
                        p.println("number = {" + bib.getNumber() + "},");
                    }
                    if (bib.getVolume() != null && !bib.getVolume().isEmpty()) {
                        p.println("volume = {" + bib.getVolume() + "},");
                    }
                    if (bib.getMonth() != null && !bib.getMonth().isEmpty()) {
                        p.println("month = {" + bib.getMonth() + "},");
                    }
                    if (bib.getEPrint() != null && !bib.getEPrint().isEmpty()) {
                        p.println("eprint = {" + bib.getEPrint() + "},");
                    }

                    if (bib.getLicense() != null && !bib.getLicense().isEmpty()) {
                        p.println("copyright = {" + bib.getLicense() + "},"); // in bibtex copyright is the standard field for license details
                    }
                    if (bib.getUserUUID() != null && !bib.getUserUUID().toString().isEmpty()) {
                        p.println("user = {" + bib.getUserUUID() + "},");
                    }
                    if (bib.getUrl() != null && !bib.getUrl().isEmpty()) {
                        p.println("url = {" + bib.getUrl() + "},");
                    }
                    p.println("urldate = {" + bib.getUrlDate() + "},");
                    if (bib.getLanguage() != null && !bib.getLanguage().isEmpty()) {
                        p.println("language = {" + bib.getLanguage() + "},");
                    }
                    p.println("isPrivate = {" + bib.isPrivate() + "},");
                    if (bib.getIdentifier() != null && !bib.getIdentifier().isEmpty()) {
                        p.println("identifier = {" + bib.getIdentifier() + " },");
                    }
                    if (bib.getCustomFields() != null && !bib.getCustomFields().isEmpty()) {
                        for (String key : bib.getCustomFields().keySet()) {
                            p.println(key + " = {" + bib.getCustomFields().get(key) + "},");
                        }
                    }

                    p.println("}"); // ends bibliography entry
                    p.println();


                    if (exportQuotes) {
                        /**  EXPORT THE QUOTATIONS  **/
                        ArrayList<Quotation> vQuot = new ArrayList<Quotation>();

                        // get all the quotations related to this bibligraphy from database
                        vQuot = dbModel.getBibliographyQuotes(bib);
                        // store each of them with their fields in the database
                        for (int iQ = 0; iQ < vQuot.size(); iQ++) {
                            if (!vQuot.get(iQ).isPrivate() || exportPrivate) {
                                p.println("@Quotation{ ");
                                p.println("quoteContent = {" + vQuot.get(iQ).getQuote() + "},");
                                p.println("quoteUUID = {" + vQuot.get(iQ).getUUID() + "},");
                                p.println("quoteIsPrivat e= {" + vQuot.get(iQ).isPrivate() + "},");
                                p.println("quoteUser = {" + vQuot.get(iQ).getUserUUID() + "},");
                                p.println("quoteKey = {" + vQuot.get(iQ).getKey() + "},");
                                String related = "";
                                for (int iRel = 0; iRel < vQuot.get(iQ).getRelatedObjects().size(); iRel++) {
                                    if (iRel < vQuot.get(iQ).getRelatedObjects().size() - 1) { // all but the last entry end with an inserted semicolon
                                        related = related.concat(vQuot.get(iQ).getRelatedObjects().get(iRel) + "; ");
                                    } else { // the last entry does not get a semicolon at the end
                                        related = related.concat(vQuot.get(iQ).getRelatedObjects().get(iRel) + "");
                                    }
                                }

                                p.println("quoteRelations = {" + related + "},");
                                p.println("quoteGist = {" + vQuot.get(iQ).getGist() + "},");
                                p.println("quoteDateEdited = {" + vQuot.get(iQ).getDateEdited() + "},");
                                p.println("quotePageStart = {" + vQuot.get(iQ).getPageStart() + "},");
                                p.println("quotePageEnd = {" + vQuot.get(iQ).getPageEnd() + "},");
                                String tagString = "";
                                for (int iQTags = 0; iQTags < vQuot.get(iQ).getTags().size(); iQTags++) {
                                    if (iQTags < vQuot.get(iQ).getTags().size() - 1) { // all but the last entry end with an inserted semicolon
                                        tagString = tagString.concat(vQuot.get(iQ).getTags().get(iQTags) + "; ");
                                    } else { // the last entry does not get a semicolon at the end
                                        tagString = tagString.concat(vQuot.get(iQ).getTags().get(iQTags));
                                    }
                                }
                                p.println("quoteTags = {" + tagString + "},");
                                p.println("}"); // ends quote
                                p.println();
                            }
                        }
                    }

                    if (exportNotes) {
                        /**  EXPORT THE NOTES  **/
                        ArrayList<Note> vNotes = new ArrayList<Note>();

                        // get all the notes related to this object from database
                        vNotes = dbModel.getNotesRelatedToUUID(bib.getUUID());
                        // store each of them with their fields in the database
                        for (int iNotes = 0; iNotes < vNotes.size(); iNotes++) {
                            if (!vNotes.get(iNotes).isPrivate() || exportPrivate) {
                                p.println("@Note{ ");
                                p.println("noteContent = {" + vNotes.get(iNotes).getNoteText() + "},");
                                p.println("noteUUID = {" + vNotes.get(iNotes).getUUID() + "},");
                                p.println("noteIsPrivate= {" + vNotes.get(iNotes).isPrivate() + "},");
                                p.println("noteUser = {" + vNotes.get(iNotes).getUserUUID() + "},");
                                p.println("noteKey = {" + vNotes.get(iNotes).getKey() + "},");
                                p.println("noteName = {" + vNotes.get(iNotes).getName() + "},");
                                String related = "";
                                for (int iRel = 0; iRel < vNotes.get(iNotes).getRelatedObjects().size(); iRel++) {
                                    if (iRel < vNotes.get(iNotes).getRelatedObjects().size() - 1) { // all but the last entry end with an inserted semicolon
                                        related = related.concat(vNotes.get(iNotes).getRelatedObjects().get(iRel) + "; ");
                                    } else { // the last entry does not get a semicolon at the end
                                        related = related.concat(vNotes.get(iNotes).getRelatedObjects().get(iRel) + "");
                                    }
                                }

                                p.println("noteRelations = {" + related + "},");
                                p.println("noteDateEdited = {" + vNotes.get(iNotes).getDateEdited() + "},");
                                String tagString = "";
                                for (int iNTags = 0; iNTags < vNotes.get(iNotes).getTags().size(); iNTags++) {
                                    if (iNTags < vNotes.get(iNotes).getTags().size() - 1) { // all but the last entry end with an inserted semicolon
                                        tagString = tagString.concat(vNotes.get(iNotes).getTags().get(iNTags) + "; ");
                                    } else { // the last entry does not get a semicolon at the end
                                        tagString = tagString.concat(vNotes.get(iNotes).getTags().get(iNTags));
                                    }
                                }
                                p.println("noteTags = {" + tagString + "},");
                                p.println("}"); // ends quote
                                p.println();
                            }
                        }
                    }

                    // if persons are to be exported, export persons
                    if (exportPersons) {
                        // EXPORT PERSONS

                        ArrayList<Person> alPersons = dbModel.getPersonsNewestVersions();

                        for (Person person : alPersons) {
                            // only export person if not private or private fields are to be exported
                            if (!person.isPrivate() || exportPrivate) {
                                p.println("@Person{");
                                p.println("name = {" + person.getName() + "},");
                                if (person.getAddressMail() != null && !person.getAddressMail().isEmpty()) {
                                    p.println("addressmail = {" + person.getAddressMail() + "},");
                                }
                                if (person.getAddressPostal() != null && !person.getAddressPostal().isEmpty()) {
                                    p.println("addresspostal = {" + person.getAddressPostal() + "},");
                                }
                                if (person.getDescription() != null && !person.getDescription().isEmpty()) {
                                    p.println("description = {" + person.getDescription() + "},");
                                }
                                if (person.getMainJob() != null && !person.getMainJob().isEmpty()) {
                                    p.println("mainjob = {" + person.getMainJob() + "},");
                                }
                                if (person.getDateBirth() != null && person.getDateBirth() != 0L) {
                                    p.println("birthdate = {" + person.getDateBirth() + "},");
                                }
                                if (person.getDateDeath() != null && person.getDateDeath() != 0L) {
                                    p.println("deathdate = {" + person.getDateDeath() + "},");
                                }
                                if (person.getDateEdited() != null) {
                                    p.println("dateedited = {" + person.getDateEdited() + "},");
                                }
                                p.println("isalive = {" + person.isAlive() + "},");
                                if (person.getKey() != null) {
                                    p.println("key = {" + person.getKey() + "},");
                                }
                                if (person.getPlaceBirth() != null && !person.getPlaceBirth().isEmpty()) {
                                    p.println("placebirth = {" + person.getPlaceBirth() + "},");
                                }
                                if (person.getPlaceDeath() != null && !person.getPlaceDeath().isEmpty()) {
                                    p.println("placedeath = {" + person.getPlaceDeath() + "},");
                                }
                                if (person.getPlaceCurrent() != null && !person.getPlaceCurrent().isEmpty()) {
                                    p.println("placecurrent = {" + person.getPlaceCurrent() + "},");
                                }
                                p.println("useruuid = {" + person.getUserUUID() + "},");
                                p.println("uuid = {" + person.getUUID() + "},");
                                if (person.getLanguageSkills() != null && !person.getLanguageSkills().isEmpty()) {
                                    ArrayList<String> languages = person.getLanguageSkills();
                                    p.println("languageskills = {");
                                    int iL = 0;
                                    for (String s : languages) {
                                        if (iL == languages.size()) {
                                            p.println(s);
                                        } else {
                                            p.println(s + ",");
                                        }
                                        iL++;
                                    }
                                    p.println("}");
                                }
                                if (person.getFriends() != null && !person.getFriends().isEmpty()) {
                                    ArrayList<UUID> friends = person.getFriends();
                                    p.println("friendsuuid = {");
                                    int iL = 0;
                                    for (UUID u : friends) {
                                        if (iL == friends.size()) {
                                            p.println(u);
                                        } else {
                                            p.println(u + ",");
                                        }
                                        iL++;
                                    }
                                    p.println("}");
                                }
                                if (person.getFamily() != null && !person.getFamily().isEmpty()) {
                                    ArrayList<UUID> family = person.getFamily();
                                    p.println("familyuuid = {");
                                    int iL = 0;
                                    for (UUID u : family) {
                                        if (iL == family.size()) {
                                            p.println(u);
                                        } else {
                                            p.println(u + ",");
                                        }
                                        iL++;
                                    }
                                    p.println("}");
                                }
                                p.println("}");
                            }
                        }
                    }
                }
                if (separateEntries) {
                    p.close();
                }
            }
            p.close();
        } catch (Exception e) {
            System.err.println("Error writing to file: " + e.getMessage());
            e.printStackTrace();
        }
    }
}
