/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package net.geoadel.bibcite.ImportExport;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.DatabaseObjects.Quotation;

/**
 * Exports into a html file containing css layout instructions - DOES NOT EXPORT ALL DATA
 * @author Georg Adelmann
 */
public class Export_html {

    static public File exportHtml(Bibliography bib, ArrayList<Quotation> alQuotes) {
        File f = new File(bib.getTitle() + ".html");

        String header = createHtmlHeader(bib.getTitle());
        String body = createBody(bib, alQuotes);
        String end = createEnd();

        FileWriter fw;
        try {
            fw = new FileWriter(f);
            fw.write(header + body + end);
        } catch (IOException ex) {
            Logger.getLogger(Export_html.class.getName()).log(Level.SEVERE, null, ex);
        }

        return f;
    }

    static public File exportTempHtml(Bibliography bib, ArrayList<Quotation> alQuotes) {
        File f = null;
        // System.out.println("bib title to export to temp html file: " + bib.getUUID());
        try {
            f = File.createTempFile(bib.getUUID().toString(), ".html");
            f.deleteOnExit();
            // System.out.println("Created temporary file " + f.getPath());

            String header = createHtmlHeader(bib.getTitle());
            String body = createBody(bib, alQuotes);
            String end = createEnd();
            OutputStreamWriter fw;
            try {
                fw = new OutputStreamWriter(new FileOutputStream(f), "UTF-8");
                fw.write(header + body + end);
                fw.close();
            } catch (IOException ex) {
                Logger.getLogger(Export_html.class.getName()).log(Level.SEVERE, null, ex);
            }
        } catch (IOException ex) {
            Logger.getLogger(Export_html.class.getName()).log(Level.SEVERE, null, ex);
        }

        return f;
    }

    static private String createHtmlHeader(String title) {
        String header = "";
        header = "<html>\n";
        header = header.concat("<header>\n");
        header = header.concat("<meta http-equiv=\"content-type\" content=\"text/html\"; charset=\"UTF-8\">\n");
        header = header.concat("<title>" + title + "</title>\n");

        header = header.concat("<style type=\"text/css\">\n<!--\n");
        header = header.concat(getStyle());
        header = header.concat("\n-->\n</style>\n");
        header = header.concat("</header>\n");
        return header;
    }

    static private String createBody(Bibliography bib, ArrayList<Quotation> alQuotes) {
        String body = "<body>\n";
        body = body.concat("\n  <div class=\"bibliography\">\n");
        body = body.concat("\n      <div class=\"bibtitle\"><h1>" + bib.getTitle() + "</h1></div>\n");
        if (!bib.getSubtitle().isEmpty()) {
            body = body.concat("        <div class=\"subtitle\">\n");
            body = body.concat("            " + bib.getSubtitle() + "        </div>\n");
        }
        if(!bib.getBibAbstract().isEmpty()){
            body = body.concat("        <div class=\"abstract\">\n");
            body = body.concat("            " + bib.getBibAbstract() + "        </div>\n");
        }
        body = body.concat("        <div class=\"authors\">\n");
        for (String name : bib.getAuthors()) {
            body = body.concat("            " + name + "\n");
        }
        body = body.concat("        </div>\n\n");

        Collections.sort(alQuotes);
        for (Quotation quote : alQuotes) {
            body = body.concat("\n      <div class=\"quote\">\n");

            if (!quote.getPageStart().equals("") || !quote.getPageEnd().equals("")) {
                body = body.concat("            <div class=\"pages\">");

                if (Integer.valueOf(quote.getPageEnd()) == Integer.valueOf(quote.getPageStart())) {
                    body = body.concat("                " + quote.getPageStart() + "");
                } else {
                    body = body.concat("                " + quote.getPageStart() + " - " + quote.getPageEnd());
                }
                body = body.concat("            </div>\n");
            }

            body = body.concat("            <div class=\"gist\">\n");
            String separator = System.getProperty("line.Separator");
            body = body.concat("                " + quote.getGist() + "          </div> \n");
            body = body.concat("            <div class=\"content \">\n");
            body = body.concat("            " + quote.getQuote().replaceAll("[\n\t]", "<br />") + "              </div>\n");
            body = body.concat("        "+"</div>\n");
        }
        body = body.concat("    </div> \n</body>\n");
        return body;
    }

    static private String createEnd() {
        String end = "</html>";
        return end;
    }

    static public String getStyle() {

        String styleBody = "body {background-color: #f3f3dd; color: #a52; font-size: 11px; font-family: Lucida, Verdana, Arial, SunSans-Regular, Sans-Serif;} \n";
        String styleQuote = ".quote {margin: 2em 4em 2em 4em; background-color: #f1f1dd; color: #1212cc; font-size:large;}\n";
        String stylePages = ".pages {font-size:x-large;}\n";
        String styleBibliography = ".bibliography {background-color: #ffffdd; color: #1212cc;}\n";
        String styleBibTitle = ".bibtitle {font-size:large; margin: 1em 1em 1em 1em;}\n";
        String styleGist = ".gist {font-size:large; margin: 1em;}\n";
        String styleNotes = ".notes {margin: 1em;}\n";
        String styleContent = ".content {margin: 2em; color: #000066; }\n";

        String style = styleBody + styleQuote + stylePages + styleBibliography + styleBibTitle + styleGist + styleNotes + styleContent;

        return style;
    }
}
