/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.ImportExport;

import net.geoadel.bibcite.Tools.BaseN;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.Database.db4o;
import net.geoadel.bibcite.DatabaseObjects.*;
import net.geoadel.bibcite.Tools.ObjectTools;
import net.geoadel.bibcite.Tools.StringTools;

/**
 * Imports bibTex files - tested with JabRef, one internet bib file and the exports of bibcite
 * Unknown fields are not stored or imported but just ignored - there may be dataloss, until custom fields are implemented properly and exported as well
 * 
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class Import_bibTex {

    ArrayList<String> bibKeys; // all bibliography keys, needed for avoiding duplicates / collisions when importing bibliography entries
    HashMap<String, String> hmReplacer = new HashMap<String, String>();
    ArrayList<Person> alPersons = new ArrayList<Person>();
    ArrayList<Bibliography> alBibliographies = new ArrayList<Bibliography>();
    ArrayList<Quotation> alQuotations = new ArrayList<Quotation>();
    ArrayList<Note> alNotes = new ArrayList<Note>();

    /**
     * @param file  the bib file which values should be imported
     * @param user  the user, doing the importing
     */
    public void importObjectsFromBibTex(File[] files) {
        net.geoadel.bibcite.Database.db4o db = new net.geoadel.bibcite.Database.db4o();
        bibKeys = dbModel.getObjectKeys();
        for (File file : files) {
            // System.out.println("Import - Checking " + file.getName());
            // 1. get file to String
            String fileText = "";

            // prepare a byte[] buffer for reading the file bytes into
            byte[] buffer = new byte[(int) file.length()];
            BufferedInputStream f = null;
            try {
                f = new BufferedInputStream(new FileInputStream(file));
            } catch (FileNotFoundException ex) {
            }
            try {
                f.read(buffer);// read file to bite array buffer
                fileText = new String(buffer); // transform byte-array to String
            } catch (IOException ex) {
            }
            // if length of text is longer than 1, something has been successfully taken in.
            if (fileText.length() > 1) {
                // Now try to parse bibliographies into the database
                // System.out.println("Import - Parsing " + file.getName());
                parseBibliography(fileText);
            }
        }

        // store the bibliographies and quotations
        db.storeObjects(ObjectTools.bibToDBObject(alBibliographies), SessionData.getUser());
        db.storeObjects(ObjectTools.quoteToDBObject(alQuotations), SessionData.getUser());
        db.storeObjects(ObjectTools.personToDBObject(alPersons), SessionData.getUser());
        db.storeObjects(ObjectTools.noteToDBObject(alNotes), SessionData.getUser());
    }


        /**
     * @param file  the bib file which values should be imported
     * @param user  the user, doing the importing
     */
    public void parseString(String text) {
        net.geoadel.bibcite.Database.db4o db = new net.geoadel.bibcite.Database.db4o();
        bibKeys = dbModel.getObjectKeys();

        // find database objects and put them in arrayLists
        parseBibliography(text);

        // store the bibliographies and quotations
        db.storeObjects(ObjectTools.bibToDBObject(alBibliographies), SessionData.getUser());
        db.storeObjects(ObjectTools.quoteToDBObject(alQuotations), SessionData.getUser());
        db.storeObjects(ObjectTools.personToDBObject(alPersons), SessionData.getUser());
        db.storeObjects(ObjectTools.noteToDBObject(alNotes), SessionData.getUser());
    }


    /**
     * Parses the text for bibliography and quotation fields and extracts them
     * @param text
     * @param user
     * @return
     */
    public ArrayList<Object> parseBibliography(String text) {
        ArrayList<Object> result = new ArrayList<Object>();
        // text = text.replaceAll("\t", ""); // remove tabs

        // System.out.println(text);
        Pattern patternEntries = Pattern.compile("@\\w+\\{.+?\\n*?\\}\\s*?\\n", Pattern.DOTALL); // works and ignores anything in between entries // also ignores the distinction between different @string entries, because the do not have \\n\\}\\n at the ending but just \\}\\n
        Matcher matcherEntries = patternEntries.matcher(text);
        // for each object entry found, look for its content fields
        while (matcherEntries.find()) {
            String entry = matcherEntries.group();
            // System.out.println(entry);
            // if it is a bibtex standard string entry, used for replacing abreviated strings in entries with these long forms
            if (entry.toLowerCase().contains("@string{")) {
                // System.out.println("String: " + entry);
                String field = entry;
                String fieldName = field.substring(field.indexOf("{") + 1, field.indexOf("=") - 1);
                String fieldContent = field.substring(field.indexOf("=") + 1, field.lastIndexOf("}"));
                fieldName = StringTools.cleanString(fieldName).replaceAll("[\\n\\r]", "");
                fieldContent = StringTools.cleanString(fieldContent);
                hmReplacer.put(fieldName, fieldContent);
                // System.out.println("-> "+fieldName+":"+fieldContent);
            }

            if (entry.toLowerCase().contains("@note")) {
                // System.out.println(entry);
                HashMap<String, String> hmNote = new HashMap<String, String>();

                String key = "";
                String content = "";
                String name = "";

                Long dateEdited = null;

                Boolean noteIsPrivate = false;

                UUID noteUUID = UUID.randomUUID(), noteUserUUID = UUID.randomUUID();
                ArrayList<UUID> alRelatedObjects = new ArrayList<UUID>();
                ArrayList<String> alTags = new ArrayList<String>();

                // Find all individual fields within entry
                Pattern patternField = Pattern.compile("\\w+\\s*?=\\s*?\\{*.+?\\s*?\\n*?\\},\\s*?\\n", Pattern.DOTALL);
                Matcher matcherField = patternField.matcher(entry);
                while (matcherField.find()) {
                    String field = matcherField.group();
                    // System.out.println(field);
                    String fieldName = field.substring(0, field.indexOf("=")).toLowerCase().trim();
                    String fieldContent = field.substring(field.indexOf("{") + 1, field.lastIndexOf("}"));
                    fieldName = StringTools.cleanString(fieldName).replaceAll("[\\n\\r]", "").toLowerCase();
                    fieldContent = StringTools.cleanString(fieldContent);
                    hmNote.put(fieldName, fieldContent);
                }

                if(hmNote.containsKey("notename")){
                    name = hmNote.get("notename");
                }
                if(hmNote.containsKey("notecontent")){
                    content = hmNote.get("notecontent");
                }
                if(hmNote.containsKey("notedateedited")){
                    dateEdited = Long.valueOf(hmNote.get("notedateedited"));
                }
                if(hmNote.containsKey("noteuuid")){
                    noteUUID = UUID.fromString(hmNote.get("noteuuid"));
                }
                if(hmNote.containsKey("noteisprivate")){
                    noteIsPrivate = Boolean.parseBoolean(hmNote.get("noteisprivate"));
                }
                if(hmNote.containsKey("noteuser")){
                    noteUserUUID = UUID.fromString(hmNote.get("noteuser"));
                }
                if(hmNote.containsKey("notekey")){
                    key = hmNote.get("notekey");
                }

                if (hmNote.containsKey("noterelations")) {
                    String sRelations = hmNote.get("noterelations");
                    String[] relationArray = sRelations.split(";");
                    for (String s : relationArray) {
                        s = s.replaceAll(";", "");
                        alRelatedObjects.add(UUID.fromString(s));
                    }
                }

                if (hmNote.containsKey("notetags")) {
                    String sTags = hmNote.get("notetags");
                    String[] tagArray = sTags.split("; ");
                    for (String s : tagArray) {
                        s = s.replaceAll(";", "").trim();
                        alTags.add(s);
                    }
                }

                User userImport = dbModel.getUserByUUID(noteUserUUID);
                // is user cannot be found in database, the current user is set as user
                if (userImport == null) {
                    userImport = SessionData.getUser();
                }

                Note note = new Note(noteUUID, alRelatedObjects, name, key, content, false, new ArrayList<String>(), dateEdited);

                alNotes.add(note);
            }

            if (entry.toLowerCase().contains("@quotation")) {
                // System.out.println(entry);
                HashMap<String, String> hmQuote = new HashMap<String, String>();

                String key = "";
                String content = "";
                String gist = "";
                String notes = "";

                String pageStart = "";
                String pageEnd = "";
                UUID quoteUUID = UUID.randomUUID(), quoteUserUUID = UUID.randomUUID();
                ArrayList<UUID> alRelatedObjects = new ArrayList<UUID>();
                ArrayList<String> alTags = new ArrayList<String>();


                // Find all individual fields within entry
                Pattern patternField = Pattern.compile("\\w+\\s*?=\\s*?\\{*.+?\\s*?\\n*?\\},\\s*?\\n", Pattern.DOTALL);
                Matcher matcherField = patternField.matcher(entry);
                while (matcherField.find()) {
                    String field = matcherField.group();
                    // System.out.println(field);
                    String fieldName = field.substring(0, field.indexOf("=")).toLowerCase().trim();
                    String fieldContent = field.substring(field.indexOf("{") + 1, field.lastIndexOf("}"));
                    fieldName = StringTools.cleanString(fieldName).replaceAll("[\\n\\r]", "").toLowerCase();
                    fieldContent = StringTools.cleanString(fieldContent);
                    hmQuote.put(fieldName, fieldContent);
                }

                boolean quoteIsPrivate = false;

                Long dateEdited = 0L;
                // System.out.println("quote " + hmQuote);
                if (hmQuote.containsKey("quoteuuid")) {
                    quoteUUID = UUID.fromString(hmQuote.get("quoteuuid"));
                }
                if (hmQuote.containsKey("quoteuser")) {
                    quoteUserUUID = UUID.fromString(hmQuote.get("quoteuser"));
                }
                if (hmQuote.containsKey("quoterelations")) {
                    String sRelations = hmQuote.get("quoterelations");
                    String[] relationArray = sRelations.split(";");
                    for (String s : relationArray) {
                        s = s.replaceAll(";", "");
                        alRelatedObjects.add(UUID.fromString(s));
                    }
                }

                if (hmQuote.containsKey("quotetags")) {
                    String sTags = hmQuote.get("quotetags");
                    String[] tagArray = sTags.split(";");
                    for (String s : tagArray) {
                        s = s.replaceAll(";", "");
                        alTags.add(s);
                    }
                }

                if (hmQuote.containsKey("quotecontent")) {
                    content = hmQuote.get("quotecontent");
                }
                if (hmQuote.containsKey("quotedateedited")) {
                    dateEdited = Long.parseLong(hmQuote.get("quotedateedited"));
                }
                if (hmQuote.containsKey("quotegist")) {
                    gist = hmQuote.get("quotegist");
                }
                if (hmQuote.containsKey("quotepagestart")) {
                    pageStart = hmQuote.get("quotepagestart");
                }
                if (hmQuote.containsKey("quotepageend")) {
                    pageEnd = hmQuote.get("quotepageend");
                }

                if (hmQuote.containsKey("quotekey")) {
                    key = hmQuote.get("quotekey");
                }


                User userImport = dbModel.getUserByUUID(quoteUserUUID);
                // is user cannot be found in database, the current user is set as user
                if (userImport == null) {
                    userImport = SessionData.getUser();
                }

                /**
                 * For compatibility, look for the field quotenote
                 * when importing from old backups (versions before revision 96)
                 */
                if (hmQuote.containsKey("quotenote")) {
                    if (!hmQuote.get("quotenote").isEmpty()) {
                        ArrayList<UUID> alRelated = new ArrayList<UUID>();
                        alRelated.add(quoteUUID);
                        alNotes.add(new Note(UUID.randomUUID(), alRelated, "", hmQuote.get("quotenote"), "", false, new ArrayList<String>(), dateEdited));
                    }
                }

                Quotation quote = new Quotation(quoteUUID, quoteIsPrivate, alRelatedObjects, alTags, key, content, gist, pageStart, pageEnd, dateEdited);

                alQuotations.add(quote);
            }

            // <editor-fold defaultstate="collapsed" desc="@person">
            if (entry.toLowerCase().contains("@person{")) {
                // System.out.println("Person: " + entry);

                HashMap<String, String> hmPersonFields = new HashMap<String, String>();
                // Find all individual fields within entry
                Pattern patternField = Pattern.compile("\\w+\\s*?=\\s*?[\\w\\d*\'*\"*\\{*].+?\\s*?\\n*?[,\'\"\\}]\\s*?\\n", Pattern.DOTALL);
                Matcher matcherField = patternField.matcher(entry);
                while (matcherField.find()) {
                    String field = matcherField.group();
                    String fieldName = field.substring(0, field.indexOf("=")).toLowerCase().trim();
                    String fieldContent = field.substring(field.indexOf("{") + 1, field.lastIndexOf("}"));
                    fieldName = StringTools.cleanString(fieldName).replaceAll("[\\n\\r]", "").toLowerCase();
                    fieldContent = StringTools.cleanString(fieldContent);
                    hmPersonFields.put(fieldName, fieldContent);
                }
                //  System.out.println(hmPersonFields);
                Person person = new Person();
                if (hmPersonFields.containsKey("name")) {
                    person.setName(hmPersonFields.get("name"));
                }
                if (hmPersonFields.containsKey("addressmail")) {
                    person.setAddressMail(hmPersonFields.get("addressmail"));
                }
                if (hmPersonFields.containsKey("addresspostal")) {
                    person.setAddressPostal(hmPersonFields.get("addresspostal"));
                }
                if (hmPersonFields.containsKey("description")) {
                    person.setDescription(hmPersonFields.get("description"));
                }
                if (hmPersonFields.containsKey("mainjob")) {
                    person.setMainJob(hmPersonFields.get("mainjob"));
                }
                if (hmPersonFields.containsKey("birthdate")) {
                    person.setBirthDate(Long.valueOf(hmPersonFields.get("birthdate")));
                }
                if (hmPersonFields.containsKey("deathdate")) {
                    person.setDeathDate(Long.valueOf(hmPersonFields.get("deathdate")));
                }
                if (hmPersonFields.containsKey("dateedited")) {
                    person.setDateEdited(Long.valueOf(hmPersonFields.get("dateedited")));
                }
                if (hmPersonFields.containsKey("uuid")) {
                    person.setUUID(UUID.fromString(hmPersonFields.get("uuid")));
                }
                person.setUserUUID(SessionData.getUser().getUUID());
                if (hmPersonFields.containsKey("isalive")) {
                    person.setIsAlive(Boolean.parseBoolean(hmPersonFields.get("isalive")));
                }
                if (hmPersonFields.containsKey("key")) {
                    person.setKey(hmPersonFields.get("key"));
                }
                if (hmPersonFields.containsKey("placebirth")) {
                    person.setPlaceBirth(hmPersonFields.get("placebirth"));
                }
                if (hmPersonFields.containsKey("placedeath")) {
                    person.setPlaceBirth(hmPersonFields.get("placedeath"));
                }

                if (hmPersonFields.containsKey("currentplace")) {
                    person.setCurrentPlace(hmPersonFields.get("currentplace"));
                }
                if (hmPersonFields.containsKey("familyuuid")) {
                    ArrayList<UUID> alFamily = new ArrayList<UUID>();
                    String[] arrayFamily = hmPersonFields.get("familyuuid").split(",");
                    for (String s : arrayFamily) {
                        alFamily.add(UUID.fromString(s.trim()));
                    }
                    person.setFamily(alFamily);
                }
                if (hmPersonFields.containsKey("friendsuuid")) {
                    ArrayList<UUID> alFriends = new ArrayList<UUID>();
                    String[] arrayFriends = hmPersonFields.get("friendsuuid").split(",");
                    for (String s : arrayFriends) {
                        alFriends.add(UUID.fromString(s.trim()));
                    }
                    person.setFriends(alFriends);
                }
                if (hmPersonFields.containsKey("languageskills")) {
                    ArrayList<String> alLanguages = new ArrayList<String>();
                    String[] arrayLanguages = hmPersonFields.get("languageskills").split(",");
                    for (String s : arrayLanguages) {
                        alLanguages.add(s.trim());
                    }
                    person.setLanguageSkills(alLanguages);
                }
                if(person.getName() != null & !person.getName().isEmpty()){
                    // System.out.println("Added person "+person);
                    alPersons.add(person);
                }
                
                // System.out.println("-> "+fieldName+":"+fieldContent);
            }// </editor-fold>

            parseBibliography(text, entry);
        }

        return result;
    }

    public Bibliography parseBibliography(String text, String entry){

        Bibliography bibTemp = null;

        // EXTRACT THE TYPE
            String type = "";
            String key = "";
            if (text.indexOf(entry) < 1) { // if entry begins directly at beginning of file, remove the @ from the type
                type = text.substring((text.indexOf(entry)) + 1, text.indexOf("{", text.indexOf(entry)));
                type = type.trim().toLowerCase().replaceAll("\\@", "");
            } else {
                type = text.substring((text.indexOf(entry) - 1), text.indexOf("{", text.indexOf(entry)));
                type = type.trim().toLowerCase().replaceAll("\\@", "");
            }
            // System.out.println("Entry type: " + type);
            HashMap<String, String> hmFields = new HashMap<String, String>();

            // if type of bibliography (e.g. article, book,...) is known as a bibliography entry
            if (net.geoadel.bibcite.DataFields.BibliographyTypes.getBibTypesArrayList().contains(type)) {
                // System.out.println("*****************");
                // System.out.println(entry);
                // System.out.println("Bibliography type");
                key = entry.substring(entry.indexOf("{") + 1, entry.indexOf(","));
                hmFields.put("key", key);
                // System.out.println(key);
                // if type is a bibliography entry
                // Find all individual fields within entry
                Pattern patternField = Pattern.compile("\\w+\\s*?=\\s*?[\\w\\d*\'*\"*\\{*].+?\\s*?\\n*?[,\'\"\\}]\\s*?\\n", Pattern.DOTALL);
                Matcher matcherField = patternField.matcher(entry);
                while (matcherField.find()) {

                    String field = matcherField.group();
                    // System.out.println("Field: " + field.replaceAll("[\\n\\r\\t]", ""));
                    String fieldName = field.substring(0, field.indexOf("=")).toLowerCase();
                    fieldName = StringTools.cleanString(fieldName);
                    String fieldContent = field.substring(field.indexOf("=") + 1);
                    fieldContent = fieldContent.trim();
                    // remove the fields start-denoting characters
                    if (fieldContent.startsWith("{") || fieldContent.startsWith("\"") || fieldContent.startsWith("'")) {
                        fieldContent = fieldContent.substring(1);
                        // if fieldContent starts with one {, take the ending one as well
                        // remove the fields end-denoting characters
                        if (fieldContent.endsWith("},") || fieldContent.endsWith("\",") || fieldContent.endsWith("',")) {
                            fieldContent = fieldContent.substring(0, fieldContent.length() - 2);
                        } else {
                            if (fieldContent.endsWith("}") || fieldContent.endsWith("\"") || fieldContent.endsWith("'") || fieldContent.endsWith(",")) {
                                fieldContent = fieldContent.substring(0, fieldContent.length() - 1);
                            }
                        }
                    }

                    while (fieldContent.contains("  ")) {
                        fieldContent = fieldContent.replaceAll("  ", " ");
                    }
                    if (!fieldName.contains("quote")) { // quotes are imported separately and importing them here qould give an erroneous and incomplete custom field
                        hmFields.put(fieldName, fieldContent);
                        // System.out.println("-> " + fieldName + ":" + fieldContent);
                    }
                }
                // System.out.println(hmFields);

                String address = "", bibAbstract = "", booktitle = "", crossref = "", ePrint = "", howpublished = "";
                String identifier = "", institution = "", journal = "", language = "", license = "", note = "", organization = "", pages = "", publisher = "", school = "", series = "";
                String subtitle = "", title = "", typeTechReport = "", url = "";
                boolean isPrivate = false;
                long urlDate = 0, dateEdited = 0;
                UUID uuid = UUID.randomUUID(), userUUID = UUID.randomUUID(); // if no uuid or userUUID is present in the text, this random uuid is used - otherwise the uuid is set below
                String chapter = "", edition = "", month = "", number = "", volume = "", year = "";
                ArrayList<String> authors = new ArrayList<String>();
                ArrayList<String> editors = new ArrayList<String>();
                ArrayList<String> alTags = new ArrayList<String>();

                if (hmFields.containsKey("title")) {
                    title = replaceAbbreviations(hmFields.get("title"));
                    hmFields.remove("title");
                    if (hmFields.containsKey("eprint")) {
                        // sagepub has the strange way of enclosing titles with double brackets, probably to preserve cases according to bibtex standard
                        if (hmFields.get("eprint").contains("sagepub.com")) {
                            if (title.startsWith("{")) {
                                title = title.substring(1, title.length());
                                if (title.endsWith("}")) {
                                    title = title.substring(0, title.length() - 1);
                                }
                            }
                        }
                    }
                    // remove the read in fields, so later the rest can be given as custom fields
                    // with doubling custom fields and fields accessed directly
                    // at the latest when exporting again, fields would be double!
                    // System.out.println("title: " + title);
                }

                if (hmFields.containsKey("uuid")) {
                    uuid = UUID.fromString(replaceAbbreviations(hmFields.get("uuid")));
                    hmFields.remove("uuid");
                    // System.out.println("uuid: " + uuid);
                } else {
                    uuid = UUID.randomUUID(); // if uuid does not exist, create a new one
                }

                if (hmFields.containsKey("subtitle")) {
                    subtitle = replaceAbbreviations(hmFields.get("subtitle"));
                    hmFields.remove("subtitle");
                }
                if (hmFields.containsKey("dateedited")) {
                    dateEdited = Long.valueOf(replaceAbbreviations(hmFields.get("dateedited")));
                    hmFields.remove("dateedited");
                }
                if (hmFields.containsKey("timestamp")) { // timestamp is the bibref standard
                    dateEdited = getLongDateFromTimestamp(replaceAbbreviations(hmFields.get("timestamp")));
                    hmFields.remove("timestamp");
                }
                if (hmFields.containsKey("date")) { // some call it date
                    dateEdited = getLongDateFromDate(replaceAbbreviations(hmFields.get("date")));
                    hmFields.remove("date");
                }
                if (hmFields.containsKey("abstract")) {
                    bibAbstract = replaceAbbreviations(hmFields.get("abstract"));
                    hmFields.remove("abstract");
                }
                if (hmFields.containsKey("address")) {
                    // System.out.println(hmFields.get("address"));
                    address = replaceAbbreviations(hmFields.get("address"));
                    hmFields.remove("address");
                }
                if (hmFields.containsKey("booktitle")) {
                    booktitle = replaceAbbreviations(hmFields.get("booktitle"));
                    hmFields.remove("booktitle");
                }
                if (hmFields.containsKey("crossref")) {
                    crossref = replaceAbbreviations(hmFields.get("crossref"));
                    hmFields.remove("crossref");
                }
                if (hmFields.containsKey("howpublished")) {
                    howpublished = replaceAbbreviations(hmFields.get("howpublished"));
                    hmFields.remove("howpublished");
                }
                if (hmFields.containsKey("institution")) {
                    institution = replaceAbbreviations(hmFields.get("institution"));
                    hmFields.remove("institution");
                }
                if (hmFields.containsKey("journal")) {
                    journal = replaceAbbreviations(hmFields.get("journal"));
                    hmFields.remove("journal");
                }
                if (hmFields.containsKey("note")) {
                    note = replaceAbbreviations(hmFields.get("note"));
                    hmFields.remove("note");
                }
                if (hmFields.containsKey("organization")) {
                    organization = replaceAbbreviations(hmFields.get("organization"));
                    hmFields.remove("organization");
                }
                if (hmFields.containsKey("publisher")) {
                    publisher = replaceAbbreviations(hmFields.get("publisher"));
                    hmFields.remove("publisher");
                }
                if (hmFields.containsKey("school")) {
                    school = (replaceAbbreviations(hmFields.get("school")));
                    hmFields.remove("school");
                }
                if (hmFields.containsKey("series")) {
                    series = replaceAbbreviations(hmFields.get("series"));
                    hmFields.remove("series");
                }
                if (hmFields.containsKey("chapter")) {
                    try {
                        chapter = replaceAbbreviations(hmFields.get("chapter"));
                    } catch (NumberFormatException nfe) {
                        // if chapter is not a number, store it as a custom field chapter title
                        hmFields.put("chaptertitle", replaceAbbreviations(hmFields.get("chapter")));
                    }
                    hmFields.remove("chapter");
                }
                if (hmFields.containsKey("edition")) {
                    try {
                        edition = replaceAbbreviations(hmFields.get("edition").replaceAll("\\D", ""));
                    } catch (NumberFormatException nfe) {
                        String sEdition = replaceAbbreviations(hmFields.get("edition")).toLowerCase();
                        if (sEdition.contains("first")) {
                            edition = "1";
                        }
                        if (sEdition.contains("sec")) {
                            edition = "2";
                        }
                        if (sEdition.contains("four")) {
                            edition = "4";
                        }
                        if (sEdition.contains("fi")) {
                            edition = "5";
                        }
                        if (sEdition.contains("si")) {
                            edition = "6";
                        }
                        if (sEdition.contains("se")) {
                            edition = "7";
                        }
                        if (sEdition.contains("eig")) {
                            edition = "8";
                        }
                        if (sEdition.contains("nin")) {
                            edition = "9";
                        }
                        if (sEdition.contains("ten")) {
                            edition = "10";
                        }
                    }

                    hmFields.remove("edition");
                }
                if (hmFields.containsKey("month")) {
                    try {
                        month = replaceAbbreviations(hmFields.get("month"));
                        hmFields.remove("month");
                    } catch (NumberFormatException nfe) {
                        // try to convert strings to numbers, else use month as custom field
                        String sMonth = replaceAbbreviations(hmFields.get("month"));
                        if (sMonth.toLowerCase().startsWith("jan")) {
                            month = "1";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("feb")) {
                            month = "2";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("mar")) {
                            month = "3";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("apr")) {
                            month = "4";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("may")) {
                            month = "5";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("jun")) {
                            month = "6";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("jul")) {
                            month = "7";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("aug")) {
                            month = "8";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("sep")) {
                            month = "9";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("oct")) {
                            month = "10";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("nov")) {
                            month = "11";
                            hmFields.remove("month");
                        }
                        if (sMonth.toLowerCase().startsWith("dec")) {
                            month = "12";
                            hmFields.remove("month");
                        }
                    }

                }
                if (hmFields.containsKey("year")) {
                    year = replaceAbbreviations(hmFields.get("year")).replaceAll("\\D", "");
                    hmFields.remove("year");
                }
                if (hmFields.containsKey("number")) {
                    try {
                        number = replaceAbbreviations(hmFields.get("number"));
                        hmFields.remove("number");
                    } catch (NumberFormatException nfe) {
                        // if number is not an integer, store it as customField number
                        // e.g.   number =       "ISO 9660: 1988 (E)",
                    }
                }
                if (hmFields.containsKey("pages")) {
                    pages = replaceAbbreviations(hmFields.get("pages"));
                    hmFields.remove("pages");
                }
                if (hmFields.containsKey("volume")) {
                    volume = replaceAbbreviations(hmFields.get("volume")).replaceAll("\\D", "");
                    hmFields.remove("volume");
                }
                if (hmFields.containsKey("author")) {
                    // if there was no authorkey field, take the author field as author
                    if (!hmFields.containsKey("authorkeys")) {
                        authors = StringTools.parseNamesTextArea(replaceAbbreviations(hmFields.get("author")));
                    }
                    // remove it in any case, so it does not double on export (and if an authorkeys field exists, the authors should be identical, unless edited by an external application
                    // System.out.println("Found authors: "+authors);
                    hmFields.remove("author");
                }
                if (hmFields.containsKey("authorkeys")) {
                    authors = StringTools.parseNamesTextArea(replaceAbbreviations(hmFields.get("authorkeys")));
                    hmFields.remove("authorkeys");
                }
                // System.out.println("Parsed the following authors: "+authors);
                if (hmFields.containsKey("editor")) {
                    editors = StringTools.parseNamesTextArea(replaceAbbreviations(hmFields.get("editor")));
                    hmFields.remove("editor");
                }
                if (hmFields.containsKey("isprivate")) {
                    isPrivate = Boolean.parseBoolean(replaceAbbreviations(hmFields.get("isprivate")));
                    hmFields.remove("isprivate");
                }
                if (hmFields.containsKey("key")) {
                    key = replaceAbbreviations(hmFields.get("key"));
                    hmFields.remove("key");
                } else {
                    if (entry.substring(entry.indexOf("{") + 1, entry.indexOf(",")).length() > 0) {
                        key = (entry.substring(entry.indexOf("{") + 1, entry.indexOf(",")));
                        // System.out.println("Key: " + key);
                    } else {
                        String firstAuthorLastName = authors.iterator().next();
                        firstAuthorLastName = firstAuthorLastName.substring(firstAuthorLastName.lastIndexOf(" "));
                        BaseN b = new BaseN();
                        key = (b.updateKeyValueBibliography(key, String.valueOf(year), firstAuthorLastName, bibKeys));
                    }
                }


                if (hmFields.containsKey("copyright")) { // in BibTex, copyright seems to be more standard than license
                    license = replaceAbbreviations(hmFields.get("copyright"));
                    hmFields.remove("copyright");
                }
                if (hmFields.containsKey("user")) {
                    userUUID = UUID.fromString(replaceAbbreviations(hmFields.get("user")));
                    hmFields.remove("user");
                }
                if (hmFields.containsKey("eprint")) {
                    ePrint = (replaceAbbreviations(hmFields.get("eprint")));
                    hmFields.remove("eprint");
                }
                if (hmFields.containsKey("url")) {
                    url = (replaceAbbreviations(hmFields.get("url")));
                    hmFields.remove("url");
                }
                if (hmFields.containsKey("urldate")) {
                    urlDate = (Long.valueOf(replaceAbbreviations(hmFields.get("urldate"))));
                    hmFields.remove("urldate");
                }
                if (hmFields.containsKey("language")) {
                    language = (replaceAbbreviations(hmFields.get("language")));
                    hmFields.remove("language");
                }
                if(hmFields.containsKey("identifier")){
                    identifier = hmFields.get("identifier");
                    hmFields.remove("identifier");
                }
                if(hmFields.containsKey("keywords") || hmFields.containsKey("tags")){
                    String tags = "";
                    if(hmFields.containsKey("keywords")){
                        tags = hmFields.get("keywords");
                    } else{
                        tags = hmFields.get("tags");
                    }
                    alTags.addAll(Arrays.asList(tags.split(", ")));
                }
                
                bibTemp = new Bibliography(uuid, bibAbstract, address, booktitle, crossref, howpublished, institution, journal, key, organization, publisher, school, series, title, type, chapter, edition, month, year, number, pages, volume, authors, editors, license, url, typeTechReport, ePrint, urlDate, SessionData.getUser(), subtitle, dateEdited, isPrivate, language, identifier, alTags);
                
                bibTemp.setCustomFields(hmFields);
                // System.out.println("Custom fields: " + hmFields);
                // System.out.println("Imported "+bibTemp.getFieldSummary());
                alBibliographies.add(bibTemp);

                /**
                 * The following is only there for converting from older bibcite version exports to newer ones - in old versions, notes were not independent objects but rather directly implemented in other objects as text fields
                 */
                if (!note.isEmpty()) {
                    // System.out.println("Storing note "+note);
                    ArrayList<UUID> alRelatedObjects = new ArrayList<UUID>();
                    alRelatedObjects.add(uuid);
                    // ArrayList<String> tags = ArrayList<String>();
                    alNotes.add(new Note(UUID.randomUUID(), alRelatedObjects, "", note, "", true, new ArrayList<String>(), new GregorianCalendar().getTimeInMillis()));
                }
                // System.out.println(alBibliographies.size());
                // System.out.println(alBibliographies.toString());
            }
            return bibTemp;
    }

    /**
     * Get the timestamp, since JabRef stores time differently (and does not store hours, minutes and seconds)
     * @param timestamp the timestamp (e.g. 2010.10.05)
     * @return  database dateEdited compatible format
     */
    private long getLongDateFromTimestamp(String timestamp) {
        // System.out.println(timestamp);
        timestamp = timestamp.replace(".", "");
        timestamp = timestamp + "000000";
        // System.out.println(timestamp);
        return Long.valueOf(timestamp);
    }

    private String replaceAbbreviations(String s) {
        ArrayList<String> matchingKeys = new ArrayList<String>();
        // if string contains key, add key to matching keys, to later find the one that fits best
        for (String key : hmReplacer.keySet()) {
            if (s.contains(key)) {
                // System.out.println(s + " contains: " + key + ": " + hmReplacer.get(key));
                matchingKeys.add(key);
            }
        }
        // System.out.println("These keys matched: " + matchingKeys);
        // find longest matching key and replace the match with it [longest is the one, that should be used, otherwise there would be problems with keys like ANSI, ANSI-1, ANSI-359]
        if (matchingKeys.size() > 0) {
            // System.out.println("These keys matched: " + matchingKeys);
            // find longest key
            String longestKey = "";
            for (String key : matchingKeys) {
                if (key.length() > longestKey.length()) {
                    // System.out.println(key + " is longer than " + longestKey);
                    longestKey = key;

                }
            }
            // replace match with key-value
            // System.out.println("Using " + longestKey + " - " + hmReplacer.get(longestKey));
            if (s.contains(longestKey)) {
                s = s.replaceAll(longestKey, hmReplacer.get(longestKey));
                // System.out.println("New string: " + s);
            }
        }
        return s;
    }

    private long getLongDateFromDate(String date) {
        date = date.replace("-", "");
        date = date + "000000";
        // System.out.println(timestamp);
        return Long.valueOf(date);
    }
}