/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.ImportExport;

import java.util.ArrayList;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.GUI.ProgressNotificator;
import net.geoadel.bibcite.ImportExport.DOIImport.Import_DOI;
import net.geoadel.bibcite.ImportExport.ISBNImport.Import_ISBN;

/**
 * Class distinguishes between DOIs and ISBNs and chooses the appropriate lookup class
 * @author Georg Adelmann
 */
public class Import_bibTexByISBNDOI {

    public Bibliography getBibTex(String identifier, JPanel panel) {
        Bibliography bib = null;
        ProgressNotificator.setText("Importing data for " + identifier);
        ProgressNotificator.setProgressIndeterminate(true);
        String identifierNumbers = identifier.replaceAll("[^0-9]", "");
        ArrayList<Integer> alIntValue = new ArrayList<Integer>();
        for(int i = 0; i < identifierNumbers.length(); i++){
            alIntValue.add(Integer.valueOf(identifierNumbers.charAt(i)));
        }
        if (identifier.startsWith("10.")) {
            bib = Import_DOI.importByDOI(identifier);
        } else {
            if (identifierNumbers.startsWith("978") || identifierNumbers.startsWith("979") || isISBN10(alIntValue)) {
                bib = Import_ISBN.importByISBN(identifier);
            } else {
                JOptionPane.showMessageDialog(panel, identifier + " could not be verified as either DOI or ISBN.");
            }
        }
        if (bib != null) {
            bib.setIdentifier(identifier);
            // System.out.println("Imported bib:");
            // System.out.println(bib.getFieldSummary());
        } else {
            // inform user of failed import by DOI
            JOptionPane.showMessageDialog(panel, identifier + " data were not retrievable. Please try manually dx.doi.org or www.ottobib.com.");
        }
        ProgressNotificator.setText("");
        ProgressNotificator.setProgressIndeterminate(false);

        return bib;
    }

    private Boolean isISBN10(ArrayList<Integer> numbers) {
        int a = 0;
        int b = 0;
        for(int number : numbers){
            a = a + number;
            b = b + a;
        }
        return b % 11 == 0;
    }
}
