/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.Tools;

import java.awt.Component;
import java.awt.Dimension;
import java.io.FileNotFoundException;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.geoadel.bibcite.DataFields.Preferences;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.UUID;
import javax.imageio.ImageIO;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import net.geoadel.bibcite.Cryptography.EncryptDecrypt;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.db4o;
import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.DatabaseObjects.User;
import net.geoadel.bibcite.DatabaseObjects.dbFile;
import net.geoadel.bibcite.DatabaseObjects.dbObject;
import net.geoadel.bibcite.Database.dbModel;

import net.geoadel.bibcite.FileFilters.FileFilter_bibTex;
import net.geoadel.bibcite.FileFilters.FileFilter_images;
import net.geoadel.bibcite.FileFilters.FileFilter_text;
import net.geoadel.bibcite.GUI.BibCiteGUI;
import net.geoadel.bibcite.GUI.ObjectEditors.FileCreateDialog;
import net.geoadel.bibcite.GUI.ObjectEditors.FileExternalEditingDialog;
import net.geoadel.bibcite.ImportExport.DOIImport.Import_DOI;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.util.PDFTextStripper;

/**
 * Helps moving and deleting files
 *
 * @GPLStatus   clean
 * @DocumentationStatus finished
 * @author  Georg Adelmann
 */
public class FileOperations {

    static String fileSeparator = System.getProperty("file.separator"); // the file separator (for separating file from folder) for current operating system
    static HashMap<UUID, Boolean> hmPreview = new HashMap<UUID, Boolean>();

    public static void importPDFsToBibs(File[] files, boolean singleBib) {
        if (singleBib) {
            UUID bibUUID = UUID.randomUUID();
            String parentFolder = files[0].getParent();
            parentFolder = parentFolder.substring(parentFolder.lastIndexOf(fileSeparator) + 1);
            Bibliography bib = new Bibliography(bibUUID, "", "", "", "", "", "", "", "", "", "", "", "", parentFolder, "", "", "", "", "", "", "", "", new ArrayList<String>(), new ArrayList<String>(), "", "", "", "", 0L, SessionData.getUser(), "", new dbObject().setDateEdited(), false, "", "", new ArrayList<String>());
            dbModel.storeObject(bib);
            for (File file : files) {
                // System.out.println("Importing file " + file.getName());


                UUID fileUUID = UUID.randomUUID();
                ArrayList<UUID> alRelations = new ArrayList<UUID>();
                alRelations.add(bibUUID);
                dbFile dbfile = new dbFile(file, SessionData.getUser(), alRelations, fileUUID, true, file.getName(), fileUUID.toString());
                copyFileToDataStore(dbfile);
                dbModel.storeObject(dbfile);
            }
        } else {
            for (File file : files) {
                System.out.println("Importing file " + file.getName());
                UUID bibUUID = UUID.randomUUID();
                UUID fileUUID = UUID.randomUUID();
                ArrayList<UUID> alRelations = new ArrayList<UUID>();
                alRelations.add(bibUUID);
                dbFile dbfile = new dbFile(file, SessionData.getUser(), alRelations, fileUUID, true, file.getName(), fileUUID.toString());


                try {
                    PDDocument pddDocument;
                    pddDocument = PDDocument.load(file);
                    PDFTextStripper textStripper = new PDFTextStripper();
                    // System.out.println(textStripper.getText(pddDocument));
                    dbfile.setWordList(textStripper.getText(pddDocument).replaceAll("[\\n\\r\\t]", ""));
                    pddDocument.close();
                } catch (IOException ex) {
                    Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, ex);
                }
                copyFileToDataStore(dbfile);
                dbModel.storeObject(dbfile);
                String wordList = dbfile.getWordList();
                Bibliography bib;
                if (wordList.contains("DOI 10.") || wordList.contains("doi: 10.") || wordList.contains("doi:10.")) {
                    String doi = null;
                    if (wordList.contains("DOI 10.")) {
                        doi = wordList.substring(wordList.indexOf("DOI 10.") + 4, wordList.indexOf(" ", wordList.indexOf("DOI 10.") + 5));
                    } else if (wordList.contains("doi: 10.")) {
                        doi = wordList.substring(wordList.indexOf("doi: 10.") + 5, wordList.indexOf(" ", wordList.indexOf("doi: 10.") + 6));
                    } else if (wordList.contains("doi:10.")) {
                        doi = wordList.substring(wordList.indexOf("doi:10.") + 4, wordList.indexOf(" ", wordList.indexOf("doi:10.") + 5));
                    }
                    if (doi != null) {
                        // System.out.println("DOI: "+doi);
                        String response = JOptionPane.showInputDialog("Please correct the extracted DOI for file "+dbfile.getName()+" if necessary.", doi);
                        bib = Import_DOI.importByDOI(response);
                        if (bib == null) {
                            // JOptionPane.showMessageDialog(null, response + " data were not retrievable. Please try manually dx.doi.org or www.ottobib.com.");
                            bib = new Bibliography(bibUUID, "", "", "", "", "", "", "", "", "", "", "", "", dbfile.getName(), "", "", "", "", "", "", "", "", new ArrayList<String>(), new ArrayList<String>(), "", "", "", "", 0L, SessionData.getUser(), "", new dbObject().setDateEdited(), false, "", "", new ArrayList<String>());
                            bib.setUUID(bibUUID);
                            dbModel.storeObject(bib);
                        }
                    }

                } else {
                    bib = new Bibliography(bibUUID, "", "", "", "", "", "", "", "", "", "", "", "", dbfile.getName(), "", "", "", "", "", "", "", "", new ArrayList<String>(), new ArrayList<String>(), "", "", "", "", 0L, SessionData.getUser(), "", new dbObject().setDateEdited(), false, "", "", new ArrayList<String>());
                    bib.setUUID(bibUUID);
                    dbModel.storeObject(bib);
                }
            }
        }
        
        dbModel.updateTree();

    }

    /**
     * copy file from data store to another directory
     * @param file  the file to be copied to the data directory
     * @param uuid  the uuid of the dbFile object
     * @throws IOException
     */
    public static void copyFileFromDataStore(dbFile file, File directory) {
        byte[] byteArray = getFileAsByte(file.getUUID(), file.getRevision());
        File target = null;
        EncryptDecrypt crypt = new EncryptDecrypt();
        if (file.isPrivate()) { // if file is private, decrypt the byte stream

            byteArray = crypt.decryptPrivate(byteArray, file.getUUID());
            // System.out.println("Decrypt private file "+file.getName()+", "+byteArray.length);
        } else { // else use universal program password to decrypt it
            // System.out.println("Decrypt public file "+file.getName()+", "+byteArray.length);
            byteArray = crypt.decryptPublic(byteArray);
        }
        FileOutputStream fos = null;
        try {
            // String type = file.getName().substring(file.getName().lastIndexOf(".") + 1).toUpperCase();
            target = new File(directory.getAbsolutePath() + fileSeparator + file.getName());
            while (target.exists()) {
                String newName = JOptionPane.showInputDialog("File " + target.getName() + " already exists, please choose different name:", target.getName());
                target = new File(directory.getAbsolutePath() + fileSeparator + newName);
            }
            System.out.println(target.getPath());
            // route the output stream to the target file
            fos = new FileOutputStream(target);
            // write the bytes into the target file
            fos.write(byteArray);
            fos.close();
        } catch (IOException e) {
            Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, e);
        }
    }

    /**
     * copy file to data directory
     * @param file  the file to be copied to the data directory
     * @param uuid  the uuid of the dbFile object
     * @throws IOException
     */
    public static void copyFileToDataStore(dbFile file) {
        try {
            new File(getFilePath(file.getUUID())).mkdirs(); // create the necessary directories
            FileInputStream input = new FileInputStream(file.getFile());
            FileOutputStream output = new FileOutputStream(new File(getFilePath(file.getUUID()) + file.getDateEdited()));

            byte[] b = new byte[(int) file.getFile().length()];

            try {
                input.read(b); // read the input stream into the byte array
            } catch (IOException e) {
                throw e;
            }
            if (input != null) {
                input.close();
            }
            EncryptDecrypt crypt = new EncryptDecrypt();
            if (file.isPrivate()) { // if file is private, encrypt it on byte level
                // System.out.println("Private file "+file.getName()+" was encrypted using private password.");
                b = crypt.encryptPrivate(b, file.getUUID());
            } else {
                // try to decrypt the file in case it was private before and is now set isPrivate FALSE
                try {
                    b = crypt.decryptPrivate(b, file.getUUID());
                } catch (org.jasypt.exceptions.EncryptionOperationNotPossibleException EONPE) {
                    // System.out.println("File was not encrypted?");
                }
                // then encrypt bytes using the universal program password
                b = crypt.encryptPublic(b);
            }

            if (output != null) {
                output.write(b);
                output.close();
            }
        } catch (IOException ex) {
            Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * takes three differently sized preview images and saves them to files
     * @param smallImage    a small image
     * @param mediumImage   a medium sized image
     * @param largeImage    a large image
     * @param sourceUUID    the uuid of the entry, these images should be a preview for
     * @throws IOException
     */
    public static void transformBufferedImageToPreviewImages(BufferedImage smallImage, BufferedImage mediumImage, BufferedImage largeImage, final UUID uuid, boolean isPrivate, User user) throws IOException {
        // create the necessary directories
        new File(getFilePath(uuid)).mkdirs();

        FileOutputStream outputSmall = new FileOutputStream(new File(getFilePath(uuid) + uuid.toString() + "-small"));
        FileOutputStream outputMedium = new FileOutputStream(new File(getFilePath(uuid) + uuid.toString() + "-medium"));
        FileOutputStream outputLarge = new FileOutputStream(new File(getFilePath(uuid) + uuid.toString() + "-large"));

        EncryptDecrypt crypt = new EncryptDecrypt();

        ByteArrayOutputStream byteArraySmall = new ByteArrayOutputStream(1000);
        ImageIO.write(smallImage, "png", byteArraySmall);
        byteArraySmall.flush();
        byte[] smallAsRawBytes = byteArraySmall.toByteArray();
        smallAsRawBytes = crypt.encryptPublic(smallAsRawBytes);

        ByteArrayOutputStream byteArrayMedium = new ByteArrayOutputStream(1000);
        ImageIO.write(mediumImage, "png", byteArraySmall);
        byteArrayMedium.flush();
        byte[] mediumAsRawBytes = byteArrayMedium.toByteArray();
        mediumAsRawBytes = crypt.encryptPublic(mediumAsRawBytes);

        ByteArrayOutputStream byteArrayLarge = new ByteArrayOutputStream(1000);
        ImageIO.write(largeImage, "png", byteArrayLarge);
        byteArrayLarge.flush();
        byte[] largeAsRawBytes = byteArrayLarge.toByteArray();
        largeAsRawBytes = crypt.encryptPublic(largeAsRawBytes);

        if (outputSmall != null) {
            outputSmall.write(smallAsRawBytes);
            outputSmall.close();
        }

        if (outputMedium != null) {
            outputMedium.write(mediumAsRawBytes);
            outputMedium.close();
        }

        if (outputLarge != null) {
            outputLarge.write(largeAsRawBytes);
            outputLarge.close();
        }
    }

    public static BufferedImage loadPreviewImageSmall(UUID uuid) {
        // check if there is a preview image for the object with given uuid
        if (hasPreviewSmall(uuid)) {
            byte[] b = getFileAsByte(uuid, uuid + "-small");
            EncryptDecrypt crypt = new EncryptDecrypt();
            // decrypt the byte stream using the permanent program password
            b = crypt.decryptPublic(b);
            try {
                // return the BufferedImage
                return ImageIO.read(new ByteArrayInputStream(b));
            } catch (IOException ex) {
                Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return null;
    }

    public static boolean hasPreviewSmall(UUID uuid) {
        if (hmPreview.containsKey(uuid)) {
            return hmPreview.get(uuid);
        } else {
            File fIcon = getFile(uuid, uuid + "-small");
            hmPreview.put(uuid, fIcon.exists());
            return (fIcon != null && fIcon.exists());
        }
    }

    public static File getImagePreviewMedium(UUID uuid) {
        File outMedium = new File(getFilePath(uuid) + uuid.toString() + "-medium.png");
        return outMedium;
    }

    /**
     * Erases a given file from the harddisk data path
     * @param uuid  uuid of the file
     * @param fileName  name of the file
     */
    public static void eraseFile(UUID uuid, dbFile file) {
        // A File object to represent the filename
        // System.out.println("FilesOperation.eraseFile(" + uuid + ", String " + file.getRevision() + ")");
        File f = new File(getFilePath(uuid) + fileSeparator + file.getRevision());
        // System.out.println("Path: " + f.getPath());

        // Make sure the file or directory exists and isn't write protected
        if (!f.exists()) {
            // if file does not exist, delete entry in database
        }
        if (!f.canWrite()) {
            // cannot write the file, possibly because it does not exist
        }
        // If it is a directory, make sure it is empty
        if (f.isDirectory()) {
            String[] files = f.list();
            if (files.length > 0) {
                throw new IllegalArgumentException("Delete: directory not empty: " + file.getRevision());
            }
        }

        // Attempt to delete it
        boolean success = f.delete();
        if (success) { // if file was successfully deleted, erase the direct parent directory
            f = new File(getFilePath(uuid));
            boolean delete = f.delete();
        }
    }

    /**
     * returns path of file with given fileUUID
     * @param fileUUID  uuid of file
     * @return  path of file
     */
    public static String getFilePath(UUID fileUUID) {
        // System.out.println(fileUUID);
        return (SessionData.getPathFiles() + fileUUID.toString().substring(0, 2) + fileSeparator + fileUUID.toString().substring(2, 4) + fileSeparator + fileUUID.toString() + fileSeparator);
    }

    /**
     * returns file object (used for preview pictures)
     * @param uuid  uuid of file to be returned
     * @param fileName  name of file to be returned
     * @return  file
     */
    public static File getFile(UUID uuid, dbFile file) {
        if (file != null & uuid != null) {
            return new File(getFilePath(uuid) + file.getRevision());
        }
        return null;
    }

    /**
     *
     * @param uuid  uuid of the related element (e.g. the dbFile, the bibliography [if it is a preview icon])
     * @param identifier    the filename of the file on the harddrive - for preview icons this is the uuid, for dbFile-files this is the revision date
     * @return  File
     */
    public static File getFile(UUID uuid, String identifier) {
        if (identifier != null & uuid != null) {
            return new File(getFilePath(uuid) + identifier);
        }
        return null;
    }

    /**
     *
     * @param uuid  uuid of the related element (e.g. the dbFile, the bibliography [if it is a preview icon])
     * @param identifier    the filename of the file on the harddrive - for preview icons this is the uuid, for dbFile-files this is the revision date
     * @return  byte[]
     */
    public static byte[] getFileAsByte(UUID uuid, String identifier) {
        FileInputStream input = null;
        byte[] b = null;
        try {
            File f = new File(getFilePath(uuid) + identifier);
            b = new byte[(int) f.length()];
            input = new FileInputStream(f);
            try {
                input.read(b);
            } catch (IOException ex) {
                Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, ex);
            }
        } catch (FileNotFoundException ex) {
            Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                input.close();
            } catch (IOException ex) {
                Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return b;
    }

    public static dbFile createFileFromTemplate() {
        dbFile dbfile = null;

        // 1st create dialog for asking for filename
        JDialog dialogCreateFile = new JDialog(BibCiteGUI.frame, "Create new file by template...", true);
        FileCreateDialog fCreate = new FileCreateDialog(dialogCreateFile);
        dialogCreateFile.add(fCreate);
        dialogCreateFile.setSize(new Dimension(348, 88));
        dialogCreateFile.setVisible(true);
        dialogCreateFile.setModal(true);

        // System.out.println("filename: " + fCreate.getFilename() + "    type: " + fCreate.getType());

        if (fCreate.isCreateFile()) {
            try {
                // set file uuid
                UUID fileUUID = UUID.randomUUID();
                // create file
                File file = new File(FileOperations.getFilePath(fileUUID));
                // create a new dbFile object
                dbfile = new dbFile(new File("template." + fCreate.getType()), SessionData.getUser(), null, fileUUID, false, fCreate.getFilename(), fCreate.getFilename() + "-" + new GregorianCalendar().getTime().getTime());
                // create the temporary file
                File tempFile = FileOperations.getTemplateFile(fCreate.getFilename(), fCreate.getType());
                // System.out.println("Temporary file from template: "+tempFile.getPath());

                // create the program execute string that has the temporary file as its parameter
                String type = tempFile.getName().substring(tempFile.getName().lastIndexOf(".") + 1).toUpperCase();
                String execute = SessionData.getUser().getOperatingSystemDefaultProgram(SessionData.getUser().getDefaultOperatingSystem(), type) + " " + tempFile.getPath();

                // System.out.println("Current settings "+SessionData.getUser().getProgramSettingsForType(dbfile.getType()) + " for file type "+ fCreate.getType().toUpperCase());
                System.out.println("Try to open new file " + tempFile.getPath() + " with help of " + execute);
                new ProcessBuilder(execute).start();

                // show the edit dialog with its options
                JDialog dialogEdit = new JDialog(BibCiteGUI.frame, "Edit file in external application...", true);
                FileExternalEditingDialog fEdit = new FileExternalEditingDialog(dialogEdit, true, tempFile.getName());
                dialogEdit.add(fEdit);
                dialogEdit.setSize(new Dimension(348, 300));
                dialogEdit.setVisible(true);
                dialogEdit.setModal(true);

                if (fEdit.getChosenOption().equals("New")) {
                    // create dfStore with reference to the temp file and save to database and data directory
                    dbFile dfStore = new dbFile(tempFile, tempFile.length(), SessionData.getUser(), dbfile.getUUID(), dbfile.getDateEdited(), dbfile.isPrivate(), dbfile.getRelatedObjects(), dbfile.getTags(), fCreate.getFilename() + "." + fCreate.getType(), fCreate.getFilename() + "-" + new GregorianCalendar().getTime().getTime(), dbfile.getCustomFields());
                    dbModel.storeObject(dfStore);
                    dbModel.updateTree();

                }
            } catch (IOException ex) {
                Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return dbfile;
    }

    /**
     * Is used by FileEditor to open temporary copies of files in an external program
     */
    public static File createTemporaryFile(dbFile file, User user) {
        byte[] byteArray = getFileAsByte(file.getUUID(), file.getRevision());
        File temp = null;
        EncryptDecrypt crypt = new EncryptDecrypt();
        if (file.isPrivate()) { // if file is private, decrypt the byte stream

            byteArray = crypt.decryptPrivate(byteArray, file.getUUID());
            // System.out.println("Decrypt private file "+file.getName()+", "+byteArray.length);
        } else { // else use universal program password to decrypt it
            // System.out.println("Decrypt public file "+file.getName()+", "+byteArray.length);
            byteArray = crypt.decryptPublic(byteArray);
        }
        FileOutputStream fos = null;
        try {
            // Create temp file
            String type = file.getName().substring(file.getName().lastIndexOf(".") + 1).toUpperCase();
            temp = File.createTempFile("bibcite", "." + type);
            // System.out.println("File exists "+temp.exists());
            // Delete temp file when program exits
            temp.deleteOnExit();
            // route the output stream to the temporary file
            fos = new FileOutputStream(temp);
            // write the bytes into the temporary file
            fos.write(byteArray);
            fos.close();
        } catch (IOException e) {
        }
        return temp;
    }

    public static String[] getTemplateTypes() {
        File dir = new File("./templates");

        String[] types = dir.list();
        if (types == null) {
            // Either dir does not exist or is not a directory
        } else {
            for (int i = 0; i < types.length; i++) {
                // Get filename of file or directory
                types[i] = types[i].substring(types[i].lastIndexOf("."), types[i].length());
            }
        }
        return types;
    }

    public static File getTemplateFile(String name, String type) {
        try {
            // get template file
            File template = new File("./templates/template." + type);

            // create temp file in temporary folder
            if (name.length() == 0) {
                name = "new";
            }
            File tempFile = File.createTempFile(name, "." + type);
            // System.out.println("Path of temp file: "+tempFile.getPath());
            // copy template file to temp file
            copyFile(template, tempFile);

            // return temporary file
            return tempFile;
        } catch (IOException ex) {
            Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    public static void copyFile(File fSource, File fDestination) throws IOException {
        // open file channels for source and destination
        FileChannel channelSource = new FileInputStream(fSource).getChannel();
        FileChannel channelDestination = new FileOutputStream(fDestination).getChannel();
        try {
            // copy data from source to destination
            channelSource.transferTo(0, channelSource.size(), channelDestination);
        } catch (IOException ioE) {
            throw ioE;
        } finally {
            // when finished, close the channels
            if (channelSource != null) {
                channelSource.close();
            }
            if (channelDestination != null) {
                channelDestination.close();
            }
        }
    }

    public static void exportFiles(File directory, Object[] files) {
        for (Object file : files) {
            dbFile dbf = (dbFile) file;
            copyFileFromDataStore(dbf, directory);
        }
    }

    public static dbFile importFiles(Component component, UUID sourceUUID) {
        // 1.Initialize JFileChooser and add its FileFilters
        JFileChooser fc = new JFileChooser();
        fc.addChoosableFileFilter(new FileFilter_text());
        fc.addChoosableFileFilter(new FileFilter_images());
        fc.addChoosableFileFilter(new FileFilter_bibTex());
        dbFile dbfile = null;

        // show the JFileChooser and whether a selection was made in int returnVal
        int returnVal = fc.showOpenDialog(component);
        // check, if a selection was approved
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            // get the selected file and instantiate a new File from it
            File file = fc.getSelectedFile();
            // set the fileUUID (since it is a new file, generate a random new UUID)
            // create a dbFile to be stored in the database
            dbfile = new dbFile(file, SessionData.getUser(), new ArrayList<UUID>(), UUID.randomUUID(), false, file.getName(), file.getName() + "-" + new GregorianCalendar().getTime().getTime());
            // if sourceUUID was given, add it as relation
            if (sourceUUID != null) {
                dbfile.addRelatedObject(sourceUUID);
            }

            // if it is a pdf, get its wordlist, so that its text is found by search
            String type = dbfile.getName().substring(dbfile.getName().lastIndexOf(".") + 1).toUpperCase();
            if (type.toUpperCase().endsWith("PDF")) {
                // System.out.println("Imported pdf file..");
                try {
                    PDDocument pddDocument;
                    pddDocument = PDDocument.load(file);
                    PDFTextStripper textStripper = new PDFTextStripper();
                    // System.out.println(textStripper.getText(pddDocument));
                    dbfile.setWordList(textStripper.getText(pddDocument).replaceAll("[\\n\\r\\t]", ""));
                    pddDocument.close();
                } catch (IOException ex) {
                    Logger.getLogger(FileOperations.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

        if (dbfile != null) {
            dbModel.storeObject(dbfile);
            dbModel.updateTree();
            FileOperations.copyFileToDataStore(dbfile);
        }

        return dbfile;
    }
}
