/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package net.geoadel.bibcite.Tools;

import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;

/**
 * Provides several image operations
 * @DocumentationStatus finished
 * @GPLStatus   clean
 * @author Georg Adelmann
 */
public class ImageOperations {

    // standard sizes for preview images maximum dimensions
    static int largeMax = 800;
    static int mediumMax = 320;
    static int smallMax = 32;

    /**
     * Transforms BufferedImage to ImageIcon, trivial task but in case I forgot how to do it, here it is
     * @param bImage    BufferedImage
     * @return  ImageIcon
     */
    public static ImageIcon getImageIcon(BufferedImage bImage) {
        return new ImageIcon(bImage);
    }

    /**
     * Get small BufferedImage (with same image aspect ratio) from BufferedImage
     * @param original  original BufferedImage
     * @return  scaled buffered Image
     */
    public static BufferedImage getIconSmall(BufferedImage original) {
        // scale image for icon
        int iconWidth = 1;
        int iconHeight = 1;
        if (original.getWidth() > original.getHeight()) {
            iconWidth = smallMax;
            float newHeight = ((float) iconWidth / (float) original.getWidth()) * (float) original.getHeight();
            iconHeight = Math.round(newHeight);
        } else {
            iconHeight = smallMax;
            float newWidth = ((float) iconHeight / (float) original.getHeight()) * (float) original.getWidth();
            iconWidth = Math.round(newWidth);
        }
        BufferedImage resizedImage = new BufferedImage(iconWidth, iconHeight, BufferedImage.TYPE_4BYTE_ABGR);
        Graphics2D g = resizedImage.createGraphics();
        g.drawImage(original, 0, 0, iconWidth, iconHeight, null);
        g.dispose();
        // set rendering hints for smooth scaling output
        g.setComposite(AlphaComposite.Src);
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        return resizedImage;
    }

    /**
     * Get medium BufferedImage (with same image aspect ratio) from BufferedImage
     * @param original  original BufferedImage
     * @return  scaled buffered Image
     */
    public static BufferedImage getIconMedium(BufferedImage original) {
        // scale image for icon
        int iconWidth = 1;
        int iconHeight = 1;
        if (original.getWidth() > original.getHeight()) {
            iconWidth = mediumMax;
            float newHeight = ((float) iconWidth / (float) original.getWidth()) * (float) original.getHeight();
            iconHeight = Math.round(newHeight);
        } else {
            iconHeight = mediumMax;
            float newWidth = ((float) iconHeight / (float) original.getHeight()) * (float) original.getWidth();
            iconWidth = Math.round(newWidth);
        }
        BufferedImage resizedImage = new BufferedImage(iconWidth, iconHeight, BufferedImage.TYPE_4BYTE_ABGR);
        Graphics2D g = resizedImage.createGraphics();
        g.drawImage(original, 0, 0, iconWidth, iconHeight, null);
        g.dispose();
        g.setComposite(AlphaComposite.Src);
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        return resizedImage;
    }

    /**
     * Get large BufferedImage (with same image aspect ratio) from BufferedImage
     * @param original  original BufferedImage
     * @return  scaled buffered Image
     */
    public static BufferedImage getIconLarge(BufferedImage original) {
        // scale image for icon
        int iconWidth = 1;
        int iconHeight = 1;
        if (original.getWidth() > original.getHeight()) {
            iconWidth = largeMax;
            float newHeight = ((float) iconWidth / (float) original.getWidth()) * (float) original.getHeight();
            iconHeight = Math.round(newHeight);
        } else {
            iconHeight = largeMax;
            float newWidth = ((float) iconHeight / (float) original.getHeight()) * (float) original.getWidth();
            iconWidth = Math.round(newWidth);
        }
        BufferedImage resizedImage = new BufferedImage(iconWidth, iconHeight, BufferedImage.TYPE_4BYTE_ABGR);
        Graphics2D g = resizedImage.createGraphics();
        g.drawImage(original, 0, 0, iconWidth, iconHeight, null);
        g.dispose();
        // set rendering hints
        g.setComposite(AlphaComposite.Src);
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        return resizedImage;
    }
}