/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.Tools;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.db4o;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.DatabaseObjects.dbFile;
import net.geoadel.bibcite.GUI.ProgressNotificator;

/**
 * InternetTools provides methods for accessing data in the internet
 * @author Georg Adelmann
 */
public class InternetTools {

    static String newline = System.getProperty("line.separator");

    /**
     *
     * @param link  link to html page, that should be retrieved as string
     * @return      html source code for link as string
     */
    public static String getHTML(String link) {
        System.out.println("getHTML(" + link + ")");
        String htmlsource = "";
        try {
            URL url = new URL(link.trim());

            HttpURLConnection httpcon = (HttpURLConnection) url.openConnection();
            httpcon.addRequestProperty("User-Agent", "Firefox/4.0.3");
            try {
                BufferedReader in = new BufferedReader(new InputStreamReader(httpcon.getInputStream()));

                int iLines = 1;
                String line = "";
                while ((line = in.readLine()) != null) {
                    if (line != null) {
                        // System.out.println(in.readLine());
                        try {
                            htmlsource = htmlsource.concat(line);
                        } catch (NullPointerException npe) {
                        }
                        htmlsource = htmlsource.concat(newline);
                    }
                    iLines++;
                }
            } catch (FileNotFoundException fnfe) {
                Logger.getLogger(InternetTools.class.getName()).log(Level.OFF, null, fnfe);
            }

        } catch (IOException ex) {
            Logger.getLogger(InternetTools.class.getName()).log(Level.SEVERE, null, ex);
        }

        htmlsource = htmlsource.replaceAll("&gt;", ">");
        htmlsource = htmlsource.replaceAll("&lt;", "<");

        return htmlsource;
    }

    public static dbFile getFile(String name, String link, UUID sourceUUID) {
        {
            FileOutputStream output = null;
            try {
                // System.out.println("Trying to download "+link);
                URL url = new URL(link);
                dbFile dbfile = new dbFile(sourceUUID);
                File tempFile = File.createTempFile("temp_download", "pdf");
                output = new FileOutputStream(tempFile);
                // URLConnection httpcon = url.openConnection();

                HttpURLConnection.setFollowRedirects(true);

                HttpURLConnection con = (HttpURLConnection) url.openConnection();
                // System.out.println(con.getResponseMessage());

                if (con.getContentType().equalsIgnoreCase("application/pdf")) {
                    // System.out.println("File is pdf.");
                    InputStream inputStream = con.getInputStream();
                    int i = inputStream.read();
                    while (i != -1) {
                        output.write(i);
                        i = inputStream.read();
                    }
                    // System.out.println("Name of file "+tempFile.getName());
                    // System.out.println("Assigned source UUID "+sourceUUID);
                    // System.out.println("Random uuid "+dbfile.getUUID());

                    ArrayList<UUID> alRelations = new ArrayList<UUID>();
                    alRelations.add(sourceUUID);
                    dbfile = new dbFile(tempFile, SessionData.getUser(), alRelations, dbfile.getUUID(), true, name + ".pdf", dbfile.getUUID() + ".pdf");
                    dbModel.storeObject(dbfile);
                    inputStream.close();
                    output.close();
                    return dbfile;
                }
            } catch (IOException ex) {
                Logger.getLogger(InternetTools.class.getName()).log(Level.SEVERE, null, ex);
            } finally {
                try {
                    output.close();
                } catch (IOException ex) {
                    Logger.getLogger(InternetTools.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

        return null;
    }
}
