/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package net.geoadel.bibcite.Tools;

import java.util.ArrayList;
import java.util.UUID;
import net.geoadel.bibcite.DatabaseObjects.Bibliography;
import net.geoadel.bibcite.DatabaseObjects.Note;
import net.geoadel.bibcite.DatabaseObjects.Person;
import net.geoadel.bibcite.DatabaseObjects.Quotation;
import net.geoadel.bibcite.DatabaseObjects.User;
import net.geoadel.bibcite.DatabaseObjects.dbFile;
import net.geoadel.bibcite.DatabaseObjects.dbObject;

/**
 * A helper class for handling database objects
 *
 * @author Georg Adelmann
 */
public class ObjectTools {

    /**
     * filters all dbObjects for their distinct uuid-values and returns them
     * @param objects
     * @return  ArrayList<UUID> containing all distinct uuids
     */
    public static ArrayList<UUID> filterFamiliesByUUID(ArrayList<dbObject> objects) {
        ArrayList<UUID> alFamilies = new ArrayList<UUID>();
        for (int i = 0; i < objects.size(); i++) {
            // System.out.println(i + ": " + objects.get(i).getDateEdited()+" uuid: "+objects.get(i).getUUID());
            if (!alFamilies.contains(objects.get(i).getUUID())) {
                alFamilies.add(objects.get(i).getUUID());
            }
        }
        // System.out.println("filterFamiliesByUUID() found " + alFamilies.size() + " uuid families.");
        return alFamilies;
    }

    public static ArrayList<dbObject> bibToDBObject(ArrayList<Bibliography> bibs) {
        ArrayList<dbObject> objects = new ArrayList<dbObject>();
        for (Bibliography bib : bibs) {
            objects.add((dbObject) bib);
        }
        return objects;
    }

    public static ArrayList<dbObject> quoteToDBObject(ArrayList<Quotation> quotes) {
        ArrayList<dbObject> objects = new ArrayList<dbObject>();
        for (Quotation quote : quotes) {
            objects.add((dbObject) quote);
        }
        return objects;
    }

    public static ArrayList<dbObject> noteToDBObject(ArrayList<Note> notes) {
        ArrayList<dbObject> objects = new ArrayList<dbObject>();
        for (Note note : notes) {
            objects.add((dbObject) note);
        }
        return objects;
    }

    public static ArrayList<dbObject> personToDBObject(ArrayList<Person> persons) {
        ArrayList<dbObject> objects = new ArrayList<dbObject>();
        for (Person person : persons) {
            objects.add((dbObject) person);
        }
        return objects;
    }

    public static int filterNewest(ArrayList<dbObject> objects){
        long newest = Long.MIN_VALUE;
        int iNewestIndex = -1;
        for (int i = 0; i < objects.size(); i++) {
            // System.out.println(i + " of "+objects.size());
            if (objects.get(i).getDateEdited() > newest) {
                newest = objects.get(i).getDateEdited();
                iNewestIndex = i;
            }
        }
        /**
         * If iNewestIndex is -1, then none was younger than Long.MIN_VALUE, in that case, return the first object index 0
         */
        if(iNewestIndex == -1){
            iNewestIndex = 0;
        }
        return iNewestIndex;
    }

    /**
     * 
     * @param list ArrayList of objects with one or many different uuid families
     * @return newest objects per uuid family
     */
    public static ArrayList<dbObject> filterNewestByUUID(ArrayList<dbObject> list){
        // find the uuid families in the object arraylist
        ArrayList<UUID> familyList = filterFamiliesByUUID(list);
        ArrayList<dbObject> onlyNewestByUUID = new ArrayList<dbObject>();
        
        // look through all objects per uuid
        for(UUID uuid : familyList){
            ArrayList<dbObject> currentFamily = new ArrayList<dbObject>();
            for(dbObject object : list){
                if(object.getUUID() != null & uuid!=null && object.getUUID().equals(uuid)){
                    // add object to currentFamily, if it has the currently looked for  uuid
                    currentFamily.add(object);
                }
            }
            // add newest family member to onlyNewestByUUID
            int iNewest = filterNewest(currentFamily);
            if(iNewest != -1 && currentFamily.size() > iNewest){
                onlyNewestByUUID.add(currentFamily.get(iNewest));
            }
            if(iNewest == -1){
                onlyNewestByUUID.add(currentFamily.get(0));
            }
        }
        return onlyNewestByUUID;
    }

    public static ArrayList<dbObject> fileToDBObject(ArrayList<dbFile> files) {
        ArrayList<dbObject> objects = new ArrayList<dbObject>();
        for (dbFile file : files) {
            objects.add((dbObject) file);
        }
        return objects;
    }

    public static ArrayList<dbObject> userToDBObject(ArrayList<User> users) {
        ArrayList<dbObject> objects = new ArrayList<dbObject>();
        for (User user : users) {
            objects.add((dbObject) user);
        }
        return objects;
    }

    public static ArrayList<Person> dbObjectToPerson(ArrayList<dbObject> objects){
        ArrayList<Person> persons = new ArrayList<Person>();
        for (dbObject object : objects) {
            persons.add((Person) object);
        }
        return persons;
    }

    public static ArrayList<Bibliography> dbObjectToBib(ArrayList<dbObject> objects) {
        ArrayList<Bibliography> alReturn = new ArrayList<Bibliography>();
        for(dbObject object : objects){
            if(object instanceof Bibliography){
                alReturn.add((Bibliography)object);
            }
        }
        return alReturn;
    }

    public static ArrayList<Quotation> dbObjectToQuote(ArrayList<dbObject> objects) {
        ArrayList<Quotation> alReturn = new ArrayList<Quotation>();
        for(dbObject object : objects){
            if(object instanceof Quotation){
                alReturn.add((Quotation)object);
            }
        }
        return alReturn;
    }
    
        public static ArrayList<Note> dbObjectToNote(ArrayList<dbObject> objects) {
        ArrayList<Note> alReturn = new ArrayList<Note>();
        for(dbObject object : objects){
            if(object instanceof Note){
                alReturn.add((Note)object);
            }
        }
        return alReturn;
    }
}
